/**
 * @(#) DataConnectionType; class to express DataConnectionType.
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * <japanese>
 * f[^Ԍ^\NXłB<p>
 *
 * ^ɈSȒ萔̃NXłB<p>
 *
 * <ul>
 *  <li> BROKENLINE
 *    <dd> ܂Ńf[^Ԃ܂Bgp̂łʓIł傤B
 *    <dd> f[^Ԍ^Ɏw肵Ȃꍇ͂ BROKENLINE ɐݒ肳܂B
 *  <li> STEP
 *    <dd> Xebvf[^łBA鎞ɂf[^ĺA̎܂ňlƂ܂B
 *  <li> PULSE
 *    <dd> pXf[^łBA鎞Ǝ̎Ƃ̊Ԃ̒l 0 Ƃ܂B
 *  <li> NOINTP
 *    <dd> ԂłȂƂ\܂B
 * </ul>
 *
 * </japanese>
 * <p>
 * DataConnectionType;
 *
 * Class to express DataConnectionType. <p>
 *
 * The following data connection types are defined.
 *
 * <pre>
 *  :BROKENLINE
 *    The data are connected with broken line.
 *  :STEP
 *    The value is taken constant and equal to the current data value
 *    from the previous data time till the current time.
 *  :PULSE
 *    The value between data times is taken to be zero.
 *  :NOINTP
 *    No interpolation.
 * </pre>
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.20 $ $Date: 2004/02/12 05:35:33 $
 */
public final class DataConnectionType implements Serializable
{
  // @begin {instance variable or static variable} -----------------------

  private static int nextOrdinal = 0;
  private final  int ordinal     = nextOrdinal++;

  /**
   * <japanese>
   * ܂\^ɈSȒ萔łB
   * </japanese>
   * <p>
   * BROKENLINE : The data are connected with broken line.
   *
   */
  public static final DataConnectionType BROKENLINE = new DataConnectionType();

  /**
   * <japanese>
   * Xebvf[^\^ɈSȒ萔łB
   * </japanese>
   * <p>
   * STEP: 
   * The value is taken constant and equal to the current data value
   * from the previous data time till the current time.
   *
   */
  public static final DataConnectionType STEP       = new DataConnectionType();

  /**
   * <japanese>
   * pXf[^\^ɈSȒ萔łB
   * </japanese>
   * <p>
   * PULSE : The value between data times is taken to be zero.
   *
   */
  public static final DataConnectionType PULSE      = new DataConnectionType();

  /**
   * <japanese>
   * ԂłȂƂ\^ɈSȒ萔łB
   * </japanese>
   * <p>
   * NOINTP : No interpolation.
   *
   */
  public static final DataConnectionType NOINTP     = new DataConnectionType();

  private static final DataConnectionType[] PRIVATE_VALUES = {BROKENLINE, STEP, PULSE, NOINTP};

  private static final String DCTStr[] = {"BROKENLINE", "STEP", "PULSE", "NOINTP"};
  
  // @end {instance variable or static variable} -------------------------
  
  
  // @begin {constructors} -----------------------------------------------
  
  private DataConnectionType() {}

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  /**
   * <japanese>
   * ^ɈSȒ萔̌ł 4 Ԃ܂B
   * </japanese>
   *
   * @return an <code>int</code> value
   */
  public static int getNumOfDCT () {
    return DCTStr.length;
  }


  /**
   * <japanese>
   * ̒lɂĈȉ̂悤 DataConnectionType ^Ԃ܂B
   *
   * <ul>
   *  <li> 1 : BROKENLINE
   *  <li> 2 : STEP
   *  <li> 3 : PULSE
   *  <li> 4 : NOINTP
   * </ul>
   *
   * ȊO int l̏ꍇ́A
   * O ArrayIndexOutOfBoundsException ܂B
   * </japanese>
   *
   * @param k an <code>int</code> value
   * @return a <code>DataConnectionType</code> value
   */
  public static DataConnectionType getDctType(int k) {
    return PRIVATE_VALUES[k];
  }


  /**
   * <japanese>
   * ̒lɂĈȉ̂悤Ƀf[^Ԍ^\Ԃ܂B
   *
   * <ul>
   *  <li> 1 : BROKENLINE
   *  <li> 2 : STEP
   *  <li> 3 : PULSE
   *  <li> 4 : NOINTP
   * </ul>
   *
   * ȊO int l̏ꍇ́A
   * O ArrayIndexOutOfBoundsException ܂B
   * </japanese>
   *
   * @param k an <code>int</code> value
   * @return a <code>String</code> value
   */
  public static String getDctTypeStr(int k) {
    return DCTStr[k];
  }


  private Object readResolve() throws ObjectStreamException {
    return PRIVATE_VALUES[ordinal];
  }

  // @end {methods} ------------------------------------------------------

} // DataConnectionType
