/**
 * OhBufferedReader.java
 *
 * Created: DATE
 *
 * @author <a href="mailto:shiiba@toewin.kuciv.kyoto-u.ac.jp">shiiba</a>
 * @version $Revision: 1.5 $
 *
 * @(#)
 */

package ohymosj;

import ohymosj.*;
import java.io.*;

/**
 * <japanese>
 * NX BufferedReader ɒP̓ǂݎ@\ǉNXłB<p>
 *
 * BufferedReader ́As̓ǂݎ͂ł܂AP̓ǂݎ肪
 * ł܂B OhBufferedReader ́A
 *
 * <ol>
 * <li> readWord() ɂāA󔒕(p󔒁A^uAs)
 *      ؂蕶ƌȂāAP؂oƂł܂B
 * <li> Rgǂݔ΂悤Ɏwł܂Bw肷΁As
 *      # s͖܂Bs̓rɁA# ƁA
 *      s܂Ŗ܂B
 * </ol>
 *
 *  OhBufferedReader ́Af[^ǂݎ邽߂ɁAXg[
 * 1s擾AɕێĂ܂B̕
 * 𑖍āAP؂o܂B
 *
 * RXgN^ȊOɒ`Ă֐́A
 *
 * getTreatCommentFlag(), pushBack(), read(), readLine(), readWord(),
 * skipWhites(), treatComment(boolean aMode) 
 *
 * łBȊOɐeNXp֐Ŏgpėǂ̂́A
 * close() łB
 *
 * </japanese>
 * 
 * @version $Revision: 1.5 $ $Date: 2004/02/10 14:19:13 $
 */
public class OhBufferedReader extends BufferedReader
{
  protected String _nextLine;
  protected int _ipos;
  protected boolean _treatCommentP = false;
  protected int _lineno = 0; // line number
  protected static String NEWLINE = System.getProperty("line.separator");

  /**
   * Creates a new <code>OhBufferedReader</code> instance.
   *
   */
  public OhBufferedReader (Reader aIn) {
    super(aIn);
  }

  public OhBufferedReader (Reader aIn, boolean aTreatCommentP)
    {
      super(aIn);
      _treatCommentP = aTreatCommentP;
    }

  /**
   * Creates a new <code>OhBufferedReader</code> instance.
   *
   */
  public OhBufferedReader (Reader aIn, int aSz) {
    super(aIn, aSz);
  }

  public OhBufferedReader (Reader aIn, int aSz,
    boolean aTreatCommentP)
    {
      super(aIn, aSz);
      _treatCommentP = aTreatCommentP;
    }
  
  /**
   * Sets _nextLine
   *
   */
  private void setNextLine() throws IOException {
    try {
      _nextLine = super.readLine();
      _lineno++;
      _ipos = 0;
      while (_nextLine != null &&
        _nextLine.length() > 0 &&
        _nextLine.charAt(0) == '#' &&
        _treatCommentP) {
        _nextLine = super.readLine();
        _lineno++;
      }
      if (_nextLine != null && _nextLine.length() > 0) {
        boolean incomment = false;
        char qMark = '\"'; //; "
        int i = _ipos;
        for (;i < _nextLine.length(); i++) {
          char c = _nextLine.charAt(i);
          if (incomment) {
            if (c == qMark) { incomment = false; }
            continue;
          }
          else {
            if (c == '\"' || c == '\'') {
              incomment = true;
              qMark = c;
              continue;
            }
            else if (_treatCommentP && c == '#') {
              _nextLine = _nextLine.substring(0, i);
              break;
            }
          }
        } // end of for loop
        if (incomment) {
          String message =
            "Input data error. Unballanced quotation marks: "
            + NEWLINE + _lineno + ": " + _nextLine;
          // System.err.println(message);
          throw new IOException(message);
        }
      } // if (_nextLine != null && _nextLine.length() > 0)
    } // try
    catch (IOException e) {
      // e.printStackTrace();
      // System.err.println("IO error");
      throw e;
    }
  }

  /**
   * Push back the last character which had been read in to this
   * Reader. Only the character which bad been read in from the
   * current line can be pushed back.
   *
   * <japanese>
   * <br>
   *  Reader ɍŌɓǂݍ܂ꂽǂ܂ȂƂɂ
   * āAɖ߂܂BA߂Ƃł̂́Aݓǂݍ
   * ݂̑ΏۂɂȂĂsǂ܂ꂽłB
   * </japanese>
   */
  public void pushBack() {
    if (_lineno == 0) return;
      //; ܂1sǂłȂ̂ŁApushBack łȂ͂B
    if (_ipos > 0) _ipos--;
  }
  
  /**
   * Sets treat-comment-mode on.
   *
   * When treat-comment-mode is on,
   *
   * <ol>
   * <li> lines which start with '#' are ignored;
   * <li> part string from '#' to the line end is ignored.
   * </ol>
   *
   * In default, treat-comment-mode is off.
   *
   * @return a <code>boolean</code> value
   */
  public void setTreatCommentP(boolean aTreatCommentP) {
    _treatCommentP = aTreatCommentP;
  }

  /**
   * Returns the boolean flag about treat-comment-mode. If this
   * flag is true, treat-comment-mode is on.
   *
   * @return a <code>boolean</code> value
   */
  public boolean getTreatCommentP() {
    return _treatCommentP;
  }

  /**
   * Read one character. When eof was deteced, return -1.
   * When the reading pointer is at the line end, return '\n'.
   *
   * @return an <code>int</code> value
   * @exception IOException if an error occurs
   */
  public int read() throws IOException {
    if (_lineno == 0) setNextLine();
    if (_nextLine == null) return -1;
    if (_nextLine.length() == 0) {
      setNextLine();
      return '\n';
    }
    if (_ipos >= _nextLine.length()) {
      setNextLine();
      return '\n';
    }
    else {
      int k = (char)_nextLine.charAt(_ipos);
      _ipos++;
      return k;
    }
  }

  /**
   * Reads a line of text.
   *
   * <japanese>
   * <br>
   * ݂̓ǂݎʒus܂œǂݎĕԂ܂B<p>
   *
   * ݂̓ǂ݂Ƃʒut@CI[ɗĂƂ́A
   * readLine() ́Anull Ԃ܂B<p>
   *
   * ݂̓ǂݎʒusɂƂ́A󂫕Ԃ܂B
   * <p>
   *
   * ̓ǂݎ|CǵÃ\bhœǎŝ
   * ̍s̍sɂȂ܂B<p>
   * </japanese>
   *
   * @return a <code>String</code> value
   * @exception IOException if an error occurs
   */
  public String readLine() throws IOException {
    if (_lineno == 0) setNextLine();
    String lastLine = _nextLine;
    String cline = lastLine;
    if (lastLine != null) {
      cline = lastLine.substring(_ipos, lastLine.length());
    }
    if (lastLine != null) {
      setNextLine();
    }
    return cline;
  }

  /**
   * Read a word.
   *
   * <japanese>
   * <br>
   * Pǂ݂Ƃ܂BǂݎʒusȂA̍sP
   * ǂ݂܂BPǂݎ낤ƂƂɁAs (sɂ
   * ss) ɑA󂫕Ԃ܂B<p>
   *
   * PƂ́A
   *
   * <ol>
   * <li> pȊO̔󔒕̘A
   * <li> p (" ܂ ') p܂ł̕
   * </ol>
   *
   * ܂B
   * </japanese>
   *
   * @return a <code>String</code> value
   * @exception IOException if an error occurs
   */
  public String readWord() throws IOException {
    if (_lineno == 0) setNextLine();
    if (_nextLine == null) return null;
    int i = 0;
    char c = 'a';
    while (true) { // search a word
      if (_nextLine.length() == 0) {
        setNextLine();
        return "";
      }
      if (_ipos == _nextLine.length()) {
        // blank line or now at the line end
        setNextLine();
        if (_nextLine == null) return null;
        continue;
      }
      for (i = _ipos; i < _nextLine.length(); i++) {
        c = _nextLine.charAt(i);
        if (c != ' ' && c != '\t') break;
      }
      if (i < _nextLine.length()) {
        _ipos = i;
        break; //; find a word
      }
      setNextLine();
    }

    boolean quotedStringP = false; char qMark = '\"';
    if (c == '\"' || c == '\'') {
      quotedStringP = true;
      qMark = c;
      // System.out.println("_ipos = " + _ipos);
    }
      
    for (i = _ipos+1; i < _nextLine.length(); i++) {
      c = _nextLine.charAt(i);
      if (quotedStringP) {
        if (c == qMark) {i++; break;}
      }
      else {
        if (c == ' ' || c == '\t' ||
            c == '\"' || c == '\'') break;
      }
    }
    String word = _nextLine.substring(_ipos, i);
    _ipos = i;
    // System.out.println("_nextLine: " + _nextLine);
    // System.out.println("word:      " + word);
    return word;
  }

  /**
   * Skip white letters. The line end mark is also skipped.
   *
   * <japanese>
   * <br>
   * 󔒕 (p󔒁A^uAs) ǂݔ΂A
   * ̏ɓǂݎ|Cgړ܂B
   *
   * 󔒕ǂݔ΂ĂԂɁAt@C̏I[ɒBƂ
   * ́A-1 Ԃ܂BȊÔƂ́Aǂݎ|Cg̕
   * ̃R[hԂ܂B
   * </japanese>
   */
  public int skipWhites() throws IOException {
    if (_lineno == 0) setNextLine();
    if (_nextLine == null) { return -1; }
    int i = 0;
    char c = 'c';
    while (true) { // search a word
      if (_nextLine.length() == 0 || _ipos == _nextLine.length()) {
        // blank line or now at the line end
        setNextLine();
        if (_nextLine == null) { return -1; }
        continue;
      }
      for (i = _ipos; i < _nextLine.length(); i++) {
        c = _nextLine.charAt(i);
        if (c != ' ' && c != '\t') break;
      }
      if (i < _nextLine.length()) {
        _ipos = i;
        break; //; find a word
      }
      setNextLine();
    }
    return c;
  }

  public long skip(long n) throws IOException {
    IOException e = new IOException();
    throw e;
  }

  public int read(char[] cbuf, int off, int len) throws IOException {
    IOException e = new IOException();
    throw e;
  }

  public void mark(int a) throws IOException {
    IOException e = new IOException();
    throw e;
  }

  public void reset() throws IOException {
    IOException e = new IOException();
    throw e;
  }

}// OhBufferedReader
