/**
 * @(#) OhCannotGetDataError; ohymosj error exception class.
 *                            Make "OhCannotGetDataError.java" from "oherror.h" and "oherror.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * OHyMoSJ ̗ONXłB
 * try ubNŁA
 * ̗ONX̃IuWFNg쐬 throw ܂B
 * </japanese>
 * <p>
 * OhCannotGetDataError;
 *
 * OHyMoSJ error exception class.
 * Error is a derived class of the Exception class. <p>
 *
 * The OhCannotGetDataError class object constructor is used as
 *
 * <pre>
 * .....
 *   try {
 *     ...
 *     OhCannotGetDataError e = new OhCannotGetDataError(errorMessage1, errorMessage2);
 *     throw e;
 *   }
 *   catch (OhCannotGetDataError e) {
 *     System.out.print(e.what());
 *   }
 * .....
 * </pre>
 *
 * The example of the creation method of an error message is as follows.
 *
 * <pre>
 * .....
 *   if(some error status was detected) {
 *     StringWriter buffer = new StringWriter();
 *     PrintWriter  out    = new PrintWriter(buffer);
 *     oPrint(out, " > ");
 *     String str = buffer.toString();
 *     out.close();
 *     throw new OhCannotGetDataError("Already exits \"_dataPackArray\"." + str);
 *   }
 * .....
 * </pre>
 *   
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.12 $ $Date: 2004/02/10 15:20:45 $
 */
public class OhCannotGetDataError extends Exception
{
  // @begin{field} -------------------------------------------------------

  /**
   * <japanese>
   * G[bZ[WłB
   * </japanese>
   * <p>
   * Error message.
   *
   */
  protected String _errorMessage;

  // @end{field} ---------------------------------------------------------

  
  // @begin{constructor} -------------------------------------------------

  /**
   * <japanese>
   * G[bZ[W aStr RXgN^łB
   * </japanese>
   *
   * @param aStr a <code>String</code> value
   */
  public OhCannotGetDataError(String aStr) {
    _errorMessage = aStr;
  }
  
  /**
   * <japanese>
   * 1sڂ aStr1A2sڂ aStr2 ̃G[bZ[WRXgN^łB
   * </japanese>
   *
   * @param aStr1 a <code>String</code> value
   * @param aStr2 a <code>String</code> value
   */
  public OhCannotGetDataError(String aStr1, String aStr2) {
    _errorMessage = aStr1 + System.getProperty("line.separator") + aStr2;
  }

  // @end{constructor} ---------------------------------------------------

  
  // @begin{methods} -----------------------------------------------------

  /**
   * <japanese>
   * G[bZ[W _errorMessage Ԃ܂B
   * </japanese>
   * <p>
   *
   * @return a <code>String</code> value
   */
  public String what() {
    return _errorMessage;
  }

  /**
   * <japanese>
   * G[bZ[W _errorMessage ̊es̐擪ɈɂƂ aPrefix tԂ܂B
   * </japanese>
   * <p>
   * Returns the message which is made by adding 'aPrefix' before each line of the message
   * that was given when the exception object was thrown.
   *
   * @param aPrefix a <code>String</code> value
   * @return a <code>String</code> value
   */
  public String what(String aPrefix) {
    StringReader     sr = null;
    OhBufferedReader br = null;
    
    StringWriter sw = null;
    
    try {
      sr = new StringReader(_errorMessage);
      br = new OhBufferedReader(sr);
      
      sw = new StringWriter();

      boolean afterNewLine = true;
      int c = 0;
      
      while ( (c = br.read()) != -1 ) {
        if ((char)c == '\n') {
          afterNewLine = true;
        }
        else {
          if (afterNewLine) {
            sw.write(aPrefix);
            afterNewLine = false;
          }
        }
        
        sw.write(c);
      }

      br.close();
      sw.close();
      
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    
    return sw.toString();
  }

  // @end{methods} -------------------------------------------------------

} // OhCannotGetDataError
