/**
 * @(#) OhDataPack; class to express data pack.
 *                  Make "OhDataPack.java" from "ohdatapack.h" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^f[^f\NXłB<p>
 *
 * f[^ƃf[^̃yA琬f[^pbN^쐬邽߂̒ۃf[^^fłB<p>
 *
 * ̃NX OhDataPack ł́A
 *
 * <dl>
 *  <li> f[^L^ : _dataTime
 * </dl>
 *
 * LA
 *
 * <dl>
 *  <li> f[^L^̐ݒ@\
 *  <li> f[^L^̎擾@\
 * </dl>
 *
 * `܂B<p>
 *
 * ef󂷂f[^̃f́ÃNX OhDataPack pč쐬ĉB
 *
 * </japanese>
 *
 * <p>
 *
 * OhDataPack;
 *
 * abstract class for treating a pair of data time and data value which we call datapack. <p>
 *
 * In hydrologic simulations in which the states are updated dynamically,
 * the data which are sent and received by elements, subsytems, and total sytem of the hydrologic system
 * are pairs of data value(s) and a "data time".
 * Here, "data time" means the time to which the data are related. <p>
 *
 * For example, suppose we have the value of discharge Q at time t at some stream point.
 * Then, we call the value of discharge Q "data value" and time t "data time". <p>
 *
 * We call a pair of data time and data value "datapack," and we prepare class OhDataPack,
 * which is an abstract class used as a base class from which various datapack classes are derived.
 *
 * <pre>
 * .....
 *   "datapack" = "data time" + "data value"
 * ....
 * </pre>
 *
 * Because there are various data types for data value, there could be various datapack classes. <p>
 *
 * Class OhDataPack is a base class from which various datapack classes are derived.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.20 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhDataPack
{
  // @begin{fields} -------------------------------------------------------

  /**
   * <japanese>
   * f[^L^\ OhTimePoint łB
   * </japanese>
   * <p>
   * The time when the data was recorded.
   * It is of type <a href="OhTimePoint.html">OhTimePoint</a>.
   *
   */
  protected OhTimePoint _dataTime;

  /**
   * <japanese>
   * OS ɈˑȂsR[h\łB
   * </japanese>
   * <p>
   * New Line.
   *
   */
  protected static String NEWLINE = System.getProperty("line.separator");

  // @end{fields} ---------------------------------------------------------

  
  // @begin{abstract method} ---------------------------------------------
  
  /**
   * <japanese>
   * f[^̌^Ԃ܂B
   *
   * ۃ\bhłB<p>
   * 
   * Ⴆ΁AMf[^ int ^Ȃ "int" Ԃ悤ɒ`܂B
   * </japanese>
   * <p>
   * This method returns the data type.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();


  // data transformation.

  /**
   * <japanese>
   * f[^pbNptH[}bgŃt@Cɏo܂B<p>
   *
   * ۃ\bhłB
   * </japanese>
   * <p>
   * A data pack is written out to a file by the format for internal processing.
   *
   * @param os a <code>PrintWriter</code> value
   */
  public abstract void dpack2File(PrintWriter os);

  /**
   * <japanese>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B<p>
   *
   * ۃ\bhłB
   * </japanese>
   * <p>
   * A data pack is written out to a file by simple format.
   *
   * @param os a <code>PrintWriter</code> value
   */
  public abstract void dpack2Ofile(PrintWriter os);

  // @end{abstract method} -----------------------------------------------


  // @begin{constructor} -------------------------------------------------

  /**
   * <japanese>
   * <p>
   * ftHgRXgN^ OhDataPack() ́A
   * oϐ _dataTime  <a href="OhTimePoint.html">OhTimePoint</a> ɏ܂B
   * </japanese>
   * <p>
   * OhDataPack() initializes _dataTime to OhTimePoint().
   *
   */
  public OhDataPack() {
    _dataTime = new OhTimePoint();
  }

  // @end{constructor} ---------------------------------------------------

  
  // @begin{methods} -----------------------------------------------------

  /**
   * <japanese>
   * Rs[\bhłB
   *
   * aDp Ɠ _dataTime  OhDataPack 쐬܂B
   * </japanese>
   * <p>   
   * Copy method.
   *
   * @param aDp an <code>OhDataPack</code> value
   */
  public void copy(OhDataPack aDp) {
    _dataTime.copy(aDp._dataTime);
  }

  
  // setting and getting data recording times

  /**
   * <japanese>
   * _dataTime Ԃ܂B
   * </japanese>
   * <p>
   * This method returns the data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getDataTime() {
    OhTimePoint dt = new OhTimePoint();
    dt.copy(_dataTime);
    return dt;
  }


  /**
   * <japanese>
   * _dataTime ݒ肵܂B
   * </japanese>
   * <p>
   * This method sets the data time. 
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   */
  public void setDataTime(OhTimePoint aDataTime) {
    _dataTime = aDataTime;
  }

  // @end{methods} -------------------------------------------------------

} // OhDataPack
