/**
 * @(#) OhDataPackDouble; class to express datapacks of type double.
 *                        Make "OhDataPackDouble.java" from "ohp_tmp.h", "ohp_tmp.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * f[^l double ^̃f[^pbN\NXłB
 * ̃NX <a href="OhDataPack.html">OhDataPack</a> pĂ܂B
 * </japanese>
 * <p>
 * OhDataPackDouble;
 *
 * OhDataPackDouble is a class of datapacks of type double. 
 *
 * OhDataPackDouble is derived from <a href="OhDataPack.html">OhDataPack</a>. 
 *
 * A datapack of type double is a pair of data of data time and
 * data of type double.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.26 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhDataPackDouble extends OhDataPack
{
  // @begin{field} -----------------------------------------------------

  /**
   * <japanese>
   * double ^̃f[^i[ double lłB
   * </japanese>
   */
  protected double _dataValue;

  // @end{field} -------------------------------------------------------
  

  // @begin{constructors} ----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB
   * </japanese>
   *
   */
  public OhDataPackDouble() {}
  
  /**
   * <japanese>
   * f[^l aDataValueAf[^L^ aDataTime  OhDataPackDouble 쐬܂B
   * </japanese>
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double</code> value
   */
  public OhDataPackDouble(OhTimePoint aDataTime, double aDataValue) {
    _dataTime  = aDataTime;    //; Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * aDataStr Ɋi[Ăf[^pbN񂩂f[^pbN̒lǂݎA
   * _dataTime, _dataValue Ɋi[ꂽ OhDataPackDouble 𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackDouble(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    try {    
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataValue = Double.parseDouble(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }
    
  // @end{constructors} ------------------------------------------------  

  // @begin{methods} ---------------------------------------------------  

  // 0. copy method.

  /**
   * <japanese>
   * Rs[\bhłB
   * aDpd Ɠ _dataTime, _dataValue  OhDataPackDouble 쐬܂B
   * </japanese>
   *
   * @param aDpd an <code>OhDataPackDouble</code> value
   */
  public void copy(OhDataPackDouble aDpd) {
    _dataTime.copy(aDpd._dataTime);
    _dataValue = aDpd._dataValue;
  }
  
  
  /**
   * <japanese>
   * _dataValue Ԃ܂B
   * </japanese>
   * <p>
   * This method returns the data value.
   *
   * @return a <code>double</code> value
   */
  public double getDataValue() {
    return _dataValue;
  }

  /**
   * <japanese>
   * f[^lƃf[^L^擾܂B
   * f[^l double ^z aDataValue ̑vfɊi[܂B
   * </japanese>
   * <p>
   * This method gets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double[]</code> value
   */
  public void getData(OhTimePoint aDataTime, double[] aDataValue) {
    aDataTime.copy(_dataTime);
    aDataValue[0] = _dataValue;
  }
  
  /**
   * <japanese>
   * _dataValue, _dataTime ݒ肵܂B
   * </japanese>
   * <p>
   * This method sets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double</code> value
   */
  public void setData(OhTimePoint aDataTime, double aDataValue) {
    _dataTime  = aDataTime;
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * f[^ľ^ double 𕶎 "double" ƂĕԂ܂B
   * </japanese>
   * <p>
   * This method returns the data type "double".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "double";
  }

  /**
   * <japanese>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]  2003 38600 3.1
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() + " " + _dataTime.getTsec() + " " + _dataValue);
  }

  /**
   * <japanese>
   * f[^pbNptH[}bgŃt@Cɏo܂B
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂ꂼijʂł̂
   * ЂƂsďo܂B
   *
   * <pre>
   * [EXAMPLE]  (2003)
   *            (38600)
   *            (3.1)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{methods} -----------------------------------------------------
  
} // OhDataPackDoule
