/**
 * @(#) OhDataPackFloat; class to express datapacks of type float.
 *                     Make "OhDataPackFloat.java" from "ohp_tmp.h", "ohp_tmp.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * f[^l float ^̃f[^pbN\NXłB
 * ̃NX <a href="OhDataPack.html">OhDataPack</a> pĂ܂B
 * </japanese>
 * <p>
 * OhDataPackFloat;
 *
 * OhDataPackFloat is a class of datapacks of type float. <p>
 *
 * OhDataPackFloat is derived from <a href="OhDataPack.html">OhDataPack</a>. <p>
 *
 * A datapack of type float is a pair of data of data time and
 * data of type float.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.9 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhDataPackFloat extends OhDataPack
{
  // @begin{field} -----------------------------------------------------

  /**
   * <japanese>
   * float ^̃f[^i[郁oϐłB
   * </japanese>
   */
  protected float _dataValue;

  // @end{field} -------------------------------------------------------
  

  // @begin{constructors} ----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB
   * </japanese>
   *
   */
  public OhDataPackFloat() {}
  
  /**
   * <japanese>
   * f[^l aDataValueAf[^L^ aDataTime  OhDataPackDouble 쐬܂B
   * </japanese>
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float</code> value
   */
  public OhDataPackFloat(OhTimePoint aDataTime, float aDataValue) {
    _dataTime  = aDataTime;    //; Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * aDataStr Ɋi[Ăf[^pbN񂩂f[^pbN̒lǂݎA
   * _dataTime, _dataValue Ɋi[ꂽ OhDataPackFloat 𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackFloat(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    try {
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataValue = Float.parseFloat(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }
    
  // @end{constructors} ------------------------------------------------  

  // @begin{methods} ---------------------------------------------------  

  // 0. copy method.

  /**
   * <japanese>
   * Rs[\bhłB
   * aDpf Ɠ _dataTime, _dataValue  OhDataPackFloat 쐬܂B
   * </japanese>
   *
   * @param aDpf an <code>OhDataPackFloat</code> value
   */
  public void copy(OhDataPackFloat aDpf) {
    _dataTime.copy(aDpf._dataTime);
    _dataValue = aDpf._dataValue;
  }
  
  
  /**
   * <japanese>
   * _dataValue Ԃ܂B
   * </japanese>
   * <p>
   * Returns the data value.
   *
   * @return a <code>float</code> value
   */
  public float getDataValue() {
    return _dataValue;
  }

  /**
   * <japanese>
   * f[^lƃf[^L^擾܂B
   * f[^l float ^z aDataValue ̑vfɊi[܂B
   * </japanese>
   * <p>
   * Returns the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float[]</code> value
   */
  public void getData(OhTimePoint aDataTime, float[] aDataValue) {
    aDataTime.copy(_dataTime);
    aDataValue[0] = _dataValue;
  }
  
  /**
   * <japanese>
   * _dataValue, _dataTime ݒ肵܂B
   * </japanese>
   * <p>
   * Sets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float</code> value
   */
  public void setData(OhTimePoint aDataTime, float aDataValue) {
    _dataTime  = aDataTime;    //; "setDataTime(aDataTime);"
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * f[^ľ^ float 𕶎 "float" ƂĕԂ܂B
   * </japanese>
   * <p>
   * Returns the data type "float".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "float";
  }

  /**
   * <japanese>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]  2003 38600 3.1
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() + " " + _dataTime.getTsec() + " " + _dataValue);
  }

  /**
   * <japanese>
   * f[^pbNptH[}bgŃt@Cɏo܂B
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂ꂼijʂł̂
   * ЂƂsďo܂B
   *
   * <pre>
   * [EXAMPLE]  (2003)
   *            (38600)
   *            (3.1)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{methods} -----------------------------------------------------
  
} // OhDataPackFloat
