/**
 * @(#) OhDataPackInt; class to express datapacks of type int.
 *                     Make "OhDataPackInt.java" from "ohp_tmp.h", "ohp_tmp.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * f[^l int ^̃f[^pbN\NXłB
 *
 * ̃NX <a href="OhDataPack.html">OhDataPack</a> pĂ܂B
 * </japanese>
 * <p>
 * OhDataPackInt;
 *
 * OhDataPackInt is a class of datapacks of type int. <p>
 *
 * OhDataPackInt is derived from <a href="OhDataPack.html">OhDataPack</a>. <p>
 *
 * A datapack of type int is a pair of data of data time and
 * data of type int.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.11 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhDataPackInt extends OhDataPack
{
  // @begin{field} -----------------------------------------------------

  /**
   * <japanese>
   * int ^̃f[^i[郁oϐłB
   * </japanese>
   */
  protected int _dataValue;

  // @end{field} -------------------------------------------------------
  

  // @begin{constructors} ----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB
   * </japanese>
   *
   */
  public OhDataPackInt() {}
  
  /**
   * <japanese>
   * f[^l aDataValueAf[^L^ aDataTime  OhDataPackInt 쐬܂B
   * </japanese>
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>int</code> value
   */
  public OhDataPackInt(OhTimePoint aDataTime, int aDataValue) {
    _dataTime  = aDataTime;    //; Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * _dataStr Ɋi[Ăf[^pbN񂩂f[^pbN̒lǂݎA
   * _dataTime, _dataValue Ɋi[ꂽ OhDataPackInt 𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackInt(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    try {
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataValue = Integer.parseInt(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }
    
  // @end{constructors} ------------------------------------------------  

  // @begin{methods} ---------------------------------------------------  

  // 0. copy method.

  /**
   * <japanese>
   * Rs[\bhłB
   *
   * aDpi Ɠ _dataTime, _dataValue  OhDataPackInt 쐬܂B
   * </japanese>
   *
   * @param aDpi an <code>OhDataPackInt</code> value
   */
  public void copy(OhDataPackInt aDpi) {
    _dataTime.copy(aDpi._dataTime);
    _dataValue = aDpi._dataValue;
  }
  
  
  /**
   * <japanese>
   * _dataValue Ԃ܂B
   * </japanese>
   * <p>
   * Returns the data value.
   *
   * @return a <code>int</code> value
   */
  public int getDataValue() {
    return _dataValue;
  }

  /**
   * <japanese>
   * f[^lƃf[^L^擾܂B
   * f[^l float ^z aDataValue ̑vfɊi[܂B
   * </japanese>
   * <p>
   * Returns the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>int[]</code> value
   */
  public void getData(OhTimePoint aDataTime, int[] aDataValue) {
    aDataTime.copy(_dataTime);
    aDataValue[0] = _dataValue;
  }
  
  /**
   * <japanese>
   * _dataValue, _dataTime ݒ肵܂B
   * </japanese>
   * <p>
   * Sets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>int</code> value
   */
  public void setData(OhTimePoint aDataTime, int aDataValue) {
    _dataTime  = aDataTime;    //; "setDataTime(aDataTime);"
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * f[^ľ^ int 𕶎 "int" ƂĕԂ܂B
   * </japanese>
   * <p>
   * Returns the data type "int".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "int";
  }

  /**
   * <japanese>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B
   *
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]  2003 38600 3
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() + " " + _dataTime.getTsec() + " " + _dataValue);
  }

  /**
   * <japanese>
   * f[^pbNptH[}bgŃt@Cɏo܂B
   *
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂ꂼijʂł̂
   * ЂƂsďo܂B
   *
   * <pre>
   * [EXAMPLE]  (2003)
   *            (38600)
   *            (3)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{methods} -----------------------------------------------------
  
} // OhDataPackInt
