/**
 * @(#) OhDataPackLong; class to express datapacks of type long.
 *                      Make "OhDataPackLong.java" from "ohp_tmp.h", "ohp_tmp.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * f[^l long ^̃f[^pbN\NXłB
 *
 * ̃NX <a href="OhDataPack.html">OhDataPack</a> pĂ܂B
 * </japanese>
 * <p>
 * OhDataPackLong;
 *
 * OhDataPackLong is a class of datapacks of type long. <p>
 *
 * OhDataPackLong is derived from <a href="OhDataPack.html">OhDataPack</a>. <p>
 *
 * A datapack of type long is a pair of data of data time and
 * data of type long.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.9 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhDataPackLong extends OhDataPack
{
  // @begin{field} -----------------------------------------------------

  /**
   * <japanese>
   * long ^̃f[^i[郁oϐłB
   * </japanese>
   */
  protected long _dataValue;

  // @end{field} -------------------------------------------------------
  

  // @begin{constructors} ----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB
   * </japanese>
   *
   */
  public OhDataPackLong() {}
  
  /**
   * <japanese>
   * f[^l aDataValueAf[^L^ aDataTime  OhDataPackLong 쐬܂B
   * </japanese>
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>long</code> value
   */
  public OhDataPackLong(OhTimePoint aDataTime, long aDataValue) {
    _dataTime  = aDataTime;    //; Field of the super class OhDataPack.
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * _dataStr Ɋi[Ăf[^pbN񂩂f[^pbN̒lǂݎA
   * _dataTime, _dataValue Ɋi[ꂽ OhDataPackLong 𐶐܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public OhDataPackLong(String aDataStr) {
    StringReader     sr;
    OhBufferedReader br;

    try {    
      sr = new StringReader(aDataStr);
      br = new OhBufferedReader(sr);

      _dataTime.setYear( Integer.parseInt(br.readWord()) );
      _dataTime.setTsec( Long.parseLong(br.readWord()) );
      _dataValue = Long.parseLong(br.readWord());
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }
    
  // @end{constructors} ------------------------------------------------  

  // @begin{methods} ---------------------------------------------------  

  // 0. copy method.

  /**
   * <japanese>
   * Rs[\bhłB
   *
   * aDpl Ɠ _dataTime, _dataValue  OhDataPackLong 쐬܂B
   * </japanese>
   *
   * @param aDpl an <code>OhDataPackLong</code> value
   */
  public void copy(OhDataPackLong aDpl) {
    _dataTime.copy(aDpl._dataTime);
    _dataValue = aDpl._dataValue;
  }
  
  
  /**
   * <japanese>
   * _dataValue Ԃ܂B
   * </japanese>
   * <p>
   * Returns the data value.
   *
   * @return a <code>long</code> value
   */
  public long getDataValue() {
    return _dataValue;
  }

  /**
   * <japanese>
   * f[^lƃf[^L^擾܂B
   * f[^l float ^z aDataValue ̑vfɊi[܂B
   * </japanese>
   * <p>
   * Returns the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>long[]</code> value
   */
  public void getData(OhTimePoint aDataTime, long[] aDataValue) {
    aDataTime.copy(_dataTime);
    aDataValue[0] = _dataValue;
  }
  
  /**
   * <japanese>
   * _dataValue, _dataTime ݒ肵܂B
   * </japanese>
   * <p>
   * Sets the data time and data value.
   *
   * @param aDataTime an <code>OhTimePoint</code> value
   * @param aDataValue a <code>long</code> value
   */
  public void setData(OhTimePoint aDataTime, long aDataValue) {
    _dataTime  = aDataTime;    //; "setDataTime(aDataTime);"
    _dataValue = aDataValue;
  }

  /**
   * <japanese>
   * f[^ľ^ long 𕶎 "long" ƂĕԂ܂B
   * </japanese>
   * <p>
   * Returns the data type "long".
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "long";
  }

  /**
   * <japanese>
   * f[^pbNȈՃtH[}bgŃt@Cɏo܂B
   * ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂Ĉsɏo܂B
   *
   * <pre>
   * [EXAMPLE]  2003 38600 4907
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2Ofile(PrintWriter os) {
    os.print(_dataTime.getYear() + " " + _dataTime.getTsec() + " " + _dataValue);
  }

  /**
   * <japanese>
   * f[^pbNptH[}bgŃt@Cɏo܂B
   * ptH[}bgƂ́A_year, _tsec, _dataValue ̂ꂼijʂł̂
   * ЂƂsďo܂B
   *
   * <pre>
   * [EXAMPLE]  (2003)
   *            (38600)
   *            (4907)
   * </pre>
   * </japanese>
   *
   * @param os a <code>PrintWriter</code> value
   */
  public void dpack2File(PrintWriter os) {
    os.print("(" + _dataTime.getYear() + ")" + NEWLINE);
    os.print("(" + _dataTime.getTsec() + ")" + NEWLINE);
    os.print("(" + _dataValue + ")" + NEWLINE);
  }

  // @end{methods} -----------------------------------------------------
  
} // OhDataPackLong
