/**
 * @(#) OhInputFile; abstract class to express port models of input files.
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * ̓t@CpM[qf\NXłB<p>
 *
 * ̓t@CpM[qf́A
 * M[q݂̂vffłƉ߂邱Ƃł܂B
 * Ȃ킿Ãf́At@Cf[^ǂݍ݁A
 * f[^pbNɕϊđM[qɃf[^pbNXgbN܂B<p>
 *
 * ۂɂ́A̓t@CpM[qf́A
 * vff̂PƂăfꂸɁA
 * M[qf̔hfƂĎĂ܂B
 * Ȃ킿AM[qfɃt@Cǂݎ@\ǉāA
 * ̓t@CpM[qf쐬Ă܂B
 * ̊֌Ŵ͎悤łB<p>
 *
 * NX OhInputFile ł́A<p>
 *
 *  E̓t@C             : <a href="OhObject.html#_objectName">OhObject._objectName</a> <br>
 *  E̓t@C̍ŏIXV : _fileTimeAtInput <p>
 *
 * LA<p>
 *
 *  E̓t@C̃f[^pbNM@\ <p>
 *
 * `܂B<p>
 *
 * p^t@C⏉lf[^t@CȂǂ́ANX
 * <a href="OhTotalSystem.html">OhTotalSystem</a> ̃IuWFNg
 * <a href="OhTotalSystem.html#readParameterFile()">OhTotalSystem.readParameterFile()</a> 
 * <a href="OhTotalSystem.html#readInitialStateFile()">OhTotalSystem.readInitialStateFile()</a> Ȃǂ
 * pēǂݎƂ܂B
 * āAp^⏉lȂǂ́A[qʂċ킯ł͂܂B
 * OhInputFile ̓t@CƂ́A
 * ԓIɕωf[^^Ăt@CłB<p>
 *
 * NX OhInputFile ̃\bh
 * <a href="OhInputFile.html#inputData()">OhInputFile.inputData()</a> ́A
 * f[^Ԍ^zw肷si#& dctVec Ŏn܂sj
 * t@Cǂݎ@\Ă܂B
 * </japanese>
 *
 * <P>
 *
 * OhInputFile;
 *
 * Abstract class of port models of input files. <p>
 *
 * An object of "OhInputFile" class
 *
 * <dl><dd>
 * <dl><dt> (Object) _objectName
 *       <dd> input file name
 *     <dt> _fileTimeAtInput
 *       <dd> final update time of the input file
 * </dl></dl>
 *
 * and has
 *
 * <dl><dd>
 * method to read in data and send them as datapacks to the send port.
 * </dl>
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.37 $ $Date: 2004/02/12 05:56:35 $
 */
public abstract class OhInputFile extends OhInputPort
{
  // @begin {field} ------------------------------------------------------

  /**
   * <japanese>
   * ̓t@C̍ŏIXVłB<p>
   *
   * Vf[^擾ꂽƂɁAO̓̓t@Ct@Cɒǉ邩A
   * ㏑邩Iׂ悤ɑz肵Ă܂B
   * ̂߂ɁAf[^ǂݍ񂾂ƂɁAt@C̍ŏIXVL^ĂA
   * Ƀt@CǂޕKvƂɁAt@C̍ŏIXVāA
   * t@CXVꂽ悤ɂ܂B
   * </japanese>
   * <p>
   * Final update time of the input file.
   *
   */
  protected long _fileTimeAtInput;

  // @end {field} --------------------------------------------------------


  // @begin {abstract methods} -------------------------------------------

  // 1. connection function

  /**
   * <japanese>
   * ̓f[^pbÑf[^ľ^𕶎ƂĕԂ܂B<p>
   *
   * ۃ\bhłB
   * </japanese>
   * <p>
   * Returns the string showing the type of the sending datapacks.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();


  // 2. sending function of the input file data
  
  /**
   * <japanese>
   * ŗ^f[^f[^pbN쐬AObject ^ɃLXgĖ߂lƂĕԂ܂B<p>
   *
   * ۃ\bhłB<p>
   *
   * f[^f[^pbN쐬܂B
   * ۃ\bhłB
   * hNXŎ邱ƂvĂ܂B<p>
   *
   * Psf[^P̃f[^pbNɕϊ郁\bh
   * hNXŒ`ĉB<p>
   *
   * WIȔhNXł́A
   * Mf[^pbN^ɉĎ̎葱`ĉB<p>
   *
   * <pre>
   * (1) Mpf[^pbN̋L̈q[vɐB
   * (2)  aLineData f[^L^Af[^lǂݎB
   * (3) (2) œǂݎl (1) ̃f[^pbÑoɑB
   * (4) (1) ̃f[^pbN Object ^ɃLXgĕԂB
   * </pre>
   *
   * ۃ\bhȂ̂ŔhNXŕK`ĉB
   *
   * <pre>
   * [ARGUMENT]
   *  EaLineData -> ̓f[^Ps̕
   * [RETURN]
   *  EObject : Mf[^pbN Object ^ɃLXǵB
   * </pre>
   * </japanese>
   * <p>
   * Makes datapacks from string data.
   *
   * @param aLineData a <code>String</code> value
   * @return an <code>Object</code> value
   */
  public abstract Object makeDataPack(String aLineData);

  // @end {abstract methods} ---------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNg(t@C) aFileNameAIuWFNgԍ 0 ɏ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aFileName) throws OhError {
    return super.init(aFileName, 0);
  }

  // @end{initialization} ------------------------------------------------

  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^
   * <a href="OhInputFile.html#init(String, int)">OhInputFile.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhInputFile.html#init(String, int)">OhInputFile.init(String, int)</a> 
   * păoϐĉB
   * </jananese>
   *
   * @exception OhError if an error occurs
   */
  public OhInputFile() throws OhError {
    super();
    _fileTimeAtInput = Long.MIN_VALUE;
    _className = "OhInputFile";
  }

  /**
   * <japanese>
   * NX <a href="OhInputPort.html">OhInputPort</a> ̃ftHgRXgN^pA
   * <a href="OhInputFile.html#init(String, int)">OhInputFile.init(String, int)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhInputFile"
   *  <li> _objectName        : aFileName
   *  <li> _objectNumber      : 0
   *  <li> _owner             : null
   *  <li> _bufferList        : ɊmۂB(ڑƗp)
   *  <li> _receiverArray     : null
   *  <li> _numberOfReceivers : -1
   *  <li> _dataPackArray     : null
   *  <li> _numberOfDataPacks : -1
   *  <li> _fileTimeAtInput   : Long.MIN_VALUE (lŏ long l)
   * </dl>
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhInputFile(String aFileName) throws OhError {
    super();
    _fileTimeAtInput = Long.MIN_VALUE;
    _className = "OhInputFile";
    init(aFileName);
  }

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  // 2. sending function of the input file data

  /**
   * <japanese>
   *  aStr f[^Ԍ^xNgǂݎA߂lƂĕԂ܂B
   * </japanese>
   * <p>
   * Changes into data connection type vector from the string "aStr",
   * and returns it.
   *
   * @param aStr a <code>String</code> value
   * @return a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public Vector getDctVec(String aStr) throws OhError {    //; (*1)
    int n = 0;    // The number of the ingredient of data.

    Vector dctVec = new Vector();

    StringTokenizer st = new StringTokenizer(aStr);

    try {
      n = Integer.parseInt(st.nextToken());
    }
    catch (NumberFormatException e) {
      throw e;
    }

    try {
      for (int i = 0; i < n; i++) {
        int k = Integer.parseInt(st.nextToken());

        if (0 <= k && k < DataConnectionType.getNumOfDCT())
          dctVec.addElement(DataConnectionType.getDctType(k));
        else {
          OhError e = new OhError("Invalid connection type.", "");
          throw e;
        }
      }
    }
    catch (NumberFormatException e) {
      throw e;
    }
    return dctVec;
  }


  /**
   * <japanese>
   * ̓t@CǂݎA
   * <a href="OhInputFile.html#makeDataPack(String)">OhInputFile.makeDataPack(String)</a> ɂ
   * f[^pbN쐬A쐬f[^pbN
   * <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> 
   * pđM܂(OhInputFile  OhSendPort ̈ł)B<p>
   *
   * t@CǂݎƂAs # ̏ꍇA
   * ̍sRgsƌȂ܂B<p>
   *
   * As # łǍ & ƁAȂ킿 #& Ŏn܂śA
   * Rgł͂ȂAȃf[^LڂĂ̂Ƃ܂B<p>
   *
   * ݁i2004/01/16ĵƂA
   * <a href="DataConnectionType.html">DataConnectionType</a> Vector w肷ꍇ邾łB
   *
   * <pre>
   * # DataConnectionType = {0:BROKENLINE, 1:STEP, 2:PULSE, 3:NOINTP}
   * #& dctvec 2 0 1
   * </pre>
   *
   * ̂悤ɏĂƁA<a href="DataConnectionType.html">DataConnectionType</a> Vector 
   * w肷̂ƌȂ܂B
   * ̗́Af[^QāAP BROKENLINE A
   * Q STEP ł邱ƂӖ܂B
   * wȂʂ̏ꍇ́A
   *
   * <pre>
   * #& dctvec 1 0
   * </pre>
   *
   * Ƃ̂ƓłB<p>
   *
   * t@C܂ŏIA
   * <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> 
   * p _dataPackArray 쐬܂B<p>
   *
   * A̓t@C̍ŏIύX _fileTimeAtInput ȑȌꍇ͉ return ܂B
   * RXgN^ _fileTimeAtInput  Long.MIN_VALUE ɐݒ肳̂
   * ͕̓͂Ks܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void inputData() throws OhError {    //; (*2)

    setDataPackArray();

    reduceDataPackStock();  // Delete the unnecessary datapacks.

    File f = new File(_objectName);  // Checks work directory by "System.getProperty("user.dir")".

    if (f.exists() != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("Cannot find the file " + _objectName);
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }

    if (f.lastModified() <= _fileTimeAtInput)
      return;

    _fileTimeAtInput = f.lastModified();


    FileReader       fr       = null;
    OhBufferedReader br       = null;
    String           lineData = null;

    try {
      fr = new FileReader(_objectName);
      br = new OhBufferedReader(fr);    //; Do not use OhBufferedReader(fr, true).

      while ((lineData = br.readLine()) != null) {    // -------- start "while"
        
        if (lineData.length() <= 0)
          continue;

        if (lineData.substring(0, 1).equals(" ") || lineData.substring(0,1).equals("\t"))
          continue;
        
        if (lineData.substring(0, 1).equals("#")) {     // ----------------- start "if"
          
          if (lineData.length() <= 1)
            continue;

          if (lineData.substring(1, 2).equals("&")) {
            String operation = lineData.substring(3, 9);
            if (operation.equals("dctVec") || operation.equals("dctvec")) {
              String dctString = lineData.substring(10);
              setDctVec(getDctVec(dctString));              
            }
          }
          
          continue;    // Skip comment lines.
          
        }    // --------------------------------------------------- end "if"

        dataPackStock(makeDataPack(lineData));
        
      }    // ----------------------------------------------------- end "while"
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("Cannot open " + _objectName + ".");
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();  
      throw new OhError(str);
    }
    
    setDataPackArray();    //; Set "_numberOfDataPacks = _dataPackArray.size();"

    if (getNumberOfDataPacks() > 0) {
      OhTimePoint tp = getNewestDataTime();

      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("File: " + _objectName + "  newest data time : " + tp.getYear() + " " + tp.getTsec());
      out.print(NEWLINE);
      String str = buffer.toString();
      out.close();  
      OhObject obj = (OhObject)getTopOwner();
      OhTotalSystem top = (OhTotalSystem)obj;
      if (top != null) {
        top.addNewestDataTime(str);
      }
    }
  }


  /**
   * <japanese>
   * [q͌vZ@\Ȃ̂ŁAvZvĂǂ傤ȂB
   * KO <a href="OhCannotGetDataError.html">OhCannotGetDataError</a> 𓊂܂B
   * </japanese>
   * <p>
   * When the input port receives a calculation request, throw OhCannotGetDataError.
   * This will be caught in the catch block in OhTotalSystem.work().
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError {
    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);
    out.print("Cannot get data from " + _objectName);
    String pauseComment = buffer.toString();
    out.close();
    
    OhInterface.pause(pauseComment);

    throw new OhCannotGetDataError(pauseComment);
  }
  

  // 3. information display function

  /**
   * <japanese>
   * IuWFNg(L҂̏܂܂Ȃ)o͂܂B<p>
   *
   * ̓t@CƂ̒[q̑MM[q̏Xg[ aPw ɏo͂܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> IuWFNgo͂Xg[B
   *  EaHead -> RgB
   * </pre>
   * </japanese>
   * <p>
   * The information on an input file name and the sending agency receiving port of this port
   * is outputted to File "aPw".
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void cPrint(PrintWriter aPw, String aHead) {    //; (*3)
    aPw.print(aHead + "OhInputFile : " + _className + "<" + dataType() + ">" + _objectName + " ");
    aPw.print(System.getProperty("line.separator"));
  }

  /**
   * <japanese>
   * IuWFNg(L҂̏܂)o͂܂B<p>
   *
   * ̓t@CƂ̒[q̑MM[q̏Xg[ aPw ɏo͂܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> IuWFNgo͂Xg[B
   *  EaHead -> RgB
   * </pre>
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void oPrint(PrintWriter aPw, String aHead) {    //; (*4)
    cPrint(aPw, aHead);
  }
  
  // @end {methods} ------------------------------------------------------

} // OhInputFile
