/**
 * @(#) OhInputFileDouble; class of input port which reads in datapacks of type double
 *                         from an input file and sends them.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * <japanese>
 * t@Cf[^l double ^łf[^pbNǂݎ͒[q\NXłB<p>
 * </japanese>
 * <p>
 *
 * OhInputFileDouble;
 *
 * OhInputFileDouble is a class of input port
 * which reads in datapacks of type double from an input file and sends them. <p>
 *
 * OhInputFileDouble can be connected with
 * <a href="OhReceiveDouble.html">OhReceiveDouble</a>,
 * <a href="OhRelayDouble.html">OhRelayDouble</a>, and 
 * <a href="OhOutputFileDouble.html">OhOutputFileDouble</a>. <p>
 *
 * OhInputFileDouble reads an input file using "simple data format for input and output."
 * "Simple data format for input and output" is a format in which
 *
 * <pre>
 *  .....
 *    all the data items for a datapack should be written in one line,
 *    and data items should be separated with a number of spaces or tabs.
 *
 *    Example: 2003 120 1.2 2.3 4.5
 * ....
 * </pre>
 *
 * If you need different format,
 * please make a derived class of OhInputFileDouble and redefine "makeDataPack()".
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.25 $ $Date: 2004/02/12 05:06:28 $
 */
public class OhInputFileDouble extends OhInputFile
{
  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNg(t@C) aFileNameAIuWFNgԍ 0 ɏ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aFileName) throws OhError {
    return super.init(aFileName);
  }

  // @end{initialization} ------------------------------------------------

  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^
   * <a href="OhInputFileDouble.html#init(java.lang.String, int)">OhInputFileDouble.init(String, int)</a>
   * s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA
   * <a href="OhInputFileDouble.html#init(java.lang.String, int)">OhInputFileDouble.init(String, int)</a>
   * păoϐĉB
   * </jananese>
   *
   * @exception OhError if an error occurs
   */
  public OhInputFileDouble() throws OhError {
    super();
    _className = "OhInputFileDouble";
  }


  /**
   * <japanese>
   * NX̃RXgN^
   * <a href="OhInputFile.html#OhInputFile(java.lang.String)">OhInputFile.OhInputFile(String)</a>
   * pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhInputFileDouble"
   *  <li> _objectName        : aFileName
   *  <li> _objectNumber      : 0
   *  <li> _owner             : null
   *  <li> _bufferList        : ɊmۂB(ڑƗp)
   *  <li> _receiverArray     : null
   *  <li> _numberOfReceivers : -1
   *  <li> _dataPackArray     : null
   *  <li> _numberOfDataPacks : -1
   *  <li> _fileTimeAtInput   : Long.MIN_VALUE (lŏ long l)
   * </dl>
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhInputFileDouble(String aFileName) throws OhError {
    super(aFileName);
    _className = "OhInputFileDouble";
  }

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  // 1. connection function.

  /**
   * <japanese>
   * ̓f[^pbÑf[^ľ^ double 𕶎 "double" ƂĕԂ܂B<p>
   * </japanese>
   * <p>
   * Returns the data pack type which expresses the type of data to be received. 
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "double";
  }


  // 2. data input function

  /**
   * <japanese>
   *  aLineData Ɋi[Ăf[^pbN񂩂f[^pbN쐬A
   * Object ^ɃLXgĖ߂܂B<p>
   *
   * ygpz<p>
   * Object dataPack = makeDataPack("2003 12 3.5");
   * </japanese>
   * <p>
   *
   * Reads string datapacks of type double, creates datapacks of type double, and returns it.
   *
   * @param aDataStr a <code>String</code> value
   * @return an <code>Object</code> value
   */
  public Object makeDataPack(String aDataStr) {         //; (*1)
    OhDataPackDouble dp = new OhDataPackDouble(aDataStr);
    return ((Object)dp);
  }


  /**
   * <japanese>
   *  aStr Ɋi[Ăf[^pbN񂩂f[^pbN쐬A
   * ͒[qɃXgbN܂B
   * </japanese>
   * <p>
   * Sends datapacks of type double which is stored in the form of string.
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {           //; (*2)
    OhDataPackDouble dp = new OhDataPackDouble(aDataStr);
    dataPackStock(dp);
  }

  // @end {methods} ------------------------------------------------------

} // OhInputFileDouble
