/**
 * @(#) OhInputPort; abstract class to express input ports.
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^͒[qf\NXłB<p>
 * 
 * Šn̓͒[q쐬邽߂̊NXłB <p>
 * ۃNXłB<p>
 *
 * {^͒[q́AŠn̓͒[q쐬邽߂̒ۃNXłB
 * [ÚÃNXpđŠn̓͒[q̃f쐬ĂB
 * {^͒[q̃NX錾ł́A[Uׂ͒[q̊{dl߂Ă܂B
 * ܂A̎ł͎Oɗpӂł@\`Ă܂B<p>
 *
 * Šnf́Å{^͒[q̃NXpă[U쐬
 * ͒[qʂăf[^肷̂ƂĐ݌vĂ܂B<p>
 *
 * Šn̓ɑ΂ē͒[q̓f[^󂯂ƂƂӖł́A
 * ͒[q́A{^vff̎M[qɎĂ܂ASʂ̂̂
 * l悢łB<p>
 *
 * Šnf́At@CóAlbg[NɂóAL[{[h́A
 * CRT o͂ȂǁAXGȃf[^̓o͂ɑΉȂ΂Ȃ܂B
 * Šnf́AGȃf[^n̓Ɏ荞ގ́A
 * S <a href="OhDataPack.html">OhDataPack</a> ^̃f[^ɕϊȂ΂Ȃ܂B
 * ܂AV~[VʂV~[Vs҂ɒ񋟂鎞́A
 * <a href="OhDataPack.html">OhDataPack</a> ^Œ~ςĂf[^
 * V~[Vs҂̖]ރf[^`ɕϊȂ΂Ȃ܂B
 * ̂悤ȃf[^̕ϊɊւ̓ÍA
 * [U쐬͒[qEo͒[qōsȂƂɂA
 * Šnf̕ł́A͒[qEo͒[q̎葱ő삷邱Ƃɂ܂B
 * Ȃ킿A͒[qEo͒[q́A
 * ̒^IȏƊȌƂ̒邱ƂɂȂ܂B
 *
 * </japanese>
 *
 * <p>
 *
 * OhInputPort;
 *
 * Abstract class for creating input ports for a total system. <p>
 *
 * This is the base class from which the user creates his own input port models.
 *
 * Do not confuse "OhInputPort" with "OhReceivePort".
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.29 $ $Date: 2004/02/12 05:56:35 $
 */
public abstract class OhInputPort extends OhSendPort
{
  // @begin {abstract methods} -------------------------------------------

  // 1. Connection function.
  
  /**
   * <japanese>
   * ̓f[^pbÑf[^ľ^𕶎ƂĕԂ܂B<p>
   *
   * ۃ\bhłB<p>
   * </japanese>
   * <p>
   * Returns the string which shows data type treated by the current input-port.
   * Because this is pure virtual, the method must be defined in derived classes from this class.
   *
   * <dl> <dt>Example
   * <dd> public String dataType() { return "double"; }
   * </dl>
   *
   * We call the string returned by this funcion "dataType".
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();


  // 2. Input data sending function.

  /**
   * <japanese>
   * ̓f[^͒[qɋL܂B<p>
   *
   * ۃ\bhłB<p>
   *
   * e͒[qŁA̓f[^f[^pbN쐬A
   * f[^pbNL郁\bh`ĉB
   * </japanese>
   * <p>
   * Sending input data. <p>
   *
   * With each input terminal, Please create a data pack from input data,
   * and define the method which sends a data pack.
   *
   */
  public abstract void inputData() throws OhError;

  // @end {abstract methods} ---------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------

  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^
   * <a href="OhInputPort.html#init(java.lang.String, int)">OhInputPort.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhInputPort.html#init(java.lang.String, int)">OhInputPort.init(String, int)</a> 
   * păoϐĉB
   * </jananese>
   *
   * @exception OhError if an error occurs
   */
  public OhInputPort() throws OhError {
    super();
    _className = "OhInputPort";
  }


  /**
   * <japanese>
   * NX <a href="OhSendPort.html">OhSendPort</a> ̃ftHgRXgN^pA
   * <a href="OhInputPort.html#init(java.lang.String, int)">OhInputPort.init(String, int)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhInputPort"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _owner             : null
   *  <li> _bufferList        : ɊmۂBiڑƗpj
   *  <li> _receiverArray     : null
   *  <li> _numberOfReceivers : -1
   *  <li> _dataPackArray     : null
   *  <li> _numberOfDataPacks : -1
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhInputPort(String aPortName, int aPortNum) throws OhError {
    super();
    _className = "OhInputPort";
    init(aPortName, aPortNum);
  }

  // @end {constructors} -------------------------------------------------


  // @begin {methods} ----------------------------------------------------

  // 2. input data sending function
  
  /**
   * <japanese>
   * [q͌vZ@\Ȃ̂ŁAvZvĂǂ傤ȂB
   * KO <a href="OhCannotGetDataError.html">OhCannotGetDataError</a> 𓊂܂B
   * </japanese>
   * <p>
   * Since there is no calculation method in a file,
   * even if there is a calculation request,
   * it cannot carry out what, either.
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError{
    throw new OhCannotGetDataError("");
  }
  
  // @end {methods} ------------------------------------------------------

} // OhInputPort

