/**
 * @(#) OhInterface; base class for OHyMoSJ interface.
 *
 */

package ohymosj;

import ohymosj.*;

import java.util.*;
import java.io.*;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.text.BadLocationException;

/**
 * <japanese>
 * OHyMoSJ GUI \NXłB
 * </japanese>
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.17 $ $Date: 2004/02/12 05:56:35 $
 */
public class OhInterface {

  /**
   * <japanese>
   * Window ʂďƂ肷Iꍇ́A
   * ̃̕bZ[W͗p_CAO{bNX OhWin \܂B
   * getReply \bh́A͂āuv{^ꍇA
   * ̕Ԃ܂B󔒁E󂫕F܂B
   * uv܂͉É~{^ꍇ́Anull Ԃ܂B
   * </japanese>
   * <p>
   * Gets reply.
   *
   * @param aMessage a <code>String</code> value
   * @param aDefaultReply a <code>String</code> value
   * @return a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public static String getReply(String aMessage, String aDefaultReply) throws OhError {

    Object obj = null;
    String rep = null;
    
    if (OhOption.getDefault().getUseWinP()) {    //; ------------------- Start WINDOWS.
    
      ImageIcon myIcon = new ImageIcon("khlogo.png");
      
      obj = JOptionPane.showInputDialog(
        OhWin.getDefault(),           //; Parent frame.
        aMessage,                     //; Message.
        "getReply",                   //; Title.
        JOptionPane.PLAIN_MESSAGE,    //; Message type.
        myIcon,                       //; HY icon.
        null,
        aDefaultReply                 //; Default reply.
      );

      if (obj == null)
        throw new OhError("Quit.");

      rep = obj.toString();
      
    }    //; ------------------------------------------------------------- end WINDOWS.
    else {
      System.out.print(aMessage);

      InputStreamReader isr = new InputStreamReader(System.in);
      OhBufferedReader  br  = new OhBufferedReader(isr);

      try {
        rep = br.readLine();
      }
      catch (IOException e) {
        throw new OhError("Quit.");
      }
    }

    if (rep.equals("^C"))
      throw new OhError("Quit.");
    
    return rep;
  }


  /**
   * <japanese>
   * Window ʂďƂ肷Iꍇ́A
   * ̃̕bZ[W OhWin \܂B
   * ̃EBhÉAuOKvAuBREAKvƕ\ꂽQ̃{^܂B
   * uOKv{^ƁAɖ߂܂B
   * uBREAKv܂͉É~{^ƁAuQuit.vƂ OhError 𓊂܂B
   * </japanese>
   * <p>
   * Pause.
   *
   * @param aMessage a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public static void pause(String aMessage) throws OhError {

    if (OhOption.getDefault().getUseWinP()) {    //; ------------------- Start WINDOWS.
      
      ImageIcon myIcon  = new ImageIcon("khlogo.png");
      JLabel    myLabel = new JLabel(myIcon);         //; Makes KH logo.

      Object[] bts = {myLabel, "OK", "BREAK"};        //; Makes buttons.
      
      int ret = JOptionPane.showOptionDialog(
        OhWin.getDefault(),        //; Parent frame.
        aMessage,                  //; Message.
        "OhPause",                 //; Title.
        JOptionPane.DEFAULT_OPTION,
        JOptionPane.PLAIN_MESSAGE,
        null,
        bts,bts[1]);
      

      if (ret == 1) {    //; Click "OK".
        return;
      }
      else if (ret == 2 || ret == -1) {      //; Click "BREAK" or close button.
        throw new OhError("Quit.");
      }
    }   //; -------------------------------------------------------------- end WINDOWS.
    else {
      System.out.println(aMessage);
      System.out.println("Hit return key: ");

      InputStreamReader isr = new InputStreamReader(System.in);
      OhBufferedReader  br  = new OhBufferedReader(isr);

      try {
        String s = br.readLine();
      }
      catch (IOException e) {
        throw new OhError("Quit.");
      }
    }

  }

} // OhInterface
