/**
 * @(#) OhObject; class to express a object
 *
 */

package ohymosj;

import ohymosj.*;
import java.io.*;

/**
 * <japanese>
 * {^f\NXłB <p>
 *
 * NX OhObject ł́A
 *
 * <pre>
 *  EIuWFNg̖O    : _className, _objectName, _objectNumber
 *  EIuWFNg̏L  : _owner
 * </pre>
 *
 * LA
 *
 * <pre>
 *  EIuWFNg̔r@\
 *  EL҂̐ݒ@\
 *  EIuWFNg̏o͋@\
 * </pre>
 *
 * `܂B<p>
 *
 * OHyMoSJ ̃NX͑SĊ{^f̃NX OhObject pč쐬܂B<p>
 *
 * OHyMoSJ ̃[ÚAʂ̓NX OhObject pč쐬ꂽ
 * <a href="OhElement.html">OhElement</a>  <a href="OhSendPort.html">OhSendPort</a> ȂǁA
 * NX OhObject ̉ʂ̃NXpă[UKvƂNX쐬Ώ\łāA
 * NX OhObject 𒼐ڌpăNX쐬邱Ƃ͂Ȃł傤B<p>
 *
 * NX OhObject ́Aoϐ
 *
 * <pre>
 *	String _className;   //; NX
 *	String _objectName;  //; IuWFNg
 *	int _objectNumber;   //; IuWFNgԍ
 * 	Object _owner;       //; Object ^ɃLXgꂽL
 * </pre>
 *
 * Ă܂B
 * OHyMoSJ ̃NX͑SĊ{^ OhObject pč쐬̂ŁA
 * OHyMoSJ ̑SẴCX^X̃oϐĂ邱ƂɂȂ܂B<p>
 *
 * _className ́AehNXƂɋʂ̕ɐݒ肵ĉB
 * Ⴆ΁AhNX ABC ł̓RXgN^
 *
 * <pre>
 *  _className = "ABC";
 * </pre>
 *
 * Ɛݒ肵܂B<p>
 *
 * _objectName, _objectNumber ɂ́ÃIuWFNgg̖OƔԍ
 * <a href="OhObject.html#OhObject(java.lang.String, int)">OhObject.OhObject(String, int)</a>
 * ܂
 * <a href="OhObject.html#init(java.lang.String, int)">OhObject.init(String, int)</a>
 * Őݒ肵܂B
 * dݒ͂ł܂B<p>
 *
 * vO}I _objectName, _objectNumber ݒ肵ĂȂƂ́A
 * _objectName  nullA_objectNumber  -1 ɐݒ肳Ă܂B<p>
 *
 * yȒPȗz
 *
 * <pre>
 *  public class Yahoo extends OhObject
 *  {
 *    public Yahoo() {
 *      super();
 *      _className = "Yahoo";
 *    }
 *
 *    public Yahoo(String aObjName, int aObjNum) {
 *      super();
 *      _className = "Yahoo";
 *      init(aObjName, aObjNum);
 *    }
 *    .
 *    .
 *    .
 *  }
 * </pre>
 *
 * _owner ɂ́AL҂ <a href="OhObject.html#setOwner(java.lang.Object)">OhObject.setOwner(Object)</a> 
 * ݒ肵܂B
 * IuWFNg`IuWFNgãoϐłꍇAua`̏LҁvƌĂт܂B
 * Ⴆ΁A
 *
 * <pre>
 *  EM[q`vffãoϐłꍇB
 *  Evff`nfãoϐłꍇB
 * </pre>
 *
 * ̃oϐpāA
 *
 * <ul>
 *  <li> IuWFNg̔r@\
 *  <li> L҂̐ݒ@\
 *  <li> IuWFNg̏o͋@\
 * </ul>
 *
 * Ȃǂ`Ă܂B
 *
 * </japanese>
 *
 * <p>
 *
 * Base class of all OHyMoSJ classes. <p>
 *
 * An element of type Object has four basic fields: _owner, _className, _objectName, _objectNumber.
 *
 * <dl>
 *  <li> Definition of "_owner".
 *    <dd> When an object A has an object B as its field, A is called the owner of B. <br>
 *         The field "owner", a type Object,
 *         is the pointer to the owner of the current object.
 *
 *  <li> Definition of "_className"
 *   <dd> _className stores the name of the class. <br>
 *        For example, "_className" of an object of the class Object is "Object".
 *
 *  <li> Definition of "_objectName"
 *   <dd> _objectName stores the name of the objcet. <br>
 *        User descides the name of each object.
 *
 *  <li> Definition of "_objectNumber"
 *   <dd> _objectNumber stores the identification number.
 * </dl>
 *
 * _objectName and _objectNumber are used to identify OHyMoSJ elements owned by an OHyMoSJ element.
 * Suppose an OHyMoSJ element A owns OHyMoSJ elements e1, e2, ..., eN,
 * elements e1, e2, ..., eN should be able to be identified by the pair, _objectName and _objectNumber.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.36 $ $Date: 2004/02/12 05:56:35 $
 */
public class OhObject
{
  // @begin{fields} -------------------------------------------------------

  /**
   * <japanese>
   * NX\łB
   * </japanese>
   * <p>
   * '_className' stores the class name.
   *
   */
  protected String _className;
  
  /**
   * <japanese>
   * IuWFNg\łB
   * </japanese>
   * <p>
   * '_objetcName' stores the object name.
   *
   */
  protected String _objectName;
  
  /**
   * <japanese>
   * IuWFNgԍ\intlłB
   * </japanese>
   * <p>
   * '_objectNumber' stores the object number.
   *
   */
  protected int _objectNumber;
  
  /**
   * <japanese>
   * L҂\IuWFNgłB
   * </japanese>
   * <p>
   * The "owner" of the current object casted to type <code>Object</code>.
   *
   */
  protected Object _owner;

  /**
   * <japanese>
   * OS ɈˑȂsR[h\łB
   * </japanese>
   * <p>
   * New line.
   *
   */
  protected static String NEWLINE = System.getProperty("line.separator");
  
  // @end{fields} ---------------------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * ̓Iɂ́A aObjName, aObjNum  _objectName, _objectNumber ɐݒ肵܂B
   * ܂A_objectName ݒς݂̎́AdݒƔfAG[bZ[W\ false Ԃ܂B
   * ̏ꍇA_objectName, _objectNumber ύX܂B
   * RXgN^̒`AyуftHgRXgN^ŐIuWFNg̏ɗp܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum) throws OhError {
    if (aObjName == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    if (_objectName != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("\taObjName(arg) = \"" + aObjName + "\", aObjNum(arg) = " + aObjNum + NEWLINE);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _objectName   = aObjName;
    _objectNumber = aObjNum;

    return true;
  }

  // @end{initialization} ------------------------------------------------


  // @begin{constructors} ------------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^łB
   * oϐȉ̒ʂ菉܂B
   * <pre>
   *  E_className    : "OhObject"
   *  E_objectName   : null
   *  E_objectNumber : -1
   *  E_owner        : null
   * </pre>
   * </japanese>
   *
   */
  public OhObject() {
    _className    = "OhObject";
    _objectName   = null;
    _objectNumber = -1;
    _owner        = null;
  }


  /**
   * <japanese>
   * Ɏw肳ꂽoϐ OhObject 쐬܂B
   * <a href="OhObject.html#init(java.lang.String, int)">OhObject.init(String, int)</a>  aObjNum, aObjNum nA
   * oϐȉ̒ʂ菉܂B
   * <pre>
   *  E_className    : "OhObject"
   *  E_objectName   : aObjName
   *  E_objectNumber : aObjNum
   *  E_owner        : null
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhObject(String aObjName, int aObjNum) throws OhError {
    _className    = "OhObject";
    _objectName   = null;
    _objectNumber = -1;
    _owner        = null;

    init(aObjName, aObjNum);
  }

  // @end{constructors} --------------------------------------------------


  // @begin{methods} -----------------------------------------------------
  
  /**
   * <japanese>
   * NX擾܂B
   * </japanese>
   * <p>
   * Gets the class name.
   *
   * @return a <code>String</code> value
   */
  public String getClassName() {
    return _className;
  }

  /**
   * <japanese>
   * IuWFNg擾܂B
   * </japanese>
   * <p>
   * Gets the object name.
   *
   * @return a <code>String</code> value
   */
  public String getObjectName() {
    return _objectName;
  }

  /**
   * <japanese>
   * IuWFNgԍ擾܂B
   * </japanese>
   * <p>
   * Gets the object number.
   *
   * @return an <code>int</code> value
   */
  public int getObjectNumber() {
    return _objectNumber;
  }

  /**
   * <japanese>
   * L҂擾܂B
   * </japanese>
   * <p>
   * Gets the owner of the current object casted to type Object.
   *
   * @return an <code>Object</code> value
   */
  public Object getOwner() {
    return _owner;
  }

  /**
   * <japanese>
   * ōL҂擾܂B
   * </japanese>
   * <p>
   * Gets the top owner of the receiver.
   *
   * @return an <code>Object</code> value
   */
  public Object getTopOwner() {  //; (*5)
    Object top    = this;
    Object parent = _owner;

    while (parent != null) {
      top = parent;
      parent = ((OhObject)top).getOwner();
    }
    
    return top;
  }


  // 1. function about a name

  /**
   * <japanese>
   * NXAIuWFNgAIuWFNgԍɊւIuWFNg̔rsȂ܂B<p>
   *
   * ̃\bh́A_className, _objectName, _objectNumber 
   * aClassName, aObjName, aObjNum Ɉvꍇ true Ԃ܂B
   * ǂꂩPłvȂ̂ false Ԃ܂B
   * </japanese>
   * <p>
   * If the current object is same as class name, object name, and object number given by the arguments,
   * this method returns true.
   * Otherwise, false.
   *
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean objCompare(String aClassName, String aObjName, int aObjNum) {
    if ((_objectName.equals(aObjName))
      && (_objectNumber == aObjNum)
      && (_className.equals(aClassName)))
      return true;
    
    return false;
  }

  
  /**
   * <japanese>
   * IuWFNgAIuWFNgԍɊւIuWFNg̔rsȂ܂B<p>
   *
   * ̃\bh́A_objectName, _objectNumber 
   * aObjName, aObjNum Ɉvꍇ true Ԃ܂B
   * ꂩPłvȂ̂ false Ԃ܂B
   * </japanese>
   * <p>
   * If the current object is same as object name and object number as those given as the arguments,
   * this method returns true.
   * Otherwise, false.
   *
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean objCompare(String aObjName, int aObjNum) {
    if ((_objectName.equals(aObjName))
      && (_objectNumber == aObjNum))
      return true;
    
    return false;
  }


  /**
   * <japanese>
   * NXAIuWFNgAIuWFNgԍAL҂ɊւIuWFNg̔rsȂ܂B<p>
   *
   * ̃\bh́A_className, _objectName, _objectNumber, _owner 
   * ׂ aObj ƈvꍇ true Ԃ܂B
   * ǂꂩPłvȂ̂ false Ԃ܂B
   * ܂AaObj  null ̏ꍇ false Ԃ܂B
   * </japanese>
   * <p>
   * If the current object is same class name, object name, object number, and owner which aObj has,
   * this method returns true.
   * Otherwise, false.
   *
   *
   * @param aObj an <code>OhObject</code> value
   * @return a <code>boolean</code> value
   */
  public boolean objCompare(OhObject aObj) {
    if (aObj == null)
      return false;
    
    if ((aObj.getObjectName().equals(_objectName))
      && (aObj.getOwner() == _owner)
      && (aObj.getObjectNumber() == _objectNumber)
      && (aObj.getClassName().equals(_className)))
      return true;
    
    return false;
  }


  // 2. function to set up an owner.

  /**
   * <japanese>
   * aOwnerijL҂ɐݒ肵܂B<p>
   *
   * aOwner Ƃ͈قȂIuWFNgłɏL҂ƂĐݒ肳Ăꍇ́A
   * V aOwner L҂ɐݒ肹ɁA
   * O <a href="OhError.html">OhError</a> ܂B
   * </japanese>
   * <p>
   * Set owner to be "aOwner".
   *
   * When already a different object has been registerd as the owner,
   * you cannot set the new owner.
   * If this case happens, this method throws <a href="OhError.html">OhError</a> exception.
   *
   *
   * @param aOwner an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public void setOwner(Object aOwner) throws OhError {  //; (*6)
    if (aOwner == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }
    
    if (_owner != null && _owner != aOwner) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)_owner).oPrint(out, "    first : ");
      ((OhObject)aOwner).oPrint(out, "    next  : ");
      cPrint(out, "    ");
      String str = buffer.toString();
      out.close();
      throw new OhError("\"owner\" has already been set.", str);
    }

    _owner = aOwner;
  }


  // 3. Information output function
  
  /**
   * <japanese>
   * L҂̏o͂܂B
   * </japanese>
   * <p>
   * This method returns a string which specifies the object uniquely. <p>
   *
   * The string which getAbsName() returns will be created by concatenating
   *
   * <pre>
   *
   *   "absolute name of the owner"
   *   "/"
   *   _objectName
   *   "__"
   *   _objectNumber.
   *
   * </pre>
   *
   * When _owner == NULL, "absolute name of the owner" is taken as "". <p>
   *
   * <EXAMPLE> <p>
   *
   * Assume that the absolute name of the owner is "/total_sytetem__0",
   * _objectName of the current object is "elm_t",
   * and _objectNumber is 0,
   * then "absname" is
   *
   * <pre>
   *
   *   /total_system__0/elm_t__0
   *
   * </pre>
   *
   * @return a <code>String</code> value
   */
  public String getAbsName() {                 //; (*2)
    StringBuffer absName = new StringBuffer("");
    
    if (_owner == null) {
      absName = absName.append("");
    }
    else {
      absName = absName.append(((OhObject)_owner).getAbsName());
//;   ((OhObject)_owner).getAbsName();  owner ̏ʂɂȂB
    }

    absName = absName.append("/");
    absName = absName.append(_objectName);
    absName = absName.append("__");
    absName = absName.append((new Integer(_objectNumber)).toString());

    return absName.toString();    
  }

  
  /**
   * <japanese>
   * IuWFNg(L҂̏܂܂Ȃ)o͂܂B
   * </japanese>
   * <p>
   * Print "aHead", _className, _objectName, _objectNumer to aPw. <p>
   *
   * <EXAMPLE>
   *
   * <pre>
   * |  Yahoo a("yahoo", 0);
   * |  a.Cprint(aPw, "+");
   *
   * prints to aPw
   *
   *    +Yahoo yahoo 0
   * </pre>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void cPrint(PrintWriter aPw, String aHead) {                                 //; (*3)
    aPw.print(aHead + _className + " " + _objectName + " " + _objectNumber + NEWLINE);
  }

  
  /**
   * <japanese>
   * IuWFNg(L҂̏܂)o͂܂B
   * </japanese>
   * <p>
   * Trace back the object's ancestry, that is, prints all the ancesters to aPw as far as an ancester exists.
   * "aHead" is printed before the ancestry. <p>
   *
   * <EXAMPLE>
   *
   * <pre>
   * |  Yahoo a("yahoo", 0);
   * |  Yahoo b("yahoobb", 1);
   * |  b.setOwner(a);
   * |  PrintWriter aPw;
   * |  b.Oprint(aPw, "+ ");
   * 
   * prints to aPw as follows
   * 
   * |  + Yahoo yahoobb 1
   * |  +     owner > Yahoo yahoo 0
   * </pre>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void oPrint(PrintWriter aPw, String aHead) {  //; (*4)
    cPrint(aPw, aHead);

    final int[] k = new int[2];
    k[0] = 0;

    for(Object buf = _owner; buf != null; buf = ((OhObject)buf).getOwner()) {
      aPw.print(aHead + "    _owner ");
      for(k[1] = (++k[0]); k[1] > 0; k[1]--)
        aPw.print(">");
      ((OhObject)buf).cPrint(aPw, " ");
    }
  }

  // @end{methods} -------------------------------------------------------

} // OhObject
