/**
 * @(#) OhOutputFile; receive port model class "OhOutputFile" for output files.
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * <japanese>
 * o̓t@CpM[qf\NXłB<p>
 *
 * ŠnŎMf[^pbNt@CɋL^܂B<p>
 *
 * ۃNXłB
 * <a href="OhOutputPort.html">OhOutputPort</a> NXhA
 * <a href="OhOutputFileDouble.html">OhOutputFileDouble</a>,
 * <a href="OhOutputFileInt.html">OhOutputFileInt</a>,
 * <a href="OhOutputFilelong.html">OhOutputFileLong</a>,
 * <a href="OhOutputFileFloat.html">OhOutputFileFloat</a>,
 * NXh܂B
 * </japanese>
 * <p>
 * OhOutputFile;
 *
 * Model for output ports which receive datapacks and print them in a file. <p>
 *
 * Class OhOutputFile stores "output file name" in
 * <a href="OhObject.html#_objectName">OhObject._objectName</a>.
 * Methods to output received datapacks to the output file are defined as its methods.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.29 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhOutputFile extends OhOutputPort
{
  // @begin{fields} -------------------------------------------------------

  /**
   * <japanese>
   * o̓f[^̍ő咷łB<p>
   *
   * ̒́ÃIuWFNg̃RXgN^Ŏw肵̂łB
   * </japanese>
   * <p>
   * The maximum length of an output data string.
   *
   */
  protected int _lineLength;
  
  /**
   * <japanese>
   * ŏIo̓f[^̃f[^L^łB
   * </japanese>
   * <p>
   * Data record time of the last output data.
   *
   */
  protected OhTimePoint _lastDataTime;

  /**
   * <japanese>
   * t@CɏoRgłB
   * </japanese>
   * <p>
   * The comment string written out to a file.
   *
   */
  protected String _commentString;

  // @end{fields} ---------------------------------------------------------


  // @begin {abstract methods} -------------------------------------------

  // 1. Connection function.

  /**
   * <japanese>
   * o̓f[^pbÑf[^ľ^𕶎ƂĕԂ܂B<p>
   *
   * ۃ\bhłB<p>
   * </japanese>
   * <p>
   * This is an abstract method. 
   * Returns a string which expresses the type of datapacks received by the current object. 
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();


  // 2. The write-in function of output files.

  /**
   * <japanese>
   * o̓f[^Ps̍쐬s܂B<p>
   *
   * ۃ\bhłB<p>
   *
   * o̓t@CPsɂ́AP̃f[^pbÑf[^eLq܂B<p>
   * </japanese>
   * <p>
   * Writes the content of datapack aDataPack into the area to which aStr points.
   * The space which aStr points to must be large enough to store the content of the datapack. <p>
   *
   * Since this is an abstract method,
   * implementation of this function is compulsory in derived classes.
   *
   * @param aStr a <code>String</code> value
   * @param aDataPack an <code>Object</code> value
   */
  public abstract void makeStringData(String aStr, Object aDataPack);

  // @end{abstract methods} ----------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNg(t@C)Ao̓f[^̍ő咷(_lineLength)A
   * IuWFNgԍ 0 ɐݒ肵܂B<p>
   *
   * _lineLength  0 傫ꍇAdݒƔf false Ԃ܂B
   * _lineLength  0 ȉȂ΁A_lineLength  aLineLength ɐݒ肵 true Ԃ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @param aLineLength an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aFileName, int aLineLength) throws OhError {
    if (aLineLength < 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("Invalid value of aLineLength " + aLineLength + NEWLINE);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    if (aLineLength == 0)
      aLineLength = 128;


    if ( ((super.init(aFileName, 0)) != true) || _lineLength > 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _lineLength = aLineLength;

    return true;
  }


  /**
   * <japanese>
   * IuWFNg(t@C)A
   * o̓f[^̍ő咷(_lineLength) 128 ɁAIuWFNgԍ 0 ɐݒ肵܂B<p>
   *
   * _lineLength  0 傫ꍇAdݒƔf false Ԃ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aFileName) throws OhError {
    if ( ((super.init(aFileName, 0)) != true) || _lineLength > 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _lineLength = 128;

    return true;
  }

  // @end{initialization} ------------------------------------------------


  // @begin{constructors} ------------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^
   * <a href="OhOutputFile.html#init(java.lang.String, int)">OhOutputFile.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   *  <li> _lineLength        : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhOutputFile.html#init(java.lang.String, int)">OhOutputFile.init(String, int)</a> 
   * păoϐĉB
   * </jananese>
   *
   */
  public OhOutputFile() {
    super();
    _lineLength = -1;

    _className     = "OhOutputFile";
    _commentString = null;
    _lastDataTime  = OhTimePoint.OhTimePointMin;
  }


  /**
   * <japanese>
   * NX <a href="OhOutputPort.html">OhOutputPort</a> ̃ftHgRXgN^pA
   * <a href="OhOutputFile.html#init(java.lang.String, int)">OhOutputFile.init(String, int)</a>
   * pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhOutputFile"
   *  <li> _objectName        : aFileName
   *  <li> _objectNumber      : 0
   *  <li> _lineLength        : aLineLength
   *  <li> _lastDataTime      : <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a>
   *  <li> _commentString     : null
   * </dl>
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @param aLineLength an <code>int</code> value
   */
  public OhOutputFile(String aFileName, int aLineLength) throws OhError {
    super();
    _lineLength = -1;

    _className     = "OhOutputFile";
    _commentString = null;
    _lastDataTime  = OhTimePoint.OhTimePointMin;

    init(aFileName, aLineLength);
  }


  /**
   * <japanese>
   * NX <a href="OhOutputPort.html">OhOutputPort</a> ̃ftHgRXgN^pA
   * <a href="OhOutputFile.html#init(java.lang.String)">OhOutputFile.init(String)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhOutputFile"
   *  <li> _objectName        : aFileName
   *  <li> _objectNumber      : 0
   *  <li> _lineLength        : -1
   *  <li> _lastDataTime      : <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a>
   *  <li> _commentString     : null
   * </dl>
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhOutputFile(String aFileName) throws OhError {
    super();
    _lineLength = -1;

    _className     = "OhOutputFile";
    _commentString = null;
    _lastDataTime  = OhTimePoint.OhTimePointMin;

    init(aFileName);
  }

  // @end{constructors} --------------------------------------------------

  
  // @begin{methods} -----------------------------------------------------

  /**
   * <japanese>
   * o̓f[^̍ő咷 _lineLength Ԃ܂B<p>
   *
   * ̒́ÃIuWFNg̃RXgN^Ŏw肵̂łB
   * </japanese>
   * <p>
   * Returns "_lineLength".
   *
   * @return an <code>int</code> value
   */
  public int getLineLength() {
    return _lineLength;
  }


  /**
   * <japanese>
   * ŏIo̓f[^̃f[^L^ _lastDataTime Ԃ܂B
   * </japanese>
   * <p>
   * Returns "_lastDataTime".
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getLastDataTime() {
    return _lastDataTime;
  }


  // 1. Connection function.

  // 2. The write-in function of output files.

  /**
   * <japanese>
   * t@CɏoRg _commentString  aCmt ɐݒ肵܂B
   * </japanese>
   * <p>
   * Copys the string "aCmt" to the field "_commentString".
   * The string copied to "_commentString" will be printed out into the output file,
   * while the method "writeHeader()" is executed.
   * See the source code of "writeHeader()".
   *
   * @param aCmt a <code>String</code> value
   */
  public void setCommentString(String aCmt) {    //; (*1)
    _commentString = aCmt;
  }


  /**
   * <japanese>
   * o̓f[^݂܂B<p>
   *
   * o̓t@C̃wb_ƂāA
   * ^f[^pbN𑗐MĂM[qɊւL܂B<p>
   *
   * ܂Af[^Ԍ^z̒l\܂B<p>
   *
   * yz
   * <pre>
   *   # tplkin.oa 
   *   # --------------------------------------------------------------------
   *   #   OhSendDouble sp 0 
   *   #       _owner > PowerLawKinematicWave stream 0
   *   #       _owner >> OhScfTotalSystem totalSystem 0
   *   # --------------------------------------------------------------------
   *   # DataConnectionType = 0:BROKENLINE 1:STEP 2:PULSE 3:NOINTP  
   *   #& dctVec 1  0
   * </pre>
   *
   * wb_ɂ͍s # ݁ARgAEĝŁA
   * ̏o̓t@CĂѓ̓t@CƂėp邱Ƃł܂B<p>
   * 
   * o̓t@CI[vłȂƗO𑗏o܂B<p>
   * 
   * wb_ύXꍇ́AhNXōĒ`ĉB<p>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɋLłꍇB
   *  Efalse -> o̓t@CI[vłȂꍇB
   * </pre>
   * </japanese>
   *
   * <p>
   *
   * Writes a header part to output file. The header contains <p>
   * <ul>
   *  <li> file name
   *  <li> information about the send port which sends data to this port
   *  <li> "_commentString if the "commentString" of the send port is not null
   * </ul>
   * <p>
   * If the output file cannot be opened, this method returns "false".
   *
   * @return a <code>boolean</code> value
   * @exception IOException if an error occurs
   */
  public boolean writeHeader() throws IOException {  //; (*2)
    
    BufferedWriter bw = null;
    
    try {
      bw = new BufferedWriter(new FileWriter(_objectName));
    }
    catch(IOException e) {
      throw e;
    }

    PrintWriter pw = new PrintWriter(bw, true);    //; "true" is autoFlush.

    pw.print("# " + _objectName + " " + NEWLINE + "# ");

    for (int k = 0; k < 70; k++)
      pw.print("-");

    pw.print(NEWLINE);

    ((OhObject)_sender).oPrint(pw, "#   ");

    pw.print("# ");

    for (int k = 0; k < 70; k++)
      pw.print("-");

    pw.print(NEWLINE);

    if (_commentString != null)
      pw.print("# " + _commentString + NEWLINE);
    
    String senderCom = ((OhSenderPort)_sender).getCommentString();

    if (senderCom != null)
      pw.print("# " + senderCom + NEWLINE);
    
    Vector dctVec = ((OhSenderPort)_sender).getDctVec();

    int n = dctVec.size();

    pw.print("# DataConnectionType = ");

    for (int i = 0; i < DataConnectionType.getNumOfDCT(); i++) {
      pw.print(i + ":" + DataConnectionType.getDctTypeStr(i) + " ");
    }

    pw.print(NEWLINE + "#& dctVec " + n + " ");

    for (int i = 0; i < n; i++) {
      if ((DataConnectionType)dctVec.elementAt(i) == DataConnectionType.BROKENLINE)
        pw.print(" " + 0);
      else if ((DataConnectionType)dctVec.elementAt(i) == DataConnectionType.STEP)
        pw.print(" " + 1);
      else if ((DataConnectionType)dctVec.elementAt(i) == DataConnectionType.PULSE)
        pw.print(" " + 2);
      else
        pw.print(" " + 3);
    }  

    pw.print(NEWLINE);
    pw.print(NEWLINE);

    pw.close();

    return true;
  }


  /**
   * <japanese>
   * o̓f[^݂܂B<p>
   *
   * ܂Ao̓t@C _objectName ǉ[hŃI[vA
   * I[vłȂꍇ return ܂B<p>
   * 
   * ɁAMf[^pbNz _dataPackArray ̐擪Ɋi[Ăf[^pbN珇Ԃ
   * <a href="OhOutputFile.html#makeStringData(java.lang.String, java.lang.Object)">
     OhOutputFile.makeStringData(String, Object)</a> 
   * păf[^쐬Ao̓t@CɋL܂B<p>
   * 
   * Ӂ <p>
   *
   * Mf[^pbN͂̃\bh̒Ŏ擾܂B
   * f OhOutputFile ͎M[q̔hfłāA
   * f[^pbNXgbN@\͂܂B
   * </japanese>
   * <p>
   * Writing of output data.
   *
   * @exception IOException if an error occurs
   */
  public void outputData() throws IOException {    //; (*3)
    
    OhArrayList   dpArray      = new OhArrayList();
    OhArrayList[] dpArrayArray = {dpArray};

    int   numOfDps      = 0;
    int[] numOfDpsArray = {numOfDps};

    getDataPackArray(dpArrayArray, numOfDpsArray);

    dpArray  = dpArrayArray[0];
    numOfDps = numOfDpsArray[0];

    // _dataPackArray is not updated.
    //
    if (((OhDataPack)dpArray.get(numOfDps - 1)).getDataTime().compareTo(_lastDataTime) <= 0)
      return;    

    String buffer = "";

    /* initial output */

    if (_lastDataTime == OhTimePoint.OhTimePointMin)
      writeHeader();

    ////////////////////////////////////////////////
    // Additional writing.
    //
    // <japanese>
    // ݗpo̓t@C̒ǉI[vB
    // </japanese>
    //
    BufferedWriter bw = null;
    
    try {
      bw = new BufferedWriter(new FileWriter(_objectName, true));
    }
    catch(IOException e) {
      throw e;
    }

    PrintWriter pw = new PrintWriter(bw);  

    if (((OhDataPack)dpArray.get(0)).getDataTime().compareTo(OhTimePoint.OhTimePointMin) == 0) {
      makeStringData(buffer, dpArray.get(0));
      pw.print(buffer + NEWLINE);
    }

    
    int k = 0;

    for (k = 0; k < numOfDps; k++) {
      if (((OhDataPack)dpArray.get(k)).getDataTime().compareTo(_lastDataTime) > 0)
        break;
    }

    for (; k < numOfDps; k++) {
      makeStringData(buffer, dpArray.get(k));
      pw.print(buffer + NEWLINE);
    }

    pw.close();

    _lastDataTime = ((OhDataPack)dpArray.get(numOfDps - 1)).getDataTime();

    reduceDataPackStock();
  }


  /**
   * <japanese>
   * f[^KvԂ܂B<p>
   *
   * M̑M[q getNecessaryTimeFrom() ȍ~
   * getNecessaryTimeTo ȑÕf[^pbNȂ悤Ɏw邽߂ɒ`܂B<p>
   * 
   * <pre>
   * [RETURN]
   *  EOhTimePoint f[^Kv
   * </pre>
   * </japanese>
   * <p>
   * Returns necessary data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeFrom() {
    return _lastDataTime;
  }


  /**
   * <japanese>
   * f[^KvԂ܂B<p>
   *
   * M̑M[q getNecessaryTimeFrom() ȍ~
   * getNecessaryTimeTo ȑÕf[^pbNȂ悤Ɏw邽߂ɒ`܂B<p>
   * 
   * <pre>
   * [RETURN]
   *  EOhTimePoint f[^Kv
   * </pre>
   * </japanese>
   * <p>
   * Returns necessary data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeTo() {
    return new OhTimePoint(2100, 0);
  }


  // 3. Information output function.

  /**
   * <japanese>
   * IuWFNg(L҂̏܂܂Ȃ)o͂܂B<p>
   *
   * o̓t@CƂ̒[q̎MM[q̏
   * Xg[ aPwijɏo͂܂B
   *
   * <pre>
   *  [in] aPw  : IuWFNgo͂Xg[
   *  [in] aHead: Rg
   * </pre>
   * </japanese>
   * <p>
   * Outputs an object information.
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void cPrint(PrintWriter aPw, String aHead) {  //; (*4)
    aPw.print(aHead + "OhInputFile : " + _className + "<" + dataType() + "> " + _objectName + " " + NEWLINE);
  }


  /**
   * <japanese>
   * IuWFNg(L҂̏܂)o͂܂B<p>
   *
   * o̓t@CƂ̒[q̎MM[q̏
   * Xg[ aPwijɏo͂܂B
   *
   * <pre>
   *  [in] aPw  : IuWFNgo͂Xg[
   *  [in] aHead: Rg
   * </pre>
   * </japanese>
   * <p>
   * Outputs an object information.
   * The information includes the "_owner"'s information.
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void oPrint(PrintWriter aPw, String aHead) {  //; (*5)
    cPrint(aPw, aHead);

    if (_sender == null)
      return;


    // Makes the header when outputting the information on the input port.
    //
    String newHead = aHead + "  output < ";

    ((OhObject)_sender).oPrint(aPw, newHead);
  }

  // @end{methods} -------------------------------------------------------

} // OhOutputFile
