/**
 * @(#) OhOutputFileLong; receives datapacks of type long and write them into a file.
 *                        Make "OhOutputFileLong.java" from "ohp_tmp.h" and "ohp_tmp.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * f[^l long ^łf[^pbNt@CɋL^o͒[q\NXłB <p>
 * </japanese>
 * <p>
 * OhOutputFileLong;
 *
 * OhOutputFileLong is a class of output ports
 * which receive datapacks of type long and print them in a file. <p>
 *
 * OhOutputFileLong receives datapacks of type long.
 * OhOutputFileLong can be connected with
 * <a href="OhSendLong.html">OhSendLong</a>,
 * <a href="OhRelayLong.html">OhRelayLong</a>, and 
 * <a href="OhInputFileLong.html">OhInputFileLong</a>.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.2 $ $Date: 2004/02/12 05:29:42 $
 */
public class OhOutputFileLong extends OhOutputFile
{
  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNg(t@C)Ao̓f[^̍ő咷(_lineLength)A
   * IuWFNgԍ 0 ɐݒ肵܂B<p>
   *
   * _lineLength  0 傫ꍇAdݒƔf false Ԃ܂B
   * _lineLength  0 ȉȂ΁A_lineLength  aLineLength ɐݒ肵 true Ԃ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @param aLineLength an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aFileName, int aLineLength) throws OhError {
    return super.init(aFileName, aLineLength);
  }

  
  /**
   * <japanese>
   * IuWFNg(t@C)A
   * o̓f[^̍ő咷(_lineLength) 128 ɁAIuWFNgԍ 0 ɐݒ肵܂B<p>
   *
   * _lineLength  0 傫ꍇAdݒƔf false Ԃ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aFileName a <code>String</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aFileName) throws OhError {
    return super.init(aFileName, 128);
  }

  // @end{initialization} ------------------------------------------------
  

  // @begin{constructors} ------------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^
   * NX <a href="OhOutputFile.html">OhOutputFile</a> ̃ftHgRXgN^s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   *  <li> _lineLength        : -1
   * </dl>
   * <p>
   *
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA
   * <a href="OhOutputFileLong.html#init(java.lang.String, int)">OhOutputFileLong.init(String, int)</a> 
   * păoϐĉB
   * </japanese>
   *
   */
  public OhOutputFileLong() {
    super();
    
    _className = "OhOutputFileLong";
  }
  
    
  /**
   * <japanese>
   * NX̃RXgN^
   * <a href="OhOutputFile.html#OhOutputFile(java.lang.String, int)">OhOutputFile.OhOutputFile(String, int)</a>
   * păoϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhOutputFileLong"
   *  <li> _objectName        : aFileName
   *  <li> _objectNumber      : 0
   *  <li> _lineLength        : aLineLength
   *  <li> _lastDataTime      : <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a>
   *  <li> _commentString     : null
   * </dl>
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @param aLineLength an <code>int</code> value
   */
  public OhOutputFileLong(String aFileName, int aLineLength) throws OhError {
    super(aFileName, aLineLength);

    _className = "OhOutputFileLong";
  }


  /**
   * <japanese>
   * NX̃RXgN^
   * <a href="OhOutputFile.html#OhOutputFile(java.lang.String, int)">OhOutputFile.OhOutputFile(String, int)</a>
   * păoϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhOutputFileLong"
   *  <li> _objectName        : aFileName
   *  <li> _objectNumber      : 0
   *  <li> _lineLength        : 128
   *  <li> _lastDataTime      : <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a>
   *  <li> _commentString     : null
   * </dl>
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   */
  public OhOutputFileLong(String aFileName) throws OhError {
    super(aFileName, 128);

    _className = "OhOutputFileLong";
  }

  // @end{constructors} --------------------------------------------------

  
  // @begin{methods} -----------------------------------------------------

  // 1. Connection function.
  
  /**
   * <japanese>
   * o̓f[^pbÑf[^ľ^ long 𕶎 "long" ƂĕԂ܂B
   * </japanese>
   * <p>
   * Returns the datapack type, "long", which expresses the type of data to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "long";
  }


  // 2. The write-in function of output files.

  /**
   * <japanese>
   * Ȃ悤ɒ`Ă܂B 
   * </japanese>
   * Nothing is treated.
   *
   * @param aStr a <code>String</code> value
   * @param aDataPack an <code>Object</code> value
   */
  public void makeStringData(String aStr, Object aDataPack) {}


  /**
   * <japanese>
   * o̓f[^݂܂B<p>
   *
   * ܂Ao̓t@C _objectName ǉ[hŃI[vA
   * I[vłȂꍇ return ܂B<p>
   * 
   * ɁAMf[^pbNz _dataPackArray ̐擪Ɋi[Ăf[^pbN珇Ԃ
   * <a href="OhDataPackLong.html#dpack2Ofile(java.io.PrintWriter)">OhDataPackLong.dpack2Ofile(PrintWriter)</a> 
   * pďo̓t@CɋL܂B<p>
   * 
   *  <p>
   *
   * Mf[^pbN͂̃\bh̒Ŏ擾܂B
   * f OhOutputFile ͎M[q̔hfłāA
   * f[^pbNXgbN@\͂܂B
   * </japanese>
   * <p>
   * Though this method is defined in the base class
   * <a href="OhOutputFile.html">OhOutputFile</a>,
   * it is redifined in
   * <a href="OhOutputFileLong.html">OhOutputFileLong</a>.
   *
   * @exception IOException if an error occurs
   */
  public void outputData() throws IOException {
    
    OhArrayList   dpArray      = new OhArrayList();
    OhArrayList[] dpArrayArray = {dpArray};        //; For work.

    int   numOfDps      = 0;
    int[] numOfDpsArray = {numOfDps};  //; For work.

    getDataPackArray(dpArrayArray, numOfDpsArray);

    dpArray  = dpArrayArray[0];
    numOfDps = numOfDpsArray[0];

    
    // _dataPackArray is not updated.
    //
    if (((OhDataPack)dpArray.get(numOfDps - 1)).getDataTime().compareTo(_lastDataTime) <= 0)
      return;    


    /* first time output */

    if (_lastDataTime == OhTimePoint.OhTimePointMin)
      writeHeader();

    ////////////////////////////////////////////////
    // Additional writing.
    //
    // <japanese>
    // ݗpo̓t@C̒ǉI[vB
    // </japanese>
    //
    BufferedWriter bw = null;
    
    try {
      bw = new BufferedWriter(new FileWriter(_objectName, true));
    }
    catch(IOException e) {
      throw e;
    }


    PrintWriter pw = new PrintWriter(bw);  

    if (((OhDataPack)dpArray.get(0)).getDataTime().compareTo(OhTimePoint.OhTimePointMin) == 0) {
      ((OhDataPack)dpArray.get(0)).dpack2Ofile(pw);
      pw.print(NEWLINE);
    }


    int k = 0;

    // Skip the data which have been output.
    //
    for (k = 0; k < numOfDps; k++) {
      if (((OhDataPack)dpArray.get(k)).getDataTime().compareTo(_lastDataTime) > 0)
        break;
    }

    
    /* output the data */

    for (; k < numOfDps; k++) {
      ((OhDataPack)dpArray.get(k)).dpack2Ofile(pw);
      pw.print(NEWLINE);
    }

    pw.close();

    _lastDataTime = ((OhDataPack)dpArray.get(numOfDps - 1)).getDataTime();

    reduceDataPackStock();
  }

  // @end{methods} -------------------------------------------------------

} // OhOutputFileLong
