/**
 * @(#) OhOutputPort; base class for creating output ports.
 *                    Make "OhOutputPort.java" from "ohoutput.h" and "ohoutput.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^o͒[qf\NXłB<p>
 * 
 * Šn̏o͒[q쐬邽߂̊NXłB <p>
 * ۃNXłB<p>
 *
 * {^o͒[q́AŠn̏o͒[q쐬邽߂̒ۃNXłB
 * [ÚÃNXpđŠn̏o͒[q̃f쐬ĂB
 * {^o͒[q̃NX錾ł́A[Uׂo͒[q̊{dl߂Ă܂B
 * ܂A̎ł͎Oɗpӂł@\`Ă܂B<p>
 *
 * Šnf́Å{^o͒[q̃NXpă[U쐬
 * o͒[qʂăf[^[Uɒ񋟂̂ƂĐ݌vĂ܂B<p>
 *
 * o͒[qo̓f[^[Uɗ^ƂӖł́A
 * o͒[q́A{^vff̑M[qɎĂ܂ASʂ̂̂
 * l悢łB<p>
 *
 * Šnf́At@CóAlbg[NɂóAL[{[h́A
 * CRT o͂ȂǁAXGȃf[^̓o͂ɑΉȂ΂Ȃ܂B
 * Šnf́AGȃf[^n̓Ɏ荞ގ́A
 * S <a href="OhDataPack.html">OhDataPack</a> ^̃f[^ɕϊȂ΂Ȃ܂B
 * ܂AV~[VʂV~[Vs҂ɒ񋟂鎞́A
 * <a href="OhDataPack.html">OhDataPack</a> ^Œ~ςĂf[^
 * V~[Vs҂̖]ރf[^`ɕϊȂ΂Ȃ܂B
 * ̂悤ȃf[^̕ϊɊւ̓ÍA
 * [U쐬͒[qEo͒[qōsȂƂɂA
 * Šnf̕ł́A͒[qEo͒[q̎葱ő삷邱Ƃɂ܂B
 * Ȃ킿A͒[qEo͒[q́A
 * ̒^IȏƊȌƂ̒邱ƂɂȂ܂B
 *
 * </japanese>
 * <p>
 * OhOutputPort;
 *
 * Base class for creating output ports. <p>
 * This is a abstract class.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.15 $ $Date: 2004/02/12 05:56:35 $
 */
public abstract class OhOutputPort extends OhReceivePort
{
  // @begin {abstract methods} -------------------------------------------

  // 1. Connection function.

  /**
   * <japanese>
   * o̓f[^pbÑf[^ľ^𕶎ƂĕԂ܂B<p>
   *
   * ۃ\bhłB<p>
   * </japanese>
   * <p>
   * Returns a string which expresses the type of datapacks received by the current object.
   * This is a abstract method.
   * This function must be defined in derived classes.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();


  // 2. The function which outputs receiving data.

  /**
   * <japanese>
   * ŠnŎMf[^pbNA
   * [U̖]ތ`ɕϊđŠnOɏo͂܂B<p>
   *
   * ۃ\bhłB<p>
   *
   * eo͒[qŎMf[^pbN쐬A
   * f[^o͂郁\bh`ĉB
   * </japanese>
   * <p>
   * Gets datapacks from the sender and output it.
   * This is a abstract method declaration.
   * It must be implemented in derived classes.
   *
   * @exception IOException if an error occurs
   */
  public abstract void outputData() throws IOException;

  // @end {abstract methods} ---------------------------------------------

  
  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------


  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^
   * <a href="OhOutputPort.html#init(java.lang.String, int)">OhOutputPort.init(String, int)</a>
   * s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA
   * <a href="OhOutputPort.html#init(java.lang.String, int)">OhOutputPort.init(String, int)</a>
   * păoϐĉB
   * </jananese>
   *
   */
  public OhOutputPort() {
    super();
    _className = "OhOutputPort";
  }

  /**
   * <japanese>
   * NX <a href="OhReceivePort.html">OhReceivePort</a> ̃ftHgRXgN^pA
   * <a href="OhOutputPort.html#init(java.lang.String, int)">OhOutputPort.init(String, int)</a>
   * pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhOutputPort"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _owner             : null
   *  <li> _sender            : null
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public OhOutputPort(String aPortName, int aPortNum) throws OhError {
    super();
    _className = "OhInputPort";
    init(aPortName, aPortNum);
  }

  // @end {constructors} -------------------------------------------------

} // OhOutputPort
