/**
 * @(#) OhPort; class to express a port
 *              Make "OhPort.java" from "ohport.h" and "ohport.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^[qf\NXłB <p>
 *
 * [q쐬邽߂̊ƂȂ钊ۃNXłB <p>
 *
 * NX OhPort ́ASĂ̒[qf̊NXłB
 * [qׂ{Iȃ\bh`Ă܂B
 * \bḧꕔ́ÃNXł͒`^Aۃ\bhɂȂĂ܂B
 * </japanese>
 *
 * <p>
 *
 * OhPort; <p>
 *
 * OhPort is the base class from which all port classes are derived. <p>
 *
 * We call an object of class OhPort or classes inherited from class Port a "port". <br>
 * In ohymosj, a port is used for sending and/or receiving datapacks. <p>
 *
 * Basic methods which a port should have are declared. <br>
 * Parts of those basic methods are declared as pure virtual.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.58 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhPort extends OhObject 
{
  // @begin{abstract methods} ---------------------------------------------

  /**
   * <japanese>
   * f[^̌^Ԃ܂B<p>
   *
   * ۃ\bhłB<p>
   * 
   * ̃\bh̎́A
   * <a href="OhSendDouble.html">OhSendDouble</a>,
   * <a href="OhInputFileDouble.html">OhInputFileDouble</a>,
   * <a href="OhRelayDouble.html">OhRelayDouble</a>,
   * <a href="OhReceiveDouble.html">OhReceiveDouble</a>,
   * <a href="OhOutputFileDouble.html">OhOutputFileDouble</a>
   * ŗ^邱Ƃɂ܂B
   * int, long, float ɂĂlłB
   *
   * Ⴆ΁AMf[^ int Ȃ "int" Ԃ悤ɒ`܂B
   * </japanese>
   * <p>
   *
   * Abstract method. <p>
   *
   * This method returns string DataType which shows the type of datapacks treated by the port. <p>
   * 
   * Because this method is declared as abstract,
   * in a derived class of class OhPort
   * the definition of this method must be given by a programmer.
   *
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();

  
  /**
   * <japanese>
   * vZƑO̊mFsȂ܂B <p>
   *
   * ۃ\bhłB <p>
   *
   * ڑƌAvZƑO̊mFɗp܂B
   * ڑƂIAvZƂɐiނƂłꍇ true ԂA
   * ڑƂIĂȂꍇ false Ԃ܂B
   *
   * <pre>
   * RETURN:
   *   Etrue   ڑƂIĂB
   *   Efalse  ڑƂIĂȂB
   * </pre>
   * </japanese>
   * <p>
   *
   * Confirm whether all connection work has been done successfully or not. <p>
   * 
   * In class <a href="OhPort.html">OhPort</a>,
   * which is an abstract class, method areYouReady() is an abstract method.
   *
   * @return a <code>boolean</code> value
   */
  public abstract boolean areYouReady();

  
  // 1. Function for acquiring datapacks.
  
  /**
   * <japanese>
   * f|^pbN̔z񃊃Xgݒ肵܂B<p>
   *
   * ۃ\bhłB<p>
   *
   * ۂɂ́A
   * _dataPackArray ̗vf _numberOfDataPacks ɐݒ肷邾łB
   * </japanese>
   * <p>
   * Abstract method. This method makes data pack array.
   *
   */
  public abstract void setDataPackArray();


  /**
   * <japanese>
   * 擾\f[^pbN̊mFsȂ܂B<p>
   *
   * ۃ\bhłB<p>
   *
   * canYouGetData(OhTimePoint aTFrom, OhTimePoint aTTo) , aTFrom ȑOA
   * aTTo ȍ~܂ł̃f[^pbN̒[q擾\Ȃ΁A
   * true ԂAs\Ȃ false Ԃ悤ɊehNXŒ`܂.
   *
   * <pre>
   * ARGUMENT:
   *   EaTFrom    aTFrom ȑÕf[^pbN擾\ł邱ƂmF.
   *   EaTTo      aTTo ȍ~܂ł̃f[^pbN擾\ł邱ƂmF.
   * RETURN:
   *   EYES       aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN擾\ȏꍇ.
   *               t ɂf[^擾\ȏꍇ.
   *   ENO        aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN擾s\ȏꍇ.
   * </pre>
   * </japanese>
   * <p>
   * Inquire whether the invoking port can get data from "aTFrom" to "aTTo".
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   * @param aTTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public abstract boolean canYouGetData(OhTimePoint aTFrom, OhTimePoint aTTo);
  

  /**
   * <japanese>
   * f[^pbN̔z񃊃Xg擾(M)܂B<p>
   *
   * hNXɂāAf[^pbNz擾郁\bh`܂B
   * aDpArray ɂ̓f[^pbNzi[AaNumOfDp ɂ́AaDpArray ̗vfi[܂B
   * </japanese>
   * <p>
   * Get a datapack array.
   *
   * @param aDpArray an <code>OhArrayList[]</code> value
   * @param aNumOfDp an <code>int[]</code> value
   */
  public abstract void getDataPackArray(OhArrayList[] aDpArray, int[] aNumOfDp);


  /**
   * <japanese>
   * ŌẪf[^̋L^Ԃ܂B
   * </japanese>
   * <p>
   * This method returns the oldest data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public abstract OhTimePoint getOldestDataTime();

  /**
   * <japanese>
   * ŐṼf[^̋L^Ԃ܂B
   * </japanese>
   * <p>
   * This method returns the updated data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public abstract OhTimePoint getNewestDataTime();

  /**
   * <japanese>
   * f[^pbŇԂ܂B
   * </japanese>
   * <p>
   * This method returns the number of data packs.
   *
   * @return an <code>int</code> value
   */
  public abstract int getNumberOfDataPacks();


  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * hNXɂāAw肵ăf[^pbN擾郁\bh`܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbN擾Ai[܂B<br>
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbN擾Ai[܂B<p>
   *
   * aFlag ͎擾̃[hI邽߂̃IvVŁAL̒ʂłB
   *
   * <pre>
   *   E"b"       aFormer, aLater ̗K擾B
   *               擾łȂꍇ́AƂ NULL ɐݒ肷B
   *   E"f"       aLater 擾łȂĂAaFormer ̂ݎ擾B
   *               aLater 擾łȂꍇANULL ɐݒ肷B
   *   E"l"       aFormer 擾łȂĂAaLater ̂ݎ擾B
   *               aFormer 擾łȂꍇANULL ɐݒ肷B
   *   Ȇ    aFormer, aLater ̂ȂƂ擾B
   *               擾łȂ̂ NULL ɐݒ肷B
   * </pre>
   *
   * aFlag ̃ftHgĺA "b" łB<p>
   * ܂A  aLater nȂꍇ́A aFlag  f Ƃ aFormer ̂ݎ擾A
   *  aFormer nȂꍇ́A aFlag  l Ƃ aLater ̂ݎ擾܂B<p>
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B<p>
   *
   * 𖞂f[^pbN擾łȂꍇAaFormer, aLater  null ɐݒ肳܂B<p>
   *
   * f[^擾łꍇ́Atrue Ԃ܂B
   * </japanese>
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @param aMode a <code>char</code> value
   * @return a <code>boolean</code> value
   */
  public abstract boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater, char aMode);


  /**
   * <japanese>
   * f[^KvԂ܂B<p>
   *
   * getNecessaryTimeFrom f[^L^f[^pbŃA
   * <a href="OhPort.html#reduceDataPackStock()">OhPort.reduceDataPackStock()</a>
   * ɂ܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public abstract OhTimePoint getNecessaryTimeFrom();


  /**
   * <japanese>
   * f[^KvԂ܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public abstract OhTimePoint getNecessaryTimeTo();
 

  /**
   * <japanese>
   * svf[^pbN܂B<p>
   *
   * ̒[qŎ擾\ȃf[^pbN̂A
   * getNecessaryTimeFrom() f[^L^f[^pbN悤A
   * hNXŒ`܂B
   * </japanese>
   *
   */
  public abstract void reduceDataPackStock();
  
  // @end{abstract methods} ----------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B<p>
   *
   * RXgN^Ŏgp܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------


  // @begin{constructors} -------------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^łB
   * oϐȉ̒ʂ菉܂B
   * <pre>
   *  E_className    : "OhPort"
   *  E_objectName   : null
   *  E_objectNumber : -1
   *  E_owner        : null
   * </pre>
   * </japanese>
   *
   */
  public OhPort() {
    super();
    _className = "OhPort";
  }

  /**
   * <japanese>
   * Ɏw肳ꂽoϐ OhPort 쐬܂B
   * <a href="OhPort.html#init(java.lang.String, int)">OhPort.init(String, int)</a>
   *  aObjNum, aObjNum nA
   * oϐȉ̒ʂ菉܂B
   * <pre>
   *  E_className    : "OhPort"
   *  E_objectName   : aObjName
   *  E_objectNumber : aObjNum
   *  E_owner        : null
   * </pre>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNumber an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhPort(String aPortName, int aPortNumber) throws OhError {
    super();
    _className = "OhPort";
    init(aPortName, aPortNumber);
  }

  // @end{constructors} ---------------------------------------------------


  // @begin{methods} ------------------------------------------------------
  
  /**
   * <japanese>
   * M[qo^܂B<p>
   *
   * ̃NX̔hNXł́A
   * Object ^ɃLXgꂽM[q̃AhX aSender ƂĎ󂯂ƂA
   * aSender 𑗐MƂĐݒł true AݒłȂ false Ԃ悤`ĂB<p>
   *
   * ̃\bh́AG[bZ[Wo͌ false Ԃ܂B
   * hNXŁAsetSender() Ē`Ȃꍇ́AꂪAdefault ̒`ɂȂ܂B
   * </japanese>
   * <p>
   *
   * Set the sender.
   * There is only one sender for each receiver.
   * This method returns whether it succeeded or not. <p>
   * 
   * Although a default definition is given in the Port class,
   * a programmer must give his own definitions in derived classes.
   *
   * @param aSender an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean setSender(Object aSender) throws OhError {    //; (*1)
    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);
    oPrint(out, " > ");

    if(aSender != null)
      ((OhObject)aSender).oPrint(out, " >   sender > ");

    String str = buffer.toString();
    out.close();

    throw new OhError("This Port cannot set sender. ", str);
  }
  

  /**
   * <japanese>
   * M[qo^܂B<p>
   *
   * ̃NX̔hNXł́A
   * Object ^ɃLXgꂽM[q̃AhX aReceiver ƂĎ󂯂ƂA
   * aReceiver 𑗐MƂēo^ł true Ao^łȂ false Ԃ悤`ĂB<p>
   *
   * ̃\bh́AG[bZ[Wo͌ false Ԃ܂B
   * hNXŁAsetSender() Ē`Ȃꍇ́AꂪAdefault ̒`ɂȂ܂B
   * </japanese>
   * <p>
   *
   * Register a receiver of this port.
   * There may be more than one receiver for a sender port.
   * This method returns whether it suceeded or not. <p>
   * 
   * Although a default definition is given here,
   * a programmer must give his own definitions in derived classes.
   *
   * @param aReceiver an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean registerReceiver(Object aReceiver) throws OhError {    //; (*2)
    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);

    oPrint(out, " > ");

    if(aReceiver != null)
      ((OhObject)aReceiver).oPrint(out, " >   receiver > ");

    String str = buffer.toString();
    out.close();
    
    throw new OhError("This Port cannot register receiver. ", str);
  }


  /**
   * <japanese>
   * [q̐ڑƂ̏Iуf[^MJn̂߂̏ƂsȂ܂B<p>
   *
   * ̃NX̔hNXŁA[q̐ڑƂ̏IƂ̎d̏sȂ悤`܂B<p>
   * 
   *  true As false Ԃ܂B
   * ̃\bh́AG[bZ[Wo͌ false Ԃ܂B<p>
   *
   * <a href="OhReceivePort.html">OhReceivePort</a> ł́AڑƌsȂׂ͓ɂ܂B
   *  true Ԃ悤ɒ`܂B<p>
   *
   * <a href="OhSendPort.html">OhSendPort</a> ł́A̎M[qɃf[^𑗐M邱Ƃ̂ŁA
   * M port Xg _bufferList Ɏ^Ă܂B
   * ŏIIɂ́AXgɓo^ꂽM port z _receiversArray Ɉڂւ܂B
   * ̍Ƃ <a href="OhSenderPort.html#canYouSwitch()">OhSenderPort.canYouSwitch()</a> Ŏs܂
   * i<a href="OhSendPort.html">OhSendPort</a> ́A
   * <a href="OhSenderPort.html">OhSenderPort</a> ̔hNXjB<p>
   *
   * ̌AڑIāAvZn߂܂B
   * ̎A_dataPackArray ̏܂B
   * Ȃ킿A_numberOfDataPacks ̒l -1  0 ɐݒ肵܂B
   * </japanese>
   * <p>
   *
   * Finish registration of receivers and count the number of receivers.
   * 
   * In a derived class, do work which finishes connection operations.
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean canYouSwitch() throws OhError {    //;(*3)
    StringWriter buffer = new StringWriter();
    PrintWriter out     = new PrintWriter(buffer);

    oPrint(out, " > ");

    String str = buffer.toString();
    out.close();

    throw new OhError("This Port cannot switch. ", str);
  }

  /**
   * <japanese>
   * vZv󂯂܂B<p>
   *
   * ł͉Ȃ悤ɒ`Ă܂B<p>
   *
   * ȉ̃NX OhPort ̔hNXł̂悤ɒ`Ă܂B
   *
   * <ol>
   *  <li> <a href="OhSendPort.html#calReq()">OhSendPort.calReq()</a> ́A
   *        <a href="OhSendPort.html">OhSendPort</a> LĂvfɌvZv]܂B
   *  <li> <a href="OhRelayPort.html#calReq()">OhRelayPort.calReq()</a> ́A
   *        <a href="OhRelayPort.html">OhRelayPort</a> Ƀf[^𑗐MĂ[qɌvZv]܂B
   *  <li> <a href="OhReceivePort.html#calReq()">OhReceivePort.calReq()</a> ́A
   *        <a href="OhReceivePort.html">OhReceivePort</a> Ƀf[^𑗐MĂ[qɌvZv]܂B
   * </ol>
   * </japanese>
   * <p>
   *
   * Request of calculation.
   * Transfer the request to a element which does actual caluculation work or ports
   * which send datapack to the current port.
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError {}


  // 1. Function for acquiring datapacks.
  
  /**
   * <japanese>
   * f[^pbN̎^mF܂B<p>
   *
   * ̃\bh́AaT() ɂf[^擾\mF郁\bhłB
   * ̃NX̓̒ۃ\bhĂяoĂ܂B
   * </japanese>
   * <p>
   * Inquire whether the invoking port can get data at <a href="OhTimePoint.html">OhTimePoint</a> "aT".
   *
   * @param aT an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetData(OhTimePoint aT) {
    return canYouGetData(aT, aT);              // Abstract method call.
  }


  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * hNXɂāAw肵ăf[^pbN擾郁\bh`܂B
   * ̃NX̓̒ۃ\bhĂяoĂ܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbN擾Ai[܂B<p>
   *
   * aFlag ͎擾̃[hI邽߂̃IvVŁAL̒ʂłB
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer ɂi[܂B<p>
   *
   * 𖞂f[^pbN擾łȂꍇAaFormer  null ɐݒ肳܂B<p>
   *
   * f[^擾łꍇ́Atrue Ԃ܂B
   * </japanese>
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime) {
    Object[] dummy = {null};
    return getDataPack(aFormer, _dataTime, dummy, 'f');        // Abstract method call.
  }


  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B <p>
   *
   * hNXɂāAw肵ăf[^pbN擾郁\bh`܂B
   * ̃NX̓̒ۃ\bhĂяoĂ܂B<p>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbN擾Ai[܂B<p>
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aLater ɂi[܂B<p>
   *
   * 𖞂f[^pbN擾łȂꍇAaLater  null ɐݒ肳܂B<p>
   *
   * f[^擾łꍇ́Atrue Ԃ܂B
   * </japanese>
   *
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(OhTimePoint _dataTime, Object[] aLater) {
    Object[] dummy = {null};
    return getDataPack(dummy, _dataTime, aLater, 'l');        // Abstract method call.
  }


  // 2. Object information output function.

  /**
   * <japanese>
   * IuWFNgo͂܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void cPrint(PrintWriter aPw, String aHead) {
    aPw.print(aHead + _className + " " + _objectName + " " + _objectNumber + NEWLINE);
  }


  /**
   * <japanese>
   * ڑo͂܂B<p>
   *
   * hNXŐڑ\悤`܂B
   * ̃NXł́AG[bZ[Wo͂܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printConnection(PrintWriter aPw, String aHead) {
    oPrint(aPw, aHead);
    aPw.print(aHead + "    This port is not connected." + NEWLINE);
  }


  // 3. addition

  /**
   * <japanese>
   * [q쐬܂B<p>
   *
   * ɁA[qf\NX^ƁA
   * ̒[qf̃IuWFNg𐶐܂B
   * </japanese>
   *
   * @param aClassName a <code>String</code> value
   * @return an <code>OhPort</code> value
   */
  public static OhPort create(String aClassName) {
    OhPort port = null;
    try {
      port = (OhPort)Class.forName("ohymosj." + aClassName).newInstance();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return port;
  }

  // end{methods} --------------------------------------------------------

} // OhPort
