/**
 * @(#) OhReceiveDouble; class to express a port which receive double data.
 *                       Make "OhReceiveDouble.java" from "ohp_tmp.h" and "ohp_tmp.cpp".
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * <japanese>
 * f[^l double ^̃f[^pbNMM[q\NXłB<p>
 *
 * <a href="OhReceivePort.html">OhReceivePort</a> pč쐬M[q̃NXłB
 * </japanese>
 *
 * <p>
 *
 * OhReceiveDouble;
 *
 * OhReceiveDouble is the base class to express a port which receive double data.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.26 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhReceiveDouble extends OhReceivePort
{
  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------

  // @begin {constructors} -----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^
   * <a href="OhReceiveDouble.html#init(String, int)">OhReceiveDouble.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhReceiveDouble.html#init(String, int)">OhReceiveDouble.init(String, int)</a> 
   * păoϐĉB
   * </jananese>
   *
   */
  public OhReceiveDouble() {
    super();
    _className = "OhReceiveDouble";
  }

  /**
   * <japanese>
   * NX̃RXgN^
   * <a href="OhReceivePort.html#OhReceivePort.OhReceivePort(String, int)">OhReceivePort.OhReceivePort(String, int)</a>
   * ̃RXgN^păoϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhReceiveDouble"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _owner             : null
   *  <li> _sender            : null
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNumber an <code>int</code> value
   */
  public OhReceiveDouble(String aPortName, int aPortNumber) throws OhError {
    super(aPortName, aPortNumber);
    _className = "OhReceiveDouble";
  }

  // @end {constructors} -------------------------------------------------

  // @begin {methods} ----------------------------------------------------

  /**
   * <japanese>
   * Mf[^pbÑf[^ľ^ double \ "double" Ԃ܂B
   * </japanese>
   * <p>
   * Returns the data pack type which expresses the type of data to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {    //; (*5)
    return "double";
  }

  /**
   * <japanese>
   * f[^Ԍ^w肵āA aT ɂf[^l擾܂B
   * </japanese>
   * <p>
   * Returns data value at time t.
   * This data connection type is brokenline or step or pulse.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>double</code> value
   */
  public double getData(OhTimePoint aT, Vector aDctVec) throws OhError {    //; (*1-1)
    double dataValue = 0D;
    
    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter out     = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackDouble)dataPackArray.get(0)).getDataTime() );
    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) <= 0) {    // aT <= ta ?
      return ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();
    }

    if (aT.compareTo(tb) >= 0) {    // aT >= tb ?
      return ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
    }

    DataConnectionType dataConnectionType = (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();
      }
      else if (aT.compareTo(tb) >= 0) {
        dataValue = ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;
        
        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackDouble)dataPackArray.get(ip1)).getDataTime() );
        
        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackDouble)dataPackArray.get(ip2)).getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;
          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);
          }
          else {
            return ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
          }
        }

        double y1 = ((OhDataPackDouble)dataPackArray.get(ip1)).getDataValue();
        
        if (ip1 == ip2) {
          return y1;
        }

        double y2 = ((OhDataPackDouble)dataPackArray.get(ip2)).getDataValue();

        OhTimePoint aTBackUp = new OhTimePoint();
        aTBackUp.copy(aT);
        //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
        dataValue = (((aTBackUp.subtract(t1))*y2 + (t2.subtract(aTBackUp))*y1)) / (t2.subtract(t1));
      }
    }
    else if (dataConnectionType == DataConnectionType.STEP)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();
      }
      else if (aT.compareTo(tb) >= 0) {
        dataValue = ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackDouble)dataPackArray.get(ip1)).getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackDouble)dataPackArray.get(ip2)).getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);
          }
          else {
            return ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
          }
        }
        dataValue = ((OhDataPackDouble)dataPackArray.get(ip2)).getDataValue();
      }
    }
    else if (dataConnectionType == DataConnectionType.PULSE)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = 0D;
      }
      else if (aT.compareTo(tb) >= 0) {
        dataValue = 0D;
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackDouble)dataPackArray.get(ip1)).getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackDouble)dataPackArray.get(ip2)).getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);
          }
          else {
            return ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
          }
        }
        if (aT.compareTo(t1) == 0) {
          dataValue = ((OhDataPackDouble)dataPackArray.get(ip1)).getDataValue();
        }
        else if (aT.compareTo(t2) == 0) {
          dataValue = ((OhDataPackDouble)dataPackArray.get(ip2)).getDataValue();
        }
        else {
          dataValue = 0D;
        }
      }
    }
    else {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "> ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Illegal data_connection_type.", str);
    }
    
    return dataValue;
    
  }




  /**
   * <japanese>
   *  aT ɂf[^l擾܂B
   * f[^l̕Ԃ͐܂Ƃ܂B
   * </japanese>
   * <p>
   * Returns the data value at time t.
   * This data connection type is brokenline.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @return a <code>double</code> value
   */
  public double getData(OhTimePoint aT) throws OhError {    //; (*1-2)
    double dataValue = 0D;
    
    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    
    if(numberOfDataPacks <= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter out = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackDouble)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) <= 0) {    // aT <= ta ?
      return ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();
    }

    if (aT.compareTo(tb) >= 0) {    // aT >= tb ?
      return ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
    }

    int ip1 = 0;
    int ip2 = numberOfDataPacks - 1;

    OhTimePoint t1 = new OhTimePoint();
    t1.copy( ((OhDataPackDouble)dataPackArray.get(ip1)).getDataTime() );

    OhTimePoint t2 = new OhTimePoint();
    t2.copy( ((OhDataPackDouble)dataPackArray.get(ip2)).getDataTime() );
    
    while (ip2 - ip1 > 1) {
      ip = (ip2 + ip1)/2;

      OhTimePoint tp = new OhTimePoint();
      tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime() );

      if (tp.compareTo(aT) < 0) {
        ip1 = ip;
        t1.copy(tp);
      }
      else if (tp.compareTo(aT) > 0) {
        ip2 = ip;
        t2.copy(tp);
      }
      else {
        return ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
      }
    }

    double y1 = ((OhDataPackDouble)dataPackArray.get(ip1)).getDataValue();
    
    if (ip1 == ip2) {
      return y1;
    }

    double y2 = ((OhDataPackDouble)dataPackArray.get(ip2)).getDataValue();

    OhTimePoint aTBackUp = new OhTimePoint();
    aTBackUp.copy(aT);
    //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
    dataValue = (((aTBackUp.subtract(t1))*y2 + (t2.subtract(aTBackUp))*y1)) / (t2.subtract(t1));
    
    return dataValue;
  }



  /**
   * <japanese>
   * f[^Ԍ^w肵āA aT ɂf[^l擾Adouble ^z aDataValue ̑vfɊi[܂B
   * </japanese>
   * <p>
   * Set the data value at time t to reference aDataValue[0].
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double[]</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, double[] aDataValue, Vector aDctVec) throws OhError {  //; (*4-1)
    aDataValue[0] = getData(aT, aDctVec);
  }


  /**
   * <japanese>
   *  aT ɂf[^l擾Adouble ^z aDataValue ̑vfɊi[܂B
   * f[^l̕Ԃ͐܂Ƃ܂B
   * </japanese>
   * <p>
   * Set the data value at time t to reference aDataValue[0].
   * This data connection type is brokenline.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double[]</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, double[] aDataValue) throws OhError {  //; (*4-2)
    aDataValue[0] = getData(aT);
  }

  

  /**
   * <japanese>
   * f[^Ԍ^w肵āA aT1 玞 aT2 ܂ł̐ϕlԂ܂B
   * </japanese>
   * <p>
   * Returns the integrated value from time aT1 till time aT2.
   * aT1 should be less than or equal to t2. <p>
   *
   * When data connection type is Pulse, the value summed from aT1 till
   * aT2 will be returned. The pulse at time aT1 is not contained in the
   * sum, and the pulse at time aT2 is contained in the sum.
   *
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>double</code> value
   */
  public double getDataSum(OhTimePoint aT1, OhTimePoint aT2, Vector aDctVec) throws OhError {    //(*3-1)
    double sum = 0D;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }


    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }


    OhTimePoint ta = ((OhDataPackDouble)dataPackArray.get(0)).getDataTime();
    OhTimePoint tb = ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataTime();

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {    // ta <= aT1 and aT2 <= tb, right?
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    DataConnectionType dataConnectionType = (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE) {
      double y  = 0D;
      double ya = 0D;
      double yb = 0D;
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePoint tp = ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb  = tp;

          ta = ((OhDataPackDouble)dataPackArray.get(ipa)).getDataTime();
          break;
        }
      }


      ya = ((OhDataPackDouble)dataPackArray.get(ipa)).getDataValue();
      yb = ((OhDataPackDouble)dataPackArray.get(ipb)).getDataValue();

      if(ipa == ipb) {
        y = ya;
      }
      else {
        y = ( (aT1.subtract(ta))*yb + (tb.subtract(aT1)*ya)) / (tb.subtract(ta));
      }

      OhTimePoint t  = aT1;
      ip = ipb;
      OhTimePoint ts = tb;
      double ys = yb;

      while(ts.compareTo(aT2) < 0) {
        sum += (y + ys) * (ts.subtract(t)) / 2;
        t = ts;
        y = ys;
        ip++;

        ts = ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();
        ys = ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
      }

      ys = (y*(ts.subtract(aT2)) + ys*(aT2.subtract(t))) / (ts.subtract(t));
      sum += (y+ys) * (aT2.subtract(t)) / 2;      
    }
    else if(dataConnectionType == DataConnectionType.STEP) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePoint tp = ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb = tp;
          break;
        }
      }

      OhTimePoint tn = tb;
      ip = ipb;

      double yn = ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
      OhTimePoint ts = aT1;

      while(tn.compareTo(aT2) < 0) {
        sum += yn * (tn.subtract(ts));
        ts = tn;
        ip++;

        tn = ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();
        yn = ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
      }

      if(aT2.compareTo(tn) <= 0) {
        sum += yn * (aT2.subtract(ts));
      }
    }
    else if(dataConnectionType == DataConnectionType.PULSE) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePoint tp = ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip + 1;
          break;
        }
      }

      OhTimePoint tn = new OhTimePoint();
      double yn = 0D;

      for(; ip < numberOfDataPacks; ip++) {
        tn = ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime();
        yn = ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
        
        if(tn.compareTo(aT2) > 0)
          break;

        sum += yn;
      }
    }
    else {
      throw new IllegalArgumentException("Illegal data_connection_type.");
    }
    
    return sum;
  }

  
  /**
   * <japanese>
   *  aT1 玞 aT2 ܂ł̐ϕlԂ܂B
   * f[^l̕Ԃ͐܂Ƃ܂B
   * </japanese>
   * <p>
   * Gets sum data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>double</code> value
   */
  public double getDataSum(OhTimePoint aT1, OhTimePoint aT2) throws OhError {    //(*3-2)
    double sum = 0D;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }


    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackDouble)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {      // ta <= aT1 and aT2 <= tb, right?
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    double y  = 0D;
    double ya = 0D;
    double yb = 0D;
    int ipa = 0;
    int ipb = numberOfDataPacks - 1;
    
    while(ipb - ipa > 1) {
      ip = (ipa + ipb)/2;

      OhTimePoint tp = new OhTimePoint();
      tp.copy( ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime() );
      
      if(tp.compareTo(aT1) < 0) {
        ipa = ip;
        ta.copy(tp);
      }
      else if(tp.compareTo(aT1) > 0) {
        ipb = ip;
        tb.copy(tp);
      }
      else {
        ipb = ip;
        ipa = ip - 1;
        tb.copy(tp);

        ta.copy( ((OhDataPackDouble)dataPackArray.get(ipa)).getDataTime() );

        break;
      }
    }

    ya = ((OhDataPackDouble)dataPackArray.get(ipa)).getDataValue();
    yb = ((OhDataPackDouble)dataPackArray.get(ipb)).getDataValue();

    if(ipa == ipb) {
      y = ya;
    }
    else {
      y = ( (aT1.subtract(ta))*yb + (tb.subtract(aT1)*ya)) / (tb.subtract(ta));
    }

    OhTimePoint t = new OhTimePoint();
    t.copy(aT1);

    ip = ipb;

    OhTimePoint ts = new OhTimePoint();
    ts.copy(tb);

    double ys = yb;
    
    while(ts.compareTo(aT2) < 0) {
      sum += (y + ys) * (ts.subtract(t)) / 2;

      t.copy(ts);

      y = ys;
      ip++;

      ts.copy( ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime() );
      ys = ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
    }
    
    ys = (y*(ts.subtract(aT2)) + ys*(aT2.subtract(t))) / (ts.subtract(t));
    sum += (y+ys) * (aT2.subtract(t)) / 2;      

    return sum;
  }


  

  /**
   * <japanese>
   *  aT ɂpXf[^l double ^z aDataValue Ɋi[܂B
   * </japanese>
   * <p>
   * Gets pulse data.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>double[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetPulseData(OhTimePoint aT, double[] aDataValue) {    //(*2)

    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];


    if (numberOfDataPacks <= 0)
      return false;

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackDouble)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackDouble)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) < 0 || aT.compareTo(tb) > 0)
      return false;

    OhTimePoint tn = new OhTimePoint();
    OhTimePoint tp = new OhTimePoint();
    double yn = 0D;
    double yp = 0D;

    tn.copy(ta);
    yn = ((OhDataPackDouble)dataPackArray.get(0)).getDataValue();

    if (aT.compareTo(tn) == 0) {
      aDataValue[0] = yn;
      return true;
    }

    boolean hit = false;

    for (ip = 1; ip < numberOfDataPacks; ip++) {
      tp.copy(tn);
      yp = yn;

      tn.copy( ((OhDataPackDouble)dataPackArray.get(ip)).getDataTime() );
      yn = ((OhDataPackDouble)dataPackArray.get(ip)).getDataValue();
      
      if (tn.compareTo(aT) > 0) {
        hit = false;
        break;
      }
      else if (tn.compareTo(aT) == 0){
        aDataValue[0] = yn;
        hit = true;
        break;
      }
    }
    return hit;
  }




  /**
   * <japanese>
   * f[^Ԍ^w肵āA aT1 ɂf[^lƎ aT2 ɂf[^l̕ϒlԂ܂B
   * </japanese>
   * <p>
   * Gets mean data.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>double</code> value
   * @exception OhError if an error occurs
   */
  public double getMeanData(OhTimePoint aT1, OhTimePoint aT2, Vector aDctVec) throws OhError {    //(*6)
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2, aDctVec);
    }
    else {
      return getDataSum(aT1, aT2, aDctVec) / (aT2.subtract(aT1));
    }
  }



  
  /**
   * <japanese>
   *  aT1 ɂf[^lƎ aT2 ɂf[^l̕ϒlԂ܂B
   * f[^l̕Ԃ͐܂Ƃ܂B
   * </japanese>
   * <p>
   * Gets mean data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>double</code> value
   */
  public double getMeanData(OhTimePoint aT1, OhTimePoint aT2) throws OhError {    //(*6)
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2);
    }
    else {
      return getDataSum(aT1, aT2) / (aT2.subtract(aT1));
    }
  }

  // @end {methods} ------------------------------------------------------

} // OhReceiveDouble
