/**
 * @(#) OhReceiveFloat; class to express a port which receive float data.
 *                      Make "OhReceiveFloat.java" from "ohp_tmp.h" and "ohp_tmp.cpp".
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * OhReceiveFloat;
 *
 * OhReceiveFloat is the base class to express a port which receive float data.
 *
 * <japanese>
 * <p>
 * float ^̃f[^pbNMM[q̃NX <p>
 *
 * <a href="OhReceivePort.html">OhReceivePort</a> pč쐬M[q̃NXłB
 * </japanese>
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.3 $ $Date: 2004/03/08 17:03:02 $
 */
public class OhReceiveFloat extends OhReceivePort
{
  // @begin{initialization} ----------------------------------------------

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------

  // @begin {constructors} -----------------------------------------------

  /**
   * Creates a new <code>OhReceiveFloat</code> instance.
   *
   */
  public OhReceiveFloat() {
    super();
    _className = "OhReceiveFloat";
  }

  /**
   * Creates a new <code>OhReceiveFloat</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNumber an <code>int</code> value
   */
  public OhReceiveFloat(String aPortName, int aPortNumber) throws OhError {
    super(aPortName, aPortNumber);
    _className = "OhReceiveFloat";
  }

  // @end {constructors} -------------------------------------------------

  // @begin {methods} ----------------------------------------------------

  /**
   * Copy method.
   *
   * @param aRd an <code>OhReceiveFloat</code> value
   */
  public void copy(OhReceiveFloat aRd) {
    _className    = aRd._className;
    _objectName   = aRd._objectName;
    _objectNumber = aRd._objectNumber;
    _owner        = aRd._owner;

    _sender = aRd._sender;
  }
  
  /**
   * Returns the data pack type which expresses the type of data to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {    //(*5)
    return "float";
  }

  /**
   * Returns data value at time t.
   * This data connection type is brokenline or step or pulse.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>float</code> value
   */
  public float getData(OhTimePoint aT, Vector aDctVec) throws OhError {    //; (*1-)
    float dataValue = 0F;
    
    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter out     = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackFloat)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );

    if (aT.compareTo(ta) <= 0) {    // aT <= ta ?
      return ((OhDataPackFloat)dataPackArray.get(0)).getDataValue();
    }

    if (aT.compareTo(tb) >= 0) {    // aT >= tb ?
      return ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
    }

    DataConnectionType dataConnectionType = (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = ((OhDataPackFloat)dataPackArray.get(0)).getDataValue();
      }
      else if (aT.compareTo(tb) <= 0) {
        dataValue = ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackFloat)dataPackArray.get(ip1)).getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackFloat)dataPackArray.get(ip2)).getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);
          }
          else {
            return ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
          }
        }

        float y1 = ((OhDataPackFloat)dataPackArray.get(ip1)).getDataValue();
        
        if (ip1 == ip2) {
          return y1;
        }

        float y2 = ((OhDataPackFloat)dataPackArray.get(ip2)).getDataValue();

        OhTimePoint aTBackUp = new OhTimePoint();
        aTBackUp.copy(aT);
        //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
        dataValue = (((aTBackUp.subtract(t1))*y2 + (t2.subtract(aTBackUp))*y1)) / (t2.subtract(t1));
      }
    }
    else if (dataConnectionType == DataConnectionType.STEP)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = ((OhDataPackFloat)dataPackArray.get(0)).getDataValue();
      }
      else if (aT.compareTo(tb) <= 0) {
        dataValue = ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackFloat)dataPackArray.get(ip1)).getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackFloat)dataPackArray.get(ip2)).getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);
          }
          else {
            return ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
          }
        }
        dataValue = ((OhDataPackFloat)dataPackArray.get(ip2)).getDataValue();
      }
    }
    else if (dataConnectionType == DataConnectionType.PULSE)
    {
      if (aT.compareTo(ta) <= 0) {
        dataValue = 0F;
      }
      else if (aT.compareTo(tb) <= 0) {
        dataValue = 0F;
      }
      else {
        int ip1 = 0;
        int ip2 = numberOfDataPacks - 1;

        OhTimePoint t1 = new OhTimePoint();
        t1.copy( ((OhDataPackFloat)dataPackArray.get(ip1)).getDataTime() );

        OhTimePoint t2 = new OhTimePoint();
        t2.copy( ((OhDataPackFloat)dataPackArray.get(ip2)).getDataTime() );

        while (ip2 - ip1 > 1) {
          ip = (ip2 + ip1)/2;

          OhTimePoint tp = new OhTimePoint();
          tp.copy( ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime() );

          if (tp.compareTo(aT) < 0) {
            ip1 = ip;
            t1.copy(tp);
          }
          else if (tp.compareTo(aT) > 0) {
            ip2 = ip;
            t2.copy(tp);
          }
          else {
            return ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
          }
        }
        if (aT.compareTo(t1) == 0) {
          dataValue = ((OhDataPackFloat)dataPackArray.get(ip1)).getDataValue();
        }
        else if (aT.compareTo(t2) == 0) {
          dataValue = ((OhDataPackFloat)dataPackArray.get(ip2)).getDataValue();
        }
        else {
          dataValue = 0F;
        }
      }
    }
    else {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "> ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Illegal data_connection_type.", str);
    }
    
    return dataValue;
    
  }




  /**
   * Returns the data value at time t.
   * This data connection type is brokenline.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @return a <code>float</code> value
   */
  public float getData(OhTimePoint aT) throws OhError {    //; (*1-2)
    float dataValue = 0F;
    
    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    
    if(numberOfDataPacks <= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter out = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackFloat)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) <= 0) {    // aT <= ta ?
      return ((OhDataPackFloat)dataPackArray.get(0)).getDataValue();
    }

    if (aT.compareTo(tb) >= 0) {    // aT >= tb ?
      return ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataValue();
    }

    int ip1 = 0;
    int ip2 = numberOfDataPacks - 1;

    OhTimePoint t1 = new OhTimePoint();
    t1.copy( ((OhDataPackFloat)dataPackArray.get(ip1)).getDataTime() );

    OhTimePoint t2 = new OhTimePoint();
    t2.copy( ((OhDataPackFloat)dataPackArray.get(ip2)).getDataTime() );
    
    while (ip2 - ip1 > 1) {
      ip = (ip2 + ip1)/2;

      OhTimePoint tp = new OhTimePoint();
      tp.copy( ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime() );

      if (tp.compareTo(aT) < 0) {
        ip1 = ip;
        t1.copy(tp);
      }
      else if (tp.compareTo(aT) > 0) {
        ip2 = ip;
        t2.copy(tp);
      }
      else {
        return ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
      }
    }

    float y1 = ((OhDataPackFloat)dataPackArray.get(ip1)).getDataValue();
    
    if (ip1 == ip2) {
      return y1;
    }

    float y2 = ((OhDataPackFloat)dataPackArray.get(ip2)).getDataValue();

    OhTimePoint aTBackUp = new OhTimePoint();
    aTBackUp.copy(aT);
    //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
    dataValue = (((aTBackUp.subtract(t1))*y2 + (t2.subtract(aTBackUp))*y1)) / (t2.subtract(t1));
    
    return dataValue;
  }







  /**
   * Set the data value at time t to reference aDataValue[0].
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float[]</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, float[] aDataValue, Vector aDctVec) throws OhError {  //; (*4-1)
    aDataValue[0] = getData(aT, aDctVec);
  }


  /**
   * Set the data value at time t to reference aDataValue[0].
   * This data connection type is brokenline.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float[]</code> value
   * @exception OhError if an error occurs
   */
  public void setData(OhTimePoint aT, float[] aDataValue) throws OhError {  //; (*4-2)
    aDataValue[0] = getData(aT);
  }




  
  /**
   * Returns the integrated value from time aT1 till time aT2.
   * aT1 should be less than or equal to t2. <p>
   *
   * When data connection type is Pulse, the value summed from aT1 till
   * aT2 will be returned. The pulse at time aT1 is not contained in the
   * sum, and the pulse at time aT2 is contained in the sum.
   *
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>float</code> value
   */
  public float getDataSum(OhTimePoint aT1, OhTimePoint aT2, Vector aDctVec) throws OhError {    //(*3-1)
    float sum = 0F;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }


    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }


    OhTimePoint ta = ((OhDataPackFloat)dataPackArray.get(0)).getDataTime();
    OhTimePoint tb = ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataTime();

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {    // ta <= aT1 and aT2 <= tb, right?
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    DataConnectionType dataConnectionType = (DataConnectionType)aDctVec.elementAt(0);

    if(dataConnectionType == DataConnectionType.BROKENLINE) {
      float y  = 0F;
      float ya = 0F;
      float yb = 0F;
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePoint tp = ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb  = tp;

          ta = ((OhDataPackFloat)dataPackArray.get(ipa)).getDataTime();
          break;
        }
      }


      ya = ((OhDataPackFloat)dataPackArray.get(ipa)).getDataValue();
      yb = ((OhDataPackFloat)dataPackArray.get(ipb)).getDataValue();

      if(ipa == ipb) {
        y = ya;
      }
      else {
        y = ( (aT1.subtract(ta))*yb + (tb.subtract(aT1)*ya)) / (tb.subtract(ta));
      }

      OhTimePoint t  = aT1;
      ip = ipb;
      OhTimePoint ts = tb;
      float ys = yb;

      while(ts.compareTo(aT2) < 0) {
        sum += (y + ys) * (ts.subtract(t)) / 2;
        t = ts;
        y = ys;
        ip++;

        ts = ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime();
        ys = ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
      }

      ys = (y*(ts.subtract(aT2)) + ys*(aT2.subtract(t))) / (ts.subtract(t));
      sum += (y+ys) * (aT2.subtract(t)) / 2;      
    }
    else if(dataConnectionType == DataConnectionType.STEP) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePoint tp = ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip;
          ipa = ip - 1;
          tb = tp;
          break;
        }
      }

      OhTimePoint tn = tb;
      ip = ipb;

      float yn = ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
      OhTimePoint ts = aT1;

      while(tn.compareTo(aT2) < 0) {
        sum += yn * (tn.subtract(ts));
        ts = tn;
        ip++;

        tn = ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime();
        yn = ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
      }

      if(aT2.compareTo(tn) <= 0) {
        sum += yn * (aT2.subtract(ts));
      }
    }
    else if(dataConnectionType == DataConnectionType.PULSE) {
      int ipa = 0;
      int ipb = numberOfDataPacks - 1;

      while(ipb - ipa > 1) {
        ip = (ipa + ipb)/2;

        OhTimePoint tp = ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime();

        if(tp.compareTo(aT1) < 0) {
          ipa = ip;
          ta  = tp;
        }
        else if(tp.compareTo(aT1) > 0) {
          ipb = ip;
          tb  = tp;
        }
        else {
          ipb = ip + 1;
          break;
        }
      }

      OhTimePoint tn = new OhTimePoint();
      float yn = 0F;

      for(; ip < numberOfDataPacks; ip++) {
        tn = ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime();
        yn = ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
        
        if(tn.compareTo(aT2) > 0)
          break;

        sum += yn;
      }
    }
    else {
      throw new IllegalArgumentException("Illegal data_connection_type.");
    }
    
    return sum;
  }






  
  /**
   * Gets sum data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>float</code> value
   */
  public float getDataSum(OhTimePoint aT1, OhTimePoint aT2) throws OhError {    //(*3-2)
    float sum = 0F;

    if(aT1.compareTo(aT2) == 0) {
      return sum;
    }

    if(aT1.compareTo(aT2) > 0) {
      OhTimePoint aT3 = aT1;
      aT1 = aT2;
      aT2 = aT3;
    }


    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];

    if(numberOfDataPacks <= 1) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " >");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in Get_Data", str);
    }

    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackFloat)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );

    if (aT1.compareTo(ta) < 0 || aT2.compareTo(tb) > 0) {      // ta <= aT1 and aT2 <= tb, right?
      throw new IllegalArgumentException("Illegal aT1 or aT2.");
    }

    float y  = 0F;
    float ya = 0F;
    float yb = 0F;
    int ipa = 0;
    int ipb = numberOfDataPacks - 1;
    
    while(ipb - ipa > 1) {
      ip = (ipa + ipb)/2;

      OhTimePoint tp = new OhTimePoint();
      tp.copy( ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime() );
      
      if(tp.compareTo(aT1) < 0) {
        ipa = ip;
        ta.copy(tp);
      }
      else if(tp.compareTo(aT1) > 0) {
        ipb = ip;
        tb.copy(tp);
      }
      else {
        ipb = ip;
        ipa = ip - 1;
        tb.copy(tp);

        ta.copy( ((OhDataPackFloat)dataPackArray.get(ipa)).getDataTime() );

        break;
      }
    }

    ya = ((OhDataPackFloat)dataPackArray.get(ipa)).getDataValue();
    yb = ((OhDataPackFloat)dataPackArray.get(ipb)).getDataValue();

    if(ipa == ipb) {
      y = ya;
    }
    else {
      y = ( (aT1.subtract(ta))*yb + (tb.subtract(aT1)*ya)) / (tb.subtract(ta));
    }
    
    OhTimePoint t = new OhTimePoint();
    t.copy(aT1);

    ip = ipb;

    OhTimePoint ts = new OhTimePoint();
    ts.copy(tb);

    float ys = yb;
    
    while(ts.compareTo(aT2) < 0) {
      sum += (y + ys) * (ts.subtract(t)) / 2;

      t.copy(ts);

      y = ys;
      ip++;

      ts.copy( ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime() );
      ys = ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
    }
    
    ys = (y*(ts.subtract(aT2)) + ys*(aT2.subtract(t))) / (ts.subtract(t));
    sum += (y+ys) * (aT2.subtract(t)) / 2;      

    return sum;
  }


  

  /**
   * Gets pulse data.
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aDataValue a <code>float[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetPulseData(OhTimePoint aT, float[] aDataValue) {    //(*2)

    OhArrayList dataPackArray = null;
    int numberOfDataPacks = 0;

    OhArrayList[] dataPackArrayArray = {dataPackArray};    //; Array for work.
    int[] numberOfDataPacksArray = {numberOfDataPacks};

    int ip = 0;

    getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

    dataPackArray = dataPackArrayArray[0];
    numberOfDataPacks = numberOfDataPacksArray[0];


    if (numberOfDataPacks <= 0)
      return false;


    OhTimePoint ta = new OhTimePoint();
    ta.copy( ((OhDataPackFloat)dataPackArray.get(0)).getDataTime() );

    OhTimePoint tb = new OhTimePoint();
    tb.copy( ((OhDataPackFloat)dataPackArray.get(numberOfDataPacks - 1)).getDataTime() );


    if (aT.compareTo(ta) < 0 || aT.compareTo(tb) > 0)
      return false;

    OhTimePoint tn = new OhTimePoint();
    OhTimePoint tp = new OhTimePoint();
    float yn = 0F;
    float yp = 0F;

    tn.copy(ta);
    yn = ((OhDataPackFloat)dataPackArray.get(0)).getDataValue();

    if (aT.compareTo(tn) == 0) {
      aDataValue[0] = yn;
      return true;
    }

    boolean hit = false;

    for (ip = 1; ip < numberOfDataPacks; ip++) {
      tp.copy(tn);
      yp = yn;

      tn.copy( ((OhDataPackFloat)dataPackArray.get(ip)).getDataTime() );
      yn = ((OhDataPackFloat)dataPackArray.get(ip)).getDataValue();
      
      if (tn.compareTo(aT) > 0) {
        hit = false;
        break;
      }
      else if (tn.compareTo(aT) == 0){
        aDataValue[0] = yn;
        hit = true;
        break;
      }
    }
    return hit;
  }




  /**
   * Gets mean data.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>float</code> value
   * @exception OhError if an error occurs
   */
  public float getMeanData(OhTimePoint aT1, OhTimePoint aT2, Vector aDctVec) throws OhError {  //; (*6)
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2, aDctVec);
    }
    else {
      return getDataSum(aT1, aT2, aDctVec) / (aT2.subtract(aT1));
    }
  }



  
  /**
   * Gets mean data whose data connection type is brokenline.
   *
   * @param aT1 an <code>OhTimePoint</code> value
   * @param aT2 an <code>OhTimePoint</code> value
   * @return a <code>float</code> value
   */
  public float getMeanData(OhTimePoint aT1, OhTimePoint aT2) throws OhError {  //; (*6)
    if(aT2.compareTo(aT1) == 0) {
      return getData(aT2);
    }
    else {
      return getDataSum(aT1, aT2) / (aT2.subtract(aT1));
    }
  }

  // @end {methods} ------------------------------------------------------

} // OhReceiveFloat
