/**
 * @(#) OhReceivePort; class to express a receive port
 *                     Make "OhReceivePort.java" from "ohreceive.h" and "ohreceive.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^M[qf\NXłB<p>
 *
 * ۃNXłB<p>
 *
 * M[qɂ͑M[q<b>P</b>ȂƂł܂B<p>
 *
 * NX OhReceivePort ł́A
 *
 * <ul>
 *  <li> f[^pbNM[q : _sender
 * </ul>
 *
 * LA
 *
 * <ul>
 *  <li> M[qƂ̐ڑ@\
 *  <li> _sender ̃f[^pbNM@\
 * </ul>
 *
 * `܂B
 * </japanese>
 *
 * <p>
 *
 * Class for models of receive ports. Derived from the class
 * <a href="OhPort.html">OhPort</a>.
 * 
 * This class OhReceivePort has a instance variable for storing a sender
 * which sends datapacks to the current receive port. <p>
 * 
 * Methods for
 *
 * <ul>
 *  <li>  connecting the current receive port to a corresponding sender
 *  <li>  receiving datapacks from the sender
 * </ul>
 *
 * are defined. <p>
 * 
 * NOTE: 
 * 
 * Only one send port can be connected to an object of class OhReceivePort.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.45 $ $Date: 2004/02/11 22:52:00 $
 */
public abstract class OhReceivePort extends OhPort
{
  // @begin {field} ------------------------------------------------------

  /**
   * <japanese>
   * <a href="OhSenderPort.html">OhSenderPort</a> ^ɃLXgꂽM[qłB
   * </japanese>
   * <p>
   * A sender which sends datapacks to the current object.
   *
   */
  protected OhSenderPort _sender;
  
  // @end {field} --------------------------------------------------------
  
  
  // @begin {abstract methods} -------------------------------------------

  // 1. connection function.

  /**
   * <japanese>
   * Mf[^pbÑf[^ľ^\Ԃ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   *  OhRelayPort łۃ\bhƂĂ܂B
   * OhRelayPort ̔hNXŎMf[^pbN^\Ԃ悤ɕK`ĉB
   *
   * Ⴆ΁AMf[^pbN^ "double" ^łhNXł́A
   *
   * <pre>
   *  public String dataaType() {
   *    return "double";
   *  }
   * </pre>
   *
   * ̂悤ɒ`܂B<p>
   *
   * ڑɂ́A̒[qƑM[q̃f[^pbN^v邱ƂmF܂B
   *
   * <pre>
   * [RETURN]
   *  EString -> pf[^pbN^\
   * </pre>
   * </japanese>
   * <p>
   * Returns a string which expresses the type of datapacks received by the current object.
   * This is a abstract method.
   * This function must be defined in derived classes.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();

  
  // @end {abstract methods} ---------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * </japanese>
   * <p>
   * Initialization with object name "aPortName" and object number "aPortNum".
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------
  
  
  // @begin {constructors} -----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^
   * <a href="OhReceivePort.html#init(String, int)">OhReceivePort.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhReceivePort.html#init(String, int)">OhReceivePort.init(String, int)</a> 
   * păoϐĉB
   * </jananese>
   * <p>
   *
   */
  public OhReceivePort() {
    super();
    _sender = null;
    
    _className = "OhReceivePort";
  }


  /**
   * <japanese>
   * NX <a href="OhPort.html">OhPort</a> ̃ftHgRXgN^pA
   * <a href="OhReceivePort.html#init(String, int)">OhReceivePort.init(String, int)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhReceivePort"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _sender            : null
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhReceivePort(String aPortName, int aPortNum) throws OhError {
    super();
    _sender = null;
    
    _className = "OhReceivePort";
    
    init(aPortName, aPortNum);
  }

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  // 1. connection function.

  /**
   * <japanese>
   * M[qݒ肵܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   * M[q _sender  aSender ɐݒ肵Atrue Ԃ܂B<p>
   *
   * L̏ꍇ̓G[bZ[W\Afalse Ԃ܂B<p>
   *
   * <ul>
   *  <li> aSender  null ̏ꍇB
   *  <li> ̒[q̑M[q _sender ݒς݂̏ꍇB
   *  <li> ̒[q aSender Ńf[^pbN^قȂꍇB
   * </ul>
   *
   * <pre>
   * [ARGUMENT]
   *  EaSender -> Object ^ɃLXgꂽM[qB
   *
   * [RETURN]
   *  Etrue --> M[q _sender ɐݒ肳ꂽꍇB
   *  Efalse -> aSender  null ̏ꍇB
   *             ̒[q̑M[q _sender ݒς݂̏ꍇB
   *             ̒[q aSender Ńf[^pbN^قȂꍇB
   * [ACCIDENT]
   *  EaSender  <a href="OhPort.html">OhPort</a> ^ϐi[ĂȂꍇB
   * </pre>
   * </japanese>
   * <p>
   *
   * Set "_sender" to "aSender" and return "true".
   *
   * When one of the following cases occurs <p>
   * <ul>
   *  <li> "aSender" is NULL.
   *  <li> "_sender" has been set already.
   *  <li> Data pack types between the current object and the sender are different.
   * </ul>
   * <p>
   * this function prints an error message and returns "false".
   *
   * @param aSender an <code>Object</code> value
   * @return a <code>boolean</code> value
   */
  public boolean setSender(Object aSender) throws OhError {   //; (*1)
    if(aSender == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException("Cannot set \"sender\"." + str);
    }
    
    if(_sender != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)_sender).cPrint(out, " >    connected to  : ");
      ((OhObject)aSender).cPrint(out, " >    connecting to  : ");
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("\"sender\" has already been set. Doubly setting?", str);
    }
    
    final String sendType    = ((OhPort)aSender).dataType();
    final String receiveType = dataType();

    if(!(sendType.equals(receiveType))) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)aSender).oPrint(out, " >    s > ");
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("'dataPackType' of this port and \"sendType\"(arg) are different" + str);
     }
    
    _sender = (OhSenderPort)aSender;
    
    return true;
  }


  /**
   * <japanese>
   * ڑƌ̏sȂ܂B<p>
   *
   * OhReceivePort ł͐ڑƌɍsȂׂ͓ɂ܂B
   *  true Ԃ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue ->  true ԂB
   * </pre>
   * </japanese>
   * <p>
   * Because no corresponding work is needed in the <code>OhReceivePort</code> class,
   * Switch of the <code>OhReceivePort</code> class simply returns "false". <p>
   * 
   * Note: 
   * 
   * Switch of the <code>OhSenderPort</code> class is defined differently.
   *
   * @return a <code>boolean</code> value
   */
  public boolean canYouSwitch() {
    return true;
  }


  /**
   * <japanese>
   * vZƑO̊mFsȂ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhĂ܂B<br>
   * f[^pbNM邱ƂłԂł邱ƂmF܂B
   * ܂A_sender ݒ肳Ă邩`FbN܂B
   * _sender ݒ肳Ă΁A_sender f[^pbNM邱Ƃł܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> _sender ݒς݂̏ꍇB
   *  Efalse -> _sender ݒ̏ꍇB
   * </pre>
   * </japanese>
   * <p>
   * Answers whether the current receive port has been connected to some sender or not.
   *
   * @return a <code>boolean</code> value
   */
  public boolean areYouReady() {
    return (_sender != null) ? true : false;
  }


  // 2. function for receiving datapacks.

  /**
   * <japanese>
   * f|^pbN̔z񃊃Xgݒ肵܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB
   * _sender Ƀf[^pbNz쐬悤߂܂B
   * </japanese>
   * <p>
   * Orders the sender to create a datapack array.
   *
   */
  public void setDataPackArray() {
    _sender.setDataPackArray();
  }


  /**
   * <japanese>
   * <japanese>
   * ŌẪf[^̋L^Ԃ܂B
   *
   * M[q(_sender)ɍŌẪf[^̋L^Ԃ悤w܂B
   * </japanese>
   * <p>
   * Returns the data time of the oldest data pack in the data stored in the sender.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getOldestDataTime() {
    return _sender.getOldestDataTime();
  }

  /**
   * <japanese>
   * <japanese>
   * ŐṼf[^̋L^Ԃ܂B
   *
   * M[q(_sender)ɍŐṼf[^̋L^Ԃ悤w܂B
   * </japanese>
   * <p>
   * Returns the data time of the newest data pack in the data stored in the sender.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNewestDataTime() {
    return _sender.getNewestDataTime();
  }
  

  /**
   * <japanese>
   * f[^pbŇ(_numberOfDataPacks)Ԃ܂B
   *
   * M[q(_sender)Ƀf[^pbŇ(_numberOfDataPacks)Ԃ悤w܂B
   * </japanese>
   * <p>
   *
   * Returns the number of datapacks stored in the sender.
   *
   * @return an <code>int</code> value
   */
  public int getNumberOfDataPacks() {
    return _sender.getNumberOfDataPacks();
  }
  

  /**
   * <japanese>
   * f[^^̊mFsȂ܂B
   *
   * ܂A擾\f[^pbN̊mFsȂ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB<p>
   *
   * _sender ɖ₢킹 aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNM\mF܂B
   * M\Ȃ true ԂAMs\Ȃ false Ԃ܂B<p>
   *
   * ̎M[q _owner ŌvZ̊mFƂɗp܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaTFrom -> aTFrom ȑOf[^pbNM\ł邱ƂmFB
   *  EaTTo ---> aTTo ȍ~܂Ńf[^pbNM\ł邱ƂmFB
   *
   * [RETURN]
   *  Etrue ---> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNp\ȏꍇB
   *  Efalse --> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNps\ȏꍇB
   *
   * [ACCIDENT]
   *  EڑOŁA_sender  null ̏ꍇB
   *    ĩ\bh͌vZɕpɂɗp\̂Ŋ _sender  null łȂƂmF܂łBj
   *  E_sender  <a href="OhPort.html">OhPort</a> ^ȊO̕ϐi[ĂꍇB
   * </pre>
   * </japanese>
   * <p>
   * Tells whether the data from "aTFrom" to "aTTo" can be obtained or not. <p>
   * 
   * Accident: 
   * 
   * "_sender" is "null" or not <a href="OhPort.html">OhPort</a> type.
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   * @param aTTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetData(OhTimePoint aTFrom, OhTimePoint aTTo) {
    return _sender.canYouGetData(aTFrom, aTTo);
  }


  /**
   * <japanese>
   * 擾\f[^pbN̊mFsȂ܂B<p>
   * </japanese>
   * <p>
   * Tells whether the data at "aTFromTo" can be obtained or not. <p>
   * 
   * Accident: 
   * 
   * "_sender" is "null" or not <a href="OhPort.html">OhPort</a> type.
   *
   * @param aTFromTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetData(OhTimePoint aTFromTo) {
    return _sender.canYouGetData(aTFromTo, aTFromTo);
  }


  /**
   * <japanese>
   * f[^pbN̔z񃊃Xg擾(M)܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB<p>
   *
   * M[q _sender f[^pbNzM܂B<p>
   *
   *  aDpArray ̑vfɂ Object ^ɃLXgꂽ
   * <a href="OhDataPack.html">OhDataPack</a> ^ϐ̔z񂪊i[A
   * aNumOfDp ̑vfɂ aDpArray ̗vfi[܂B<p>
   *
   * aDpArray ̎Mf[^pbÑf[^L^AĂ邱Ƃ
   * <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a>
   * ɂۏ؂܂B<p>
   *
   * Kvȃf[^pbNM\ۂ
   * canYouGetData(OhTimePoint, OhTimePoint) pĊmFł܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaDpArray -> pf[^pbNzvfɊi[B
   *  EaNumOfDp -> aDpArray ̗vfvfɊi[B
   *
   * [ACCIDENT]
   *  EڑO _sender  null ̏ꍇB
   *    ĩ\bh͌vZɕpɂɗp\̂Ŋ _sender  null łȂƂmF܂łBj
   *  E_sender  <a href="OhPort.html">OhPort</a> ^ȊO̕ϐi[ĂꍇB
   * </pre>
   * </japanese>
   * <p>
   * Get a data pack array from the sender.
   * After the execution of this function, "aDpArray" stores the data pack array
   * and "aNumOfDp" stores the dimention of "aDpArray". <p>
   * 
   * Accident: 
   * 
   * "_sender" is "null" or not <a href="OhPort.html">OhPort</a> type.
   *
   * @param aDpArray a <code>Vector</code> value
   * @param aNumOfDp an <code>int[]</code> value
   */
  public void getDataPackArray(OhArrayList[] aDpArray, int[] aNumOfDp) {
    _sender.getDataPackArray(aDpArray, aNumOfDp);
  }

  
  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p>
   *
   * w肵 _sender f[^pbNM܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbNMAi[܂B<br>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbNMAi[܂B<br>
   *
   * aFlag ͎擾̃[hI邽߂̃IvVŁAL̒ʂłB
   *
   * <pre>
   *   E"b"       aFormer, aLater ̗KMB
   *               MłȂꍇ́AƂ null ɐݒ肷B
   *   E"f"       aLater MłȂĂAaFormer ̂ݎMB
   *               aLater MłȂꍇAnull ɐݒ肷B
   *   E"l"       aFormer MłȂĂAaLater ̂ݎ擾B
   *               aFormer MłȂꍇAnull ɐݒ肷B
   *   Ȇ    aFormer, aLater ̂ȂƂ擾B
   *               MłȂ̂ null ɐݒ肷B
   * </pre>
   *
   * aFlag ̃ftHgĺA "b" łB
   * ܂A  aLater nȂꍇ́A aFlag  f Ƃ aFormer ̂ݎMA
   *  aFormer nȂꍇ́A aFlag  l Ƃ aLater ̂ݎM܂B
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B
   *
   * 𖞂f[^pbN擾łȂꍇAaFormer, aLater  null ɐݒ肳܂B
   *
   * f[^pbN擾łꍇ́Atrue Ԃ܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaFormer ---> _dataTime ȑO _dataTime ɍł߂^f[^pbNvfɊi[B
   *  E_dataTime -> w莞 [sec]
   *                 ̎ɍł߂f[^L^^f[^pbNMB
   *  EaLater ----> _dataTime ȍ~ _dataTime ɍł߂^f[^pbNvfɊi[B
   *  EaMode -----> f[^pbN̎Mw肷tOB
   *
   * [RETURN]
   *  Etrue --> w̃f[^pbNMłꍇB
   *  Efalse -> _dataPackArray  null ̏ꍇB
   *             w̃f[^pbNMłȂꍇB
   * </pre>
   * </japanese>
   *
   * <p>
   *
   * Get a data pack at specified time "_dataTime".
   * The results are stored in "aFormer" and "aLater".
   *
   * <ul>
   *  <li> "aFormer" is the newest data before "_dataTime"
   *  <li> "aLater" is the pointer to the oldest data after "_dataTime"
   * </ul>
   *
   * When a data pack just at "_dataTime" exists, "aFormer" and "aLater" point to that data pack.
   * <p>
   * "aFlag" specifies the search option.
   *
   * <dl>
   *  <dt> "b"
   *   <dd>  Receive both of "former" and "later".
   *
   *  <dt> "f"
   *   <dd>   Receive "aFormer". Even if "aLater" cannot be obtained, the function returns "true".
   *
   *  <dt> "l"
   *   <dd>  Receive "aLater". Even if "aFormer" cannot be obtained, the function returns "true".
   *
   *  <dt> other character
   *   <dd>  Receive at lease one of "aFormer" and "aLater".
   * </dl>
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @param aMode a <code>char</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater, char aMode) {
    return _sender.getDataPack(aFormer, _dataTime, aLater, aMode);
  }


  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p>
   *
   * w肵 _sender f[^pbNM܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbNMAi[܂B<br>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbNMAi[܂B<br>
   *
   * aFormer, aLater ̗KM܂B
   * MłȂꍇ́AƂ null ɐݒ肳܂B
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B
   *
   * f[^pbN擾łꍇ́Atrue Ԃ܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaFormer ---> _dataTime ȑO _dataTime ɍł߂^f[^pbNvfɊi[B
   *  E_dataTime -> w莞 [sec]
   *                 ̎ɍł߂f[^L^^f[^pbNMB
   *  EaLater ----> _dataTime ȍ~ _dataTime ɍł߂^f[^pbNvfɊi[B
   *
   * [RETURN]
   *  Etrue --> w̃f[^pbNMłꍇB
   *  Efalse -> _dataPackArray  null ̏ꍇB
   *             w̃f[^pbNMłȂꍇB
   * </pre>
   * </japanese>
   * <p>
   * By all means, Get a data pack at specified time "_dataTime".
   * The results are stored in "aFormer" and "aLater".
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater) {
      return _sender.getDataPack(aFormer, _dataTime, aLater, 'b');
  }


  /**
   * <japanese>
   * f[^KvԂ܂B<p>
   *
   * M̑M[q getNecessaryTimeFrom() ȍ~ getNecessaryTimeTo() ȑÕf[^pbN
   * Ȃ悤Ɏw邽߂ɒ`܂B<p>
   *
   * _owner 
   * <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a>
   * Ԃ܂B
   * _owner  null ̏ꍇ
   * <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a>
   * Ԃ܂B
   * </japanese>
   * <p>
   * Returns the "NecessaryTimeFrom" (this) of the owner of the current receive port.
   * You can redefine it in derived classes.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeFrom() {    //; (*2)
    return (_owner != null) ? ((OhElement)_owner).getNecessaryTimeFrom(this) : OhTimePoint.OhTimePointMin;
  }

  /**
   * <japanese>
   * f[^KvԂ܂B<p>
   *
   * M̑M[q getNecessaryTimeFrom() ȍ~ getNecessaryTimeTo() ȑÕf[^pbN
   * Ȃ悤Ɏw邽߂ɒ`܂B<p>
   *
   * _owner 
   * <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a>
   * Ԃ܂B
   * _owner  null ̏ꍇ
   * <a href="OhTimePoint.html#OhTimePointMax">OhTimePoint.OhTimePointMax</a>
   * Ԃ܂B<p>
   * </japanese>
   * <p>
   * Returns the "NecessaryTimeTo" (this) of the owner of the current receive port.
   * You can redefine it in derived classes.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeTo() {      //; (*3)
    return (_owner != null) ? ((OhElement)_owner).getNecessaryTimeTo(this) : OhTimePoint.OhTimePointMax;
  }


  /**
   * <japanese>
   * M[qւ̌vZv͑M[qɓ]܂B
   * </japanese>
   * <p>
   * Relay the calculation request to the sender.
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError {
    _sender.calReq();
  }


  /**
   * <japanese>
   * svf[^pbN܂B
   *
   * M[q _sender ɕsvf[^pbN悤w܂B<p>
   * </japanese>
   * Remove unnecessary data packs from the memory.
   *
   */
  public void reduceDataPackStock() {
    _sender.reduceDataPackStock();
  }


  // 3. Information output function.

  /**
   * <japanese>
   * ڑo͂܂B
   * </japanese>
   * <p>
   * Print the information about the connection to "aPw" with the string "aHead".
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printConnection(PrintWriter aPw, String aHead) {    //; (*4)
    oPrint(aPw, aHead);
    
    if(_sender == null) {
      aPw.print(aHead + "\tNot connected." + NEWLINE);
      return;
    }
    
    String tmp = aHead;

    tmp = tmp + "Connected > ";

    _sender.oPrint(aPw, tmp);
  }

  // @end {methods} ------------------------------------------------------
  
} // OhReceivePort
