/**
 * @(#) OhRelayPort; class to express relay ports.
 *                   Make "OhRelayPort.java" from "ohrelay.h" and "ohrelay.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * <p>
 * {^p[qf\NXłB<p>
 *
 * ۃNXłB<p>
 *
 * NX OhRelayPort ł́A
 *
 * <ul>
 *  <li> f[^pbNM[q : _sender
 * </ul>
 *
 * LA
 *
 * <ul>
 *  <li> M[qƂ̐ڑ@\
 *  <li> M[qƂ̐ڑ@\
 * </ul>
 *
 * `܂B
 * M[qƂ́A̒p[qɃf[^𑗂Ă鑤̒[qłB
 * M[qƂ́A̒p[q瑗f[^󂯂Ƃ鑤̒[qłB
 *
 * OhRelayPort ɐڑł鑗M[q͂PłB
 * OhRelayPort ɂ͕̑M[qڑł܂B
 *
 * </japanese>
 * <p>
 *
 * The OhRelayPort class is a model of relay ports. <p>
 *
 * The OhRelayPort has a instance variable "_sender".
 * "_sender" is used to store the sender
 * which sends data packs to the current port. <p>
 *
 * Member functions for
 *
 * <ul>
 *  <li> Connecting the current port with the port which sends data packs to the current port
 *  <li> Connecting the current port with the port which receives data packs from the current port
 * </ul>
 *
 * are defined. <p>
 *
 * Only one port can be connected to the port for receiving data packs.
 * More than one ports can be connected to the ports
 * which sending data packs.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.45 $ $Date: 2004/02/22 10:28:21 $
 */
public abstract class OhRelayPort extends OhSenderPort
{
  // @begin {field} ------------------------------------------------------

  /**
   * <japanese>
   * <a href="OhSenderPort.html">OhSenderPort</a> ^ɃLXgꂽM[qłB
   * Kvȃf[^pbŃAڑł _sender 擾܂B
   * </japanese>
   * <p>
   * The sender which sends data packs to this OhRelayPort casted to type OhSenderPort.
   *
   */
  protected OhSenderPort _sender;

  // @end {field} --------------------------------------------------------
  
  
  // @begin {abstract methods} -------------------------------------------

  /**
   * <japanese>
   * pf[^pbÑf[^ľ^\Ԃ܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p>
   *
   *  OhRelayPort łۃ\bhƂĂ܂B
   * OhRelayPort ̔hNXŒpf[^pbN^\Ԃ悤ɕK`ĉB<p>
   *
   * Ⴆ΁Apf[^pbN^ "double" ^łhNXł́A
   *
   * <pre>
   *  public String dataaType() {
   *    return "double";
   *  }
   * </pre>
   *
   * ̂悤ɒ`܂B<p>
   *
   * ڑɂ́Aڑ[qƂ̒[q̃f[^pbN^v邱ƂmF܂B
   *
   * <pre>
   * [RETURN]
   *  EString -> pf[^pbN^\
   * </pre>
   * </japanese>
   * <p>
   *
   * Returns the string which shows the type of data packs
   * which are treated by this OhRelayPort. <p>
   *
   * In derived classes of this class OhRelayPort,
   * this funstion should be defined so that it returnts the datapack type string.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();

  // @end {abstract methods} ---------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   *
   * RXgN^Ŏgp܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return ((OhPort)this).init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------

  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^
   * <a href="OhRelayPort.html#init(String, int)">OhRelayPort.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhRelayPort.html#init(String, int)">OhRelayPort.init(String, int)</a> 
   * păoϐĉB
   * </jananese>
   *
   */
  public OhRelayPort() {
    super();
    _sender    = null;
    _className = "OhRelayPort";
  }

  
  /**
   * <japanese>
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̃ftHgRXgN^pA
   * <a href="OhRelayPort.html#init(String, int)">OhRelayPort.init(String, int)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhRelayPort"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _owner             : null
   *  <li> _sender            : null
   *  <li> _bufferList        : ɊmۂBiڑƗpj
   *  <li> _receiverArray     : null
   *  <li> _numberOfReceivers : -1
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public OhRelayPort(String aPortName, int aPortNum) throws OhError {
    super();
    _sender    = null;
    _className = "OhRelayPort";

    init(aPortName, aPortNum);
  }

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  // 1. connection facility
  
  /**
   * <japanese>
   * _sender ̃f[^Ԍ^xNg擾܂B
   * </japanese>
   * <p>
   * Returns the data connection type of the "_sender".
   *
   * @return a <code>Vector</code> value
   */
  public Vector getDctVec() {
    return _sender.getDctVec();
  }


  /**
   * <japanese>
   * M[qݒ肵܂B<p>
   *
   * M[q _sender  aSender ɐݒ肵Atrue Ԃ܂B<p>
   *
   * L̏ꍇ̓G[bZ[W\Afalse Ԃ܂B<p>
   *
   * <ul>
   *  <li> aSender  null ̏ꍇB
   *  <li> ̒[q̑M[q _sender ݒς݂̏ꍇB
   *  <li> ̒[q aSender Ńf[^pbN^قȂꍇB
   * </ul>
   *
   * <pre>
   * [ARGUMENT]
   *  EaSender -> Object ^ɃLXgꂽM[qB
   *
   * [RETURN]
   *  Etrue --> M[q _sender ɐݒ肳ꂽꍇB
   *  Efalse -> aSender  null ̏ꍇB
   *             ̒[q̑M[q _sender ݒς݂̏ꍇB
   *             ̒[q aSender Ńf[^pbN^قȂꍇB
   * [ACCIDENT]
   *  EaSender  <a href="OhPort.html">OhPort</a> ^ϐi[ĂȂꍇB
   * </pre>
   * </japanese>
   * <p>
   *
   * Sets the sender of this OhRelayPort to aSender (arg). <p>
   *
   * [in] aSender - the sender casted to Object. <p>
   *
   * Returns "true" if the setting was successfull.
   * For the following cases, it returns "false". <p>
   *
   * <ul>
   *  <li> aSender is null.
   *  <li> already the sender has been set.
   *  <li> the type of datapacks which "aSender" treats is different from
   *       that of datapacks which this ports treats.
   * </ul>
   *
   *
   * @param aSender an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean setSender(Object aSender) throws OhError {  //; (*1)
    if (aSender == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException("cannot set sender." + str);
    }

    if (_sender != null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)_sender).cPrint(out, " >    connected to  : ");
      ((OhObject)aSender).cPrint(out, " >    connecting to  : ");
      String str = buffer.toString();
      out.close();
      throw new OhError("sender has already been set.", str);
    }

    // Gets the datapack type which this port receives, and the datapack type which "aSender" receives.
    //
    String sType = ((OhPort)aSender).dataType();
    String rType = dataType();

    if (!sType.equals(rType)) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)aSender).oPrint(out, " >  s  > ");
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("data pack type of this port and \"aSender\" are different.", str);
    }

    _sender = (OhSenderPort)aSender;

    return true;
  }
      

  /**
   * <japanese>
   * vZƑO̊mFsȂ܂B<p>
   *
   * f[^pbN𒆌p\ȏԂł邱ƂmF܂B
   * _sender ݒ肳Ă _sender f[^pbN𒆌pł܂B
   * _numberOfReceivers  0 ȏȂ _receiverArray ݒς݂ŁA
   * M[qɃf[^pbN𑗐Mł܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> _sender, _receiverArray ݒς݂̏ꍇB
   *  Efalse -> _sender, _receiverArray ̏ȂƂݒ̏ꍇB
   * </pre>
   * </japanese>
   * <p>
   *
   * Reports whether this OhRelayPort is ready or not.
   * When "_sender" and "_numberOfReceivers" have been defined,
   * it returns true, otherwise it returns false.
   *
   * @return a <code>boolean</code> value
   */
  public boolean areYouReady() {
    return ((_sender != null) && (_numberOfReceivers >= 0)) ? true : false;
  }
  

  // 2. relay facility of data packs

  /**
   * <japanese>
   * f|^pbN̔z񃊃Xgݒ肵܂B <p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB
   * _sender Ƀf[^pbNzݒ肷悤߂܂B
   * </japanese>
   * <p>
   * Orders the sender to create a data pack array.
   *
   */
  public void setDataPackArray() {
    _sender.setDataPackArray();
  }


  /**
   * <japanese>
   * ŌẪf[^̋L^Ԃ܂B
   * </japanese>
   * <p>
   * Returns the oldest data time of the data stocked in the sender.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getOldestDataTime() {
    return _sender.getOldestDataTime();
  }

  /**
   * <japanese>
   * ŐṼf[^̋L^Ԃ܂B
   * </japanese>
   * <p>
   * Returns the newest data time of the data stocked in the sender.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNewestDataTime() {
    return _sender.getNewestDataTime();
  }


  /**
   * <japanese>
   * f[^pbŇ(_numberOfDataPacks)Ԃ܂B
   * </japanese>
   * <p>
   * Returns the number of data packs which stored in the sender.
   *
   * @return an <code>int</code> value
   */
  public int getNumberOfDataPacks() {
    return _sender.getNumberOfDataPacks();
  }


  /**
   * <japanese>
   * f[^^̊mFsȂ܂B
   *
   * ܂A擾\f[^pbN̊mFsȂ܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p>
   *
   * _sender ɖ₢킹 aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNM\mF܂B
   * M\Ȃ true ԂAMs\Ȃ false Ԃ܂B<p>
   *
   * ̃\bh́AM[q canYouGetData(OhTimePoint, OhTimePoint) Ăяo܂B
   * ɂAM[qŃf[^pbNMł邩mFł܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaTFrom -> aTFrom ȑOf[^pbNM\ł邱ƂmFB
   *  EaTTo ---> aTTo ȍ~܂Ńf[^pbNM\ł邱ƂmFB
   *
   * [RETURN]
   *  Etrue ---> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNp\ȏꍇB
   *  Efalse --> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNps\ȏꍇB
   *
   * [ACCIDENT]
   *  EڑOŁA_sender  null ̏ꍇB
   *    ĩ\bh͌vZɕpɂɗp\̂Ŋ _sender  null łȂƂmF܂łBj
   *  E_sender  <a href="OhPort.html">OhPort</a> ^ȊO̕ϐi[ĂꍇB
   * </pre>
   * </japanese>
   * <p>
   * Reports whether this OhRelayPort can get the data from aTFrom (arg) to aTTo (arg). <p>
   *
   * [in] aTFrom  - start time
   * [in] aTTo    - end time
   *
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   * @param aTTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetData(OhTimePoint aTFrom, OhTimePoint aTTo) {
    return _sender.canYouGetData(aTFrom, aTTo);
  }


  /**
   * <japanese>
   * p[qւ̌vZv͑M[qɓ]܂B
   * </japanese>
   * <p>
   * The calculation request to a relay port is transmitted to a send port.
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError {
    _sender.calReq();
  }


  /**
   * <japanese>
   * f[^pbNz񃊃Xg擾(M)܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p>
   *
   * M[q _sender f[^pbNzM܂B<p>
   *
   *  aDpArray ̑vfɂ Object ^ɃLXgꂽ
   * <a href="OhDataPack.html">OhDataPack</a> ^ϐ̔z񂪊i[A
   * aNumOfDp ̑vfɂ aDpArray ̗vfi[܂B<p>
   *
   * ̃\bh́AM[q getDataPackArray Ăяo܂B
   * ɂAM[qŃf[^pbNzMł܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaDpArray -> pf[^pbNzvfɊi[B
   *  EaNumOfDp -> aDpArray ̗vfvfɊi[B
   *
   * [ACCIDENT]
   *  EڑO _sender  null ̏ꍇB
   *    ĩ\bh͌vZɕpɂɗp\̂Ŋ _sender  null łȂƂmF܂łBj
   *  E_sender  <a href="OhPort.html">OhPort</a> ^ȊO̕ϐi[ĂꍇB
   * </pre>
   * </japanese>
   * <p>
   * Get datapack array aDpArray (arg) and the number of datapacks aNumOfDp (arg) from the sender. <p>
   *
   * <ul>
   *  <li> [out] aDpArray  - datapack array
   *  <li> [out] aNumOfDp  - the number of datapacks
   * </ul>
   *
   *
   * @param aDpArray an <code>OhArrayList[]</code> value
   * @param aNumOfDp an <code>int[]</code> value
   */
  public void getDataPackArray(OhArrayList[] aDpArray, int[] aNumOfDp) {
    _sender.getDataPackArray(aDpArray, aNumOfDp);
  }


  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p>
   *
   * w肵 _sender f[^pbNM܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbNMAi[܂B<p>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbNMAi[܂B<p>
   *
   * aFlag ͎擾̃[hI邽߂̃IvVŁAL̒ʂłB<p>
   *
   * <pre>
   *   E"b"       aFormer, aLater ̗KMB
   *               MłȂꍇ́AƂ null ɐݒ肷B
   *   E"f"       aLater MłȂĂAaFormer ̂ݎMB
   *               aLater MłȂꍇAnull ɐݒ肷B
   *   E"l"       aFormer MłȂĂAaLater ̂ݎ擾B
   *               aFormer MłȂꍇAnull ɐݒ肷B
   *   Ȇ    aFormer, aLater ̂ȂƂ擾B
   *               MłȂ̂ null ɐݒ肷B
   * </pre>
   *
   * aFlag ̃ftHgĺA "b" łB<p>
   * ܂A  aLater nȂꍇ́A aFlag  f Ƃ aFormer ̂ݎMA
   *  aFormer nȂꍇ́A aFlag  l Ƃ aLater ̂ݎM܂B<p>
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B<p>
   *
   * 𖞂f[^pbN擾łȂꍇAaFormer, aLater  null ɐݒ肳܂B<p>
   *
   * f[^pbN擾łꍇ́Atrue Ԃ܂B
   *
   * ̃\bh͑M[q
   * getDataPack(Object[], OhTimePoint, Object[], char) Ăяo܂B
   * ɂ著M[qŃf[^pbNMł܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaFormer ---> _dataTime ȑO _dataTime ɍł߂^f[^pbNvfɊi[B
   *  E_dataTime -> w莞 [sec]
   *                 ̎ɍł߂f[^L^^f[^pbNMB
   *  EaLater ----> _dataTime ȍ~ _dataTime ɍł߂^f[^pbNvfɊi[B
   *  EaMode -----> f[^pbN̎Mw肷tOB
   *
   * [RETURN]
   *  Etrue --> w̃f[^pbNMłꍇB
   *  Efalse -> _dataPackArray  null ̏ꍇB
   *             w̃f[^pbNMłȂꍇB
   * </pre>
   * </japanese>
   * <p>
   *
   * Get two datapacks aFormer and aLater.
   * aFormer is the newest datapack before time_dataTime.
   * aLater is the oldest datapack after time _dataTime.
   * aMode specifies the option <p>
   *
   * <ul>
   *  <li> "b" -- get both of aFormer and aLater
   *  <li> "f" -- get aFormer at least
   *  <li> "l" -- get aLater at least
   * </ul>
   * <p>
   *
   * When the datapack(s) specified by aMode can be got,
   * it returns "true", otherwise "false". <p>
   *
   * [out] aFormer --- pointer to a datapack
   * [in]  _dataTime - specified time
   * [out] aLater ---- pointer to a datapack
   * [in]  aMode ----- option to get pointers to datapacks
   *
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @param aMode a <code>char</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater, char aMode) {
    return _sender.getDataPack(aFormer, _dataTime, aLater, aMode);
  }


  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p>
   *
   * w肵 _sender f[^pbNM܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbNMAi[܂B<p>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbNMAi[܂B<p>
   *
   * aFormer, aLater ̗KM܂B
   * MłȂꍇ́AƂ null ɐݒ肳܂B
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B<p>
   *
   * f[^pbN擾łꍇ́Atrue Ԃ܂B
   *
   * ̃\bh͑M[q
   * getDataPack(Object[], OhTimePoint, Object[], char) Ăяo܂B
   * ɂ著M[qŃf[^pbNMł܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaFormer ---> _dataTime ȑO _dataTime ɍł߂^f[^pbNvfɊi[B
   *  E_dataTime -> w莞 [sec]
   *                 ̎ɍł߂f[^L^^f[^pbNMB
   *  EaLater ----> _dataTime ȍ~ _dataTime ɍł߂^f[^pbNvfɊi[B
   *
   * [RETURN]
   *  Etrue --> w̃f[^pbNMłꍇB
   *  Efalse -> _dataPackArray  null ̏ꍇB
   *             w̃f[^pbNMłȂꍇB
   * </pre>
   * </japanese>
   * <p>
   * By all means, Get two datapacks aFormer and aLater.
   *
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater) {
    return _sender.getDataPack(aFormer, _dataTime, aLater, 'b');
  }


  /**
   * <japanese>
   * svf[^pbN܂B<p>
   *
   * M[q _sender ɕsvf[^pbN悤w܂B<p>
   *
   * ̒p[q _owner ŌvZIɗpAsv܂B<p>
   *
   * ̃\bh͑M[q
   * getDataPack(Object[], OhTimePoint, Object[], char) Ăяo܂B
   * ɂ著M[qŃf[^pbNMł܂B
   * </japanese>
   * <p>
   * Forwards the message "reduceDataPackStock()" to the sender.
   *
   */
  public void reduceDataPackStock() {
    _sender.reduceDataPackStock();
  }


  
  // 3. information output function.

  /**
   * <japanese>
   * <p>
   * ڑo͂܂B
   * </japanese>
   * <p>
   * Prints the information about connection of this Relay_port.
   * This function is used for debugging.
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printConnection(PrintWriter aPw, String aHead) {    //; (*2)
    oPrint(aPw, aHead);

    if (_sender == null) {
      aPw.print(aHead + "\tNot connected to sender." + NEWLINE);
      return;
    }

    StringBuffer sb = new StringBuffer("");

    if (aHead != null)
      sb.append(aHead);

    sb.append("Connected > ");
    String str = sb.toString();

    ((OhObject)_sender).oPrint(aPw, str);

    if (_numberOfReceivers <= 0)
      aPw.print(aHead + "\tNot connected to receiver." + NEWLINE);

    for (int k = 0; k < _numberOfReceivers; k++)
      ((OhObject)_receiverArray[k]).oPrint(aPw, str);
  }


  /**
   * <japanese>
   * <p>
   * ڑo͂܂B
   * </japanese>
   * <p>
   * Prints the information about connection of this Relay_port.
   * This function is used for debugging.
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void printConnection(PrintWriter aPw) {
    oPrint(aPw, "    ");

    if (_sender == null)
      aPw.print("    \tNot connected to sender." + NEWLINE);

    StringBuffer sb = new StringBuffer("    ");

    sb.append("Connected > ");
    String str = sb.toString();

    ((OhObject)_sender).oPrint(aPw, str);

    if (_numberOfReceivers <= 0)
      aPw.print("    \tNot connected to receiver." + NEWLINE);

    for (int k = 0; k < _numberOfReceivers; k++)
      ((OhObject)_receiverArray[k]).oPrint(aPw, str);
  }

  // @end {methods} ------------------------------------------------------

} // OhRelayPort
