/**
 * @(#) OhSendInt; class to express a port which send datapacks of type int.
 *                    Make "OhSendInt.java" from "ohp_tmp.h" and "ohp_tmp.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * OhSendInt;
 *
 * OhSendInt is the base class to express a port which send datapacks of type int.
 *
 * <japanese>
 * <p>
 * int ^̃f[^pbN𑗐M鑗M[q̃NX <p>
 *
 * <a href="OhReceivePort.html">OhReceivePort</a> pč쐬M[q̃NXłB
 * </japanese>
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.1 $ $Date: 2004/01/17 13:13:31 $
 */
public class OhSendInt extends OhSendPort
{
  // @begin{initialization} ----------------------------------------------

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  /**
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum, Vector aDctVec) throws OhError {
    return super.init(aPortName, aPortNum, aDctVec);
  }

  // @end{initialization} ------------------------------------------------

  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * Creates a new default <code>OhSendInt</code> instance.
   *
   * @exception OhError if an error occurs
   */
  public OhSendInt() throws OhError {
    super();
    _className = "OhSendInt";
  }


  /**
   * Creates a new <code>OhSendInt</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSendInt(String aPortName, int aPortNum) throws OhError {
    super(aPortName, aPortNum);
    _className = "OhSendInt";
  }

  
  /**
   * Creates a new <code>OhSendInt</code> instance.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @exception OhError if an error occurs
   */
  public OhSendInt(String aPortName, int aPortNum, Vector aDctVec) throws OhError {
    super(aPortName, aPortNum, aDctVec);
    _className = "OhSendInt";
  }

  // @end {constructors} -------------------------------------------------


  // @begin {methods} ----------------------------------------------------

  // 0. copy method.
  
  /**
   * Copy method.
   *
   * @param aSd an <code>OhSendPort</code> value
   */
  public void copy(OhSendPort aSd) {
    _className    = aSd._className;
    _objectName   = aSd._objectName;
    _objectNumber = aSd._objectNumber;
    _owner        = aSd._owner;

    _bufferList        = aSd._bufferList;
    _receiverArray     = aSd._receiverArray;
    _numberOfReceivers = aSd._numberOfReceivers;
    _commentString     = aSd._commentString;

    _dataPackArray       = aSd._dataPackArray;
    _numberOfDataPacks   = aSd._numberOfDataPacks;
    _protectPastDataFlag = aSd._protectPastDataFlag;
    _dctVec              = aSd._dctVec;
  }

  
  // 1. function for sending data.
  
  /**
   * Returns the data pack type which expresses the type of data to be received.
   *
   * @return a <code>String</code> value
   */
  public String dataType() {
    return "int";
  }


  /**
   * Send data whose data time is "aT" and data value is "aValue".
   *
   * <japanese>
   * <p>
   * ɂƂ aT, aValue 
   * <a href="OhDataPackInt.html">OhDataPackInt</a> ^f[^pbN쐬A
   * <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @param aValue a <code>int</code> value
   */
  public void sendData(OhTimePoint aT, int aValue) {    //; (*1)
    
    OhDataPackInt dpd = new OhDataPackInt();

    OhTimePoint tp = new OhTimePoint();  //; For work.
    tp.copy(aT);
    
//; dpd.setData(aT, aValue);
    dpd.setData(tp, aValue);
    
    dataPackStock(dpd);
  }


  /**
   * Send datapacks of type int whose value is stored in "aDataStr".
   *
   * <japanese>
   * <p>
   * ɂƂ aDataStr ŕ\ꂽf[^pbN
   * <a href="OhDataPackInt.html">OhDataPackInt</a> ^f[^pbN쐬A
   * <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> p
   * M[qɑM܂B
   * </japanese>
   *
   * @param aDataStr a <code>String</code> value
   */
  public void sendByString(String aDataStr) {           //; (*2)
    OhDataPackInt dp = new OhDataPackInt(aDataStr);
    dataPackStock(dp);
  }
    
  // @end {methods} ------------------------------------------------------

} // OhSendInt
