/**
 * @(#) OhSendPort; class to express a send port.
 *                  Make "OhSendPort.java" from "ohsend.h" and "ohsend.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^M[qf\NXłB<p>
 *
 * {^M[qf OhSendPort `Ă܂B
 * {^M[qf͒ۃNXłB<p>
 *
 * {^M[qf OhSendPort ́AŎĂf[^𑗐M[qłA
 * f[^𑗐M[q
 * <a href="OhSenderPort.html">OhSenderPort</a>
 * ̈łB
 * ʂ̒[qf[^𑗐M[q͒p[q
 * <a href="OhRelayPort.html">OhRelayPort</a>
 * łB<p>
 *
 * NX OhSendPort ł́A
 *
 * <ul>
 *  <li> Mf[^pbN : _dataPackArray, _numberOfDataPacks
 * </ul>
 *
 * LA
 *
 * <ul>
 *  <li> f[^pbN^ёM@\
 * </ul>
 *
 * `܂B<p>
 *
 * OhSendPort ̑M@\́ANX
 * <a href="OhSenderPort.html">OhSenderPort</a>
 * Œ`ς݂łB
 * OhSendPort ɂ́ȂM[qڑ邱Ƃł܂B
 *
 * </japanese>
 *
 * <p>
 *
 * OhSendPort;
 *
 * Abstract class for creating send ports. <p>
 *
 * Fields _dataPackArray, _numberOfDataPacks and some methods to store datapacks are defined.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.76 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhSendPort extends OhSenderPort
{
  // @begin {fields} ----------------------------------------------

  /**
   * <japanese>
   * f[^pbNMp̔z񃊃XgłB
   *
   * _dataPackArray wzɂ́AObject ^ɃLXgꂽ^f[^pbNi[܂B
   * </japanese>
   * <p>
   * An array for sending datapacks.
   *
   */
  protected OhArrayList _dataPackArray;

  /**
   * <japanese>
   * _dataPackArray ̗vfłB
   * </japanese>
   * <p>
   * The number of elements stored in "_dataPackArray".
   *
   */
  protected int _numberOfDataPacks;
  
  /**
   * <japanese>
   * f[^pbNی삷邩(true)ۂ(false)̃tOłB
   * </japanese>
   * <p>
   * This field is designed to use to control the protect mode of past "_dataPack".
   *
   */
  protected boolean _protectPastDataFlag;
  
  /**
   * <japanese>
   * ̑M[qf[^̃f[^Ԍ^xNgłB<p>
   *
   *  OhSendPort NXf[^̃f[^Ԍ^xNg
   * oϐƂċL܂B 
   * </japanese>
   * <p>
   * Data connection type vector of data provided this OhSendPort.
   *
   */
  protected Vector _dctVec;

  // @end {fields} -------------------------------------------------------
  

  // @begin {abstract methods} -------------------------------------------

  // 1. Connection function.
  
  /**
   * <japanese>
   * f[^̌^Ԃ܂B<p>
   *
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB
   * ̃NX OhSendPort łۃ\bhƂĂ܂B
   * </japanese>
   * <p>
   * Returns the string which shows the type of the datapack sent by the current port.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();

  
  // 2. Recording and sending function of datapacks.
  
  /**
   * <japanese>
   * f[^pbN̍쐬ю^sȂ܂B <p>
   *
   * ۃ\bhłB
   * aDstr (arg) Ɋi[ꂽf[^񂩂f[^pbN쐬A^܂B
   * </japanese>
   * <p>
   *
   * A data pack is created and recorded from the data character sequence
   * stored in "aDstr".
   *
   * @param aDstr a <code>String</code> value
   */
  public abstract void sendByString(String aDstr);

  // @end {abstract methods} ---------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍA
   * f[^̕Ԃ͐܂ɐݒ肵܂B
   * ̓Iɂ́A
   * _dctVec ̑vf
   * <a href="DataConnectionType.html#BROKENLINE">DataConnectionType.BROKENLINE</a> i[܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    _dctVec = new Vector();
    _dctVec.addElement(DataConnectionType.BROKENLINE);

    return super.init(aPortName, aPortNum);
  }


  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAf[^Ԍ^xNg܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum, Vector aDctVec) throws OhError {
    _dctVec = aDctVec;

    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------

  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^
   * <a href="OhSendPort.html#init(String, int)">OhSendPort.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   *  <li> _dctVec            : null
   * </dl>
   *
   * <p>
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhSendPort.html#init(String, int)">OhSendPort.init(String, int)</a> 
   * păoϐĉB
   * </jananese>
   *
   */
  public OhSendPort() {
    super();
    _dataPackArray       = null;
    _numberOfDataPacks   = -1;
    _protectPastDataFlag = false;

    _className     = "OhSendPort";
    _commentString = null;
  }

  /**
   * <japanese>
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̃ftHgRXgN^pA
   * <a href="OhSendPort.html#init(String, int)">OhSendPort.init(String, int)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhSendPort"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _owner             : null
   *  <li> _bufferList        : ɊmۂBiڑƗpj
   *  <li> _receiverArray     : null
   *  <li> _numberOfReceivers : -1
   *  <li> _dataPackArray     : null
   *  <li> _numberOfDataPacks : -1
   *  <li> _dctVec            : null
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   */
  public OhSendPort(String aPortName, int aPortNum) throws OhError {
    super();
    _dataPackArray       = null;
    _numberOfDataPacks   = -1;
    _protectPastDataFlag = false;

    _className     = "OhSendPort";
    _commentString = null;
    
    _dctVec = new Vector();
    _dctVec.addElement(DataConnectionType.BROKENLINE);

    init(aPortName, aPortNum);
  }

  /**
   * <japanese>
   * NX <a href="OhSenderPort.html">OhSenderPort</a> ̃ftHgRXgN^pA
   * <a href="OhSendPort.html#init(String, int)">OhSendPort.init(String, int)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhSendPort"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _owner             : null
   *  <li> _bufferList        : ɊmۂBiڑƗpj
   *  <li> _receiverArray     : null
   *  <li> _numberOfReceivers : -1
   *  <li> _dataPackArray     : null
   *  <li> _numberOfDataPacks : -1
   *  <li> _dctVec            : aDctVec
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @param aDctVec a <code>Vector</code> value
   */
  public OhSendPort(String aPortName, int aPortNum, Vector aDctVec) throws OhError {
    super();
    _dataPackArray       = null;
    _numberOfDataPacks   = -1;
    _protectPastDataFlag = false;

    _dctVec = aDctVec;
    
    _className     = "OhSendPort";
    _commentString = null;
    
    init(aPortName, aPortNum);
  }
  
  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  // 1. Connection function.
  
  /**
   * <japanese>
   * f[^Ԍ^xNgݒ肵܂B
   * </japanese>
   * <p>
   * Sets the data connection type vector.
   *
   * @param aDctVec a <code>Vector</code> value
   */
  public void setDctVec(Vector aDctVec) {
    _dctVec = aDctVec;
  }


  /**
   * <japanese>
   * f[^Ԍ^xNgԂ܂B
   * </japanese>
   * <p>
   * This function is decleared abstract method in <a href="OhSenderPort.html">OhSenderPort</a>.
   * Implemented in this class. 
   *
   * Returns the data connection vector which is a instance variable.
   *
   * @return a <code>Vector</code> value
   */
  public Vector getDctVec() {
    return _dctVec;
  }


  /**
   * <japanese>
   * [q̐ڑƂ̏Iуf[^MJn̂߂̏ƂsȂ܂B
   * ̓Iɂ́A_receiverArray ̍쐬 _dataPackArray ̏sȂ܂B<p>
   *
   * ̃\bh͈ȉs܂B<p>
   *
   * <ol>
   *  <li> MM[qz _dataPackArray 쐬B
   *  <li> MM[q̐ _numberOfReceivers ɐݒ肷B
   *  <li> _dataPackArray ̃mۂB
   *  <li> _numberOfDataPacks ̒l -1  0 ɕύXB
   * </ol>
   * <p>
   *
   * ܂A
   * <a href="OhSenderPort.html#canYouSwitch()">OhSenderPort.canYouSwitch()</a>
   * s܂B
   * ̓Iɂ́A_receiverArray 쐬܂B
   * Ŗ߂l false Ȃ΁Afalse Ԃ܂B<p>
   *
   * ߂l true ȂA_dataPackArray  new 邱Ƃɂ胁mۂA
   * _numberOfDataPacks ̒l -1  0 ɕϊAtrue Ԃ܂B<p>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> canYouSwitch() ɎsłꍇB
   *  Efalse -> _receivePortArray 쐬ς݂̏ꍇB
   *             _dataPackArray 쐬ς݂̏ꍇB
   * </pre>
   * </japanese>
   *
   * <p>
   *
   * Preparation of "_dataPackArray".
   * That is, "_numberOfDataPacks" is set as 0 from -1.
   *   
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean canYouSwitch() throws OhError {    //; (*1)
    if (super.canYouSwitch() != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot execute \"OhSenderPort.canYouSwitch()\".", str);
    }
    
    if ((_numberOfDataPacks >= 0) || (_dataPackArray != null)) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Already exists \"_dataPackArray\"." + str);
    }

    _dataPackArray = new OhArrayList();
    
    _numberOfDataPacks = 0;

    return true;
  }


  /**
   * <japanese>
   * vZƑO̊mFsȂ܂B<p>
   *
   * NX
   * <a href="OhSenderPort.html">OhSenderPort</a>
   * ̒ۃ\bhłB<p>
   *
   * f[^pbN𑗐M\ȏԂł邱ƂmF܂B
   * _receiverArray 쐬ς݂ŁAf[^pbN^p _bufferList ݂΁A
   * f[^pbN^AMł܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> _sender  null ŁAf[^pbN^p _bufferList ݂ꍇB
   *             ܂́A_sender ݒ肳ĂꍇB
   *  Efalse -> _sender  null ŁAf[^pbN^p _bufferList ݂ȂꍇB
   * </pre>
   * </japanese>
   *
   * <p>
   *
   * Checks before calculation.
   *
   * @return a <code>boolean</code> value
   */
  public boolean areYouReady() {
    if (super.areYouReady() == true)
      return true;

    return false;
  }


  /**
   * <japanese>
   * M[q̏L҂ɌvZv`܂B
   * </japanese>
   * <p>
   * Transer the calculation request to the owner.
   *
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public void calReq() throws OhCannotGetDataError, OhError {
    ((OhElement)_owner).calReq();
  }

  
  // 2. Recording and sending function of datapacks.

  /**
   * <japanese>
   * f[^pbN^܂B<p>
   *
   * ŐV̎^f[^pbN aDataPack (arg) ̃f[^L^rA
   * aDataPack ̕V aDataPack  _bufferList Ɏ^܂B<p>
   *
   * ̏ꍇ aDataPack ̕Âꍇ aDataPack ^܂B
   *
   * ႦΕsAfsvȏꍇȂǂ͔hNXōĒ`ĉB
   *
   * <pre>
   * [in] aDataPack : Mf[^pbN Object ^ɃLXǵB
   * </pre>
   * </japanese>
   *
   * <p>
   *
   * Stocks "_bufferList" if the data time of "aDataPack" is newer
   * than the data time of the newest datapack stored in "_bufferList.
   * Otherwise the "_bufferList" is not included into "_bufferList".
   * [in] "aDataPack": a datapack to be stocked. It must be casted as <code>OhObject</code>.
   *
   * @param aDataPack an <code>Object</code> value
   */
  public void dataPackStock(Object aDataPack) {    //; (*2)
    if ((_dataPackArray.size() > 0) && (((OhDataPack)aDataPack).getDataTime().compareTo(
                                          ((OhDataPack)_dataPackArray.getLastElement()).getDataTime()) <= 0))
      return;

    _dataPackArray.add(aDataPack);

    setDataPackArray();
  }

  
  /**
   * <japanese>
   * f|^pbN̔z񃊃Xgݒ肵܂B<p>
   *
   * NX
   * <a href="OhSenderPort.html">OhSenderPort</a>
   * ̒ۃ\bhłB<p>
   *
   * ۂɂ́A
   * _dataPackArray ̗vf _numberOfDataPacks ɐݒ肷邾łB
   * </japanese>
   * <p>
   *
   * Makes data pack array.
   *
   */
  public void setDataPackArray() {
    _numberOfDataPacks = _dataPackArray.size();
  }

  
  /**
   * <japanese>
   * ŌẪf[^̋L^Ԃ܂B
   * </japanese>
   * <p>
   * Returns the oldest data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getOldestDataTime() {    //; (*3)
    if (_numberOfDataPacks <= 0)
      return OhTimePoint.OhTimePointMin;

    return (((OhDataPack)_dataPackArray.get(0)).getDataTime());
  }

  
  /**
   * <japanese>
   * ŐṼf[^̋L^Ԃ܂B
   * </japanese>
   * <p>
   * Returns the newest data time.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNewestDataTime() {    //; (*4)
    if (_numberOfDataPacks <= 0)
      return OhTimePoint.OhTimePointMax;

    return (((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).getDataTime());
  }


  /**
   * <japanese>
   * f[^pbŇ(_numberOfDataPacks)Ԃ܂B
   * </japanese>
   * <p>
   * Returns the number of data packs which stored in the sender.
   *
   * @return an <code>int</code> value
   */
  public int getNumberOfDataPacks() {      //; (*5)
    return _numberOfDataPacks;
  }


  /**
   * <japanese>
   * f[^^̊mFsȂ܂B<p>
   *
   * NX
   * <a href="OhSenderPort.html">OhSenderPort</a>
   * ̒ۃ\bhłB
   * ̃NX OhSendPort Ŏ܂B<p>
   *
   * aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN^Ăꍇ true ԂA
   * ^ĂȂꍇ false Ԃ܂B<p>
   *
   * ܂A_dataPackArray Ɏ^Ăꍇ true Ԃ܂B
   * ^ĂȂꍇ́AX _bufferList Ɏ^Ă邩ׁA
   * _bufferList Ɏ^Ă _dataPackArray XV true Ԃ܂B
   * ^ĂȂꍇ false Ԃ܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaTFrom -> aTFrom ȑOf[^pbN^Ă邱ƂmFB
   *  EaTTo ---> aTTo ȍ~܂Ńf[^pbN^Ă邱ƂmFB
   * [RETURN]
   *  Etrue ---> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN^ĂꍇB
   *  Efalse --> aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN^ĂȂꍇB
   * </pre>
   * </japanese>
   * <p>
   * Checks the data time recorded.
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   * @param aTTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouGetData(OhTimePoint aTFrom, OhTimePoint aTTo) {    //; (*8)
    if (_numberOfDataPacks <= 0)
      return false;

    if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).getDataTime().compareTo(aTTo) < 0) ||
      (((OhDataPack)_dataPackArray.get(0)).getDataTime().compareTo(aTFrom) > 0)) {
      return false;
    }

    return true;
  }
      

  /**
   * <japanese>
   * f[^pbN̔z񃊃Xg擾(M)܂B<p>
   *
   * NX
   * <a href="OhSenderPort.html">OhSenderPort</a>
   * ̒ۃ\bhłB<p>
   *
   * Ƀoϐ _dataPackArray, _numberOfDataPacks i[܂B
   * </japanese>
   * <p>
   * Sends the datapack array.
   *
   * @param aDpArray an <code>OhArrayList[]</code> value
   * @param aNumOfDp an <code>int[]</code> value
   */
  public void getDataPackArray(OhArrayList[] aDpArray, int[] aNumOfDp) {    //; (*12)
    aNumOfDp[0] = _numberOfDataPacks;
    aDpArray[0] = _dataPackArray;
  }
  

  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * NX
   * <a href="OhSenderPort.html">OhSenderPort</a>
   * ̒ۃ\bhłB<p>
   *
   * _sender  null łȂꍇA_sender  getDataPack() s܂B
   * ̎A͂̂܂ܓn܂B<p>
   *
   * _sender  null ̏ꍇA_dataPackArray Ĉȉ̏sȂ܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂
   * ^f[^pbN擾Ai[܂B<p>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂
   * ^f[^pbN擾Ai[܂B<p>
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B<p>
   *
   * 𖞂f[^pbN擾łȂꍇAaFormer, aLater  null ɐݒ肳܂B<p>
   *
   * aMode ͎擾[hI邽߂̃IvVŁAL̒ʂłB<p>
   *
   * <ul>
   *  <li> 'b' ----> aFormer, aLater ̗K擾B擾łȂꍇ͗Ƃ null ɐݒ肷B
   *  <li> 'f' ----> aFormer K擾BaLater 擾łȂĂ aFormer ̂ݎ擾B
   *  <li> 'l' ----> aLater K擾BaFormer 擾łȂĂ aLater ̂ݎ擾B
   *  <li> ̑ -> aFormer, aLater ̏ȂƂ擾B
   * </ul>
   * <p>
   *
   * aMode ̃ftHgl 'b' łB
   * d`̂ŁA
   *  aLater nȂꍇ aMode  'f' Ƃ aFormer ݂̂擾A
   *  aFormer nȂꍇ aMode  'l' Ƃ aLater ݂̂擾܂B
   *
   * </japanese>
   * <p>
   * Sends the datapacks.
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @param aMode a <code>char</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater, char aMode) {  //; (*6-1)
    if (_numberOfDataPacks <= 0) {
      aFormer[0] = null;
      aLater[0]  = null;
      return false;
    }

    if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).getDataTime()).compareTo(_dataTime) < 0) {
      aLater[0] = null;

      if ((aMode == 'b') || (aMode == 'l')) {
        aFormer[0] = null;
        return false;
      }

      aFormer[0] = _dataPackArray.get(_numberOfDataPacks - 1);
      return true;
    }
    else if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).getDataTime()).compareTo(_dataTime) == 0) {
      aFormer[0] = aLater[0] = _dataPackArray.get(_numberOfDataPacks - 1);
      return true;
    }

    for (int k = _numberOfDataPacks - 2; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).getDataTime()).compareTo(_dataTime) <= 0) {
        aFormer[0] = _dataPackArray.get(k);
        aLater[0]  = (((OhDataPack)aFormer[0]).getDataTime().compareTo(_dataTime) == 0)
          ? aFormer[0]
          : _dataPackArray.get(k + 1);
        return true;
      }

    aFormer[0] = null;

    if ((aMode == 'b') || (aMode == 'f')) {
      aLater[0] = null;
      return false;
    }

    aLater[0] = _dataPackArray.get(0);

    return true;
  }


  /**
   * <japanese>
   * wɂf[^pbN擾(M)܂B<p>
   *
   * NX
   * <a href="OhSenderPort.html">OhSenderPort</a>
   * ̒ۃ\bhłB<p>
   *
   * _sender  null łȂꍇA_sender  getDataPack() s܂B
   * ̎A͂̂܂ܓn܂B<p>
   *
   * _sender  null ̏ꍇA_dataPackArray Ĉȉ̏sȂ܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂
   * ^f[^pbN擾Ai[܂B<p>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂
   * ^f[^pbN擾Ai[܂B<p>
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B<p>
   *
   * aFormer, aLater ̗K擾܂B擾łȂꍇ͗Ƃ null ɐݒ肵܂B
   *
   * </japanese>
   *
   * <p>
   *
   * Sends the datapacks.
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @return a <code>boolean</code> value
   */
  public boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater) {  //; (*6-2)
    if (_numberOfDataPacks <= 0) {
      aFormer[0] = null;
      aLater[0]  = null;
      return false;
    }

    if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).getDataTime()).compareTo(_dataTime) < 0) {
      aLater[0] = null;
      aFormer[0] = null;
      
      return false;
    }
    else if ((((OhDataPack)_dataPackArray.get(_numberOfDataPacks - 1)).getDataTime()).compareTo(_dataTime) == 0) {
      aFormer[0] = aLater[0] = _dataPackArray.get(_numberOfDataPacks - 1);
      return true;
    }

    for (int k = _numberOfDataPacks - 2; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).getDataTime()).compareTo(_dataTime) <= 0) {
        aFormer[0] = _dataPackArray.get(k);
        aLater[0]  = (((OhDataPack)aFormer[0]).getDataTime().compareTo(_dataTime) == 0)
          ? aFormer[0]
          : _dataPackArray.get(k + 1);
        return true;
      }

    aFormer[0] = null;
    aLater[0] = null;
    
    return false;
  }


  /**
   * <japanese>
   * svf[^pbN܂B<p>
   *
   * NX
   * <a href="OhSenderPort.html">OhSenderPort</a>
   * ̒ۃ\bhłB<p>
   *
   * <a href="OhSendPort.html#getNecessaryTimeFrom()">OhSendPort.getNecessaryTimeFrom()</a>
   * O̎^f[^pbN _bufferList A
   * _dataPackArray, _numberOfDataPacks XV܂B
   *
   * </japanese>
   * <p>
   * Deletion of an unnecessary datapacks.
   *
   */
  public void reduceDataPackStock() {    //; (*9)
    if (_protectPastDataFlag == true)
      return;

    if (_numberOfDataPacks <= 0)
      return;

    OhTimePoint reserve = getNecessaryTimeFrom();

    int k = 0;
    for (k = _numberOfDataPacks - 1; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).getDataTime()).compareTo(reserve) <= 0)
        break;

    if (k <= 0)
      return;    // No data to be deleted.

    _dataPackArray.removeHeadRange(k);

    setDataPackArray();
  }


  /**
   * <japanese>
   * wɂsvf[^pbN܂B<p>
   *
   * aTFrom (arg) O̎^f[^pbN _bufferList A
   * _dataPackArray, _numberOfDataPacks XV܂B
   *
   * </japanese>
   * <p>
   * Delete past datapack before "aTFrom",
   * and renew "_dataPackArray" and "_numberOfDataPacks".
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   */
  public void forgetDataPackStock(OhTimePoint aTFrom) {    //; (*10)
    if (_protectPastDataFlag == true)
      return;

    if (_numberOfDataPacks <= 0)
      return;

    int k = 0;
    for (k = _numberOfDataPacks - 1; k >= 0; k--)
      if ((((OhDataPack)_dataPackArray.get(k)).getDataTime()).compareTo(aTFrom) <= 0)
        break;

    if (k <= 0)
      return;    // No data to be deleted.

    _dataPackArray.removeHeadRange(k);
    
    setDataPackArray();
  }


  /**
   * <japanese>
   * S^f[^pbN܂B<p>
   *
   * _bufferList ̑S^f[^pbNA_dataPackArray ܂B
   * ̌A_dataPackArray  null ɁA_numberOfDataPacks  -1 ɐݒ肳܂B
   * </japanese>
   * <p>
   * Deletion of all recorded datapacks.
   *
   */
  public void clearDataPack() {    //; (*11)
    _dataPackArray     = null;
    _numberOfDataPacks = -1;
  }


  /**
   * <japanese>
   * _protectPastDataFlag ݒ肵܂B
   * f[^pbNȂꍇ true ɐݒ肵ĂB
   * </japanese>
   * <p>
   * Sets "_protectPastDataFlag".
   *
   * @param aFlag a <code>boolean</code> value
   * @return a <code>boolean</code> value
   */
  public boolean setProtectPastDataFlag(boolean aFlag) {
    boolean curFlag = _protectPastDataFlag;
    _protectPastDataFlag = aFlag;

    return curFlag;
  }


  // 3. Function to save the recorded datapacks at a file.

  /**
   * <japanese>
   * f[^pbNXg[ aPw ɏ݂܂B <p>
   *
   * ^Ăf[^pbNo̓Xg[ aPw Ŏw肳t@Cɏ݂܂B<p>
   *
   * f[^ׂ͂ ( ... ) ̂悤Ɋʂň݂͂܂B
   * ŏ̃f[^́A^Ăf[^pbŇłB
   * f[^pbŇȂA
   * ̂ƂɃf[^pbN𕶎ɕϊAʂň͂񂾂݂̂܂B
   * </japanese>
   * <p>
   * Saves the recorded datapacks at the file, "aPw".
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveSendPortStock(PrintWriter aPw) {    //; (*7)
    _numberOfDataPacks = _dataPackArray.size();

    aPw.print("(" + _numberOfDataPacks + ") # _numberOfDataPacks" + NEWLINE);

    if (_numberOfDataPacks <= 0)
      return;

    for (int k = 0; k < _numberOfDataPacks; k++) {
      aPw.print(NEWLINE);
      ((OhDataPack)_dataPackArray.get(k)).dpack2File(aPw);
    }
  }
  
  // @end {methods} ------------------------------------------------------

} // OhSendPort
