/**
 * @(#) OhSenderPort; class to express a sender port.
 *                    Make "OhSenderPort.java" from "ohsender.h" and "ohsender.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * <japanese>
 * {^Mn[qf\NXłB<p>
 *
 * ̃NX́AMn[q쐬邽߂̒ۃNXłB
 * Mn[q͑M@\[qłB<p>
 *
 * ̒ۃNX OhSenderPort A
 * ŎĂf[^𑗐M <a href="OhSendPort.html">OhSendPort</a> 
 * ̒[q瓾f[^𒆌p <a href="OhRelayPort.html">OhRelayPort</a> h܂B<p>
 *
 * <ul>
 *  <li> <a href="OhSendPort.html">OhSendPort</a> ́AgŃf[^Ă鑗M[qłB
 *  <li> <a href="OhRelayPort.html">OhRelayPort</a> ́A
 *       ł̓f[^ĂȂAM[q̑Mv󂯂A
 *       ̒[qf[^ē][qłB
 * </ul>
 *
 * ɂĂAM炷΁Af[^𑗐MĂ@\Ă̂ŁA
 * ̋ʕ̃C^[tF[X OhSenderPort Œ`Ă܂B<p>
 *
 * OhSenderPort ɂ͕̑M[qif[^M[qjڑ邱Ƃł܂B
 * </japanese>
 * <p>
 * OhSenderPort;
 *
 * Abstract class for creating sender ports. <p>
 *
 * By inheritance from this OhSenderPort class,
 * the classes <a href="OhSendPort.html">OhSendPort</a>
 * and <a href="OhRelayPort.html">OhRelayPort</a> are created.
 *
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.57 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhSenderPort extends OhPort
{
  // @begin {fields} ---------------------------------------------------

  /**
   * <japanese>
   * Ɨp̐`XgłB
   * </japanese>
   * <p>
   * The list for work.
   *
   */
  protected OhLinkedList _bufferList;


  /**
   * <japanese>
   * M̎M[qL邽߂ Object ^złB
   * </japanese>
   * <p>
   * Array for memorizing the ports of sending place.
   *
   */
  protected Object[] _receiverArray;

  /**
   * <japanese>
   * _receiverArray ̗vfłB
   * </japanese>
   * <p>
   * The number of elements of "_receiverArray".
   *
   */
  protected int _numberOfReceivers;

  /**
   * <japanese>
   * f[^łB
   * </japanese>
   * <p>
   * Comment about data.
   *
   */
  protected String _commentString;

  // @end {fields} -------------------------------------------------------
  
  
  // @begin {abstract methods} -------------------------------------------

  // 1. Connection function.
  
  /**
   * <japanese>
   * f[^̌^Ԃ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   * ̃NX OhSenderPort łۃ\bhƂĂ܂B
   * </japanese>
   * <p>
   *
   * Returns the datapack type string of this port.
   * Because this is an abstract method, it should be implemented in derived classes.
   *
   * @return a <code>String</code> value
   */
  public abstract String dataType();
  

  /**
   * <japanese>
   * ̑M[qf[^̃f[^Ԍ^xNgԂ܂B
   * </japanese>
   * <p>
   *
   * Abstract method.
   * It should return the data connection vector.
   *
   * @return a <code>Vector</code> value
   */
  public abstract Vector getDctVec();


  // 2. Function for sending datapacks.
  
  /**
   * <japanese>
   * f|^pbN̔z񃊃Xgݒ肵܂B <p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   * ̃NX OhSenderPort łۃ\bhƂĂ܂B<p>
   *
   * ۂɂ́A
   * _dataPackArray ̗vf _numberOfDataPacks ɐݒ肷邾łB
   * </japanese>
   * <p>
   *
   * Abstract method.
   * Makes data pack array.
   *
   */
  public abstract void setDataPackArray();


  /**
   * <japanese>
   * 擾\f[^pbN̊mFsȂ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   * ̃NX OhSenderPort łۃ\bhƂĂ܂B<p>
   *
   * canYouGetData(OhTimePoint aTFrom, OhTimePoint aTTo) , aTFrom ȑOA
   * aTTo ȍ~܂ł̃f[^pbN̒[q擾\Ȃ΁A
   * true ԂAs\Ȃ false Ԃ悤ɊehNXŒ`܂.
   *
   * <pre>
   * ARGUMENT:
   *   EaTFrom    aTFrom ȑÕf[^pbN擾\ł邱ƂmF.
   *   EaTTo      aTTo ȍ~܂ł̃f[^pbN擾\ł邱ƂmF.
   * RETURN:
   *   EYES       aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN擾\ȏꍇ.
   *               t ɂf[^擾\ȏꍇ.
   *   ENO        aTFrom ȑO aTTo ȍ~܂ł̃f[^pbN擾s\ȏꍇ.
   * </pre>
   * </japanese>
   * <p>
   *
   * Abstract method.
   * Checks the data time recorded.
   *
   *
   * @param aTFrom an <code>OhTimePoint</code> value
   * @param aTTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public abstract boolean canYouGetData(OhTimePoint aTFrom, OhTimePoint aTTo);



  /**
   * <japanese>
   * f[^pbN̔z񃊃Xg擾(M)܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   * ̃NX OhSenderPort łۃ\bhƂĂ܂B<p>
   *
   * hNXɂāAf[^pbNz擾郁\bh`܂B
   * aDpArray ɂ̓f[^pbNzi[AaNumOfDp ɂ́AaDpArray ̗vfi[܂B
   * </japanese>
   * <p>
   * Abstract method.
   * Sends the datapack array.
   *
   * @param aDpArray an <code>OhArrayList</code> value
   * @param aNumOfDp an <code>int[]</code> value
   */
  public abstract void getDataPackArray(OhArrayList[] aDpArray, int[] aNumOfDp);


  /**
   * <japanese>
   * wɂf[^pbN擾(M) ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   * ̃NX OhSenderPort łۃ\bhƂĂ܂B<p>
   *
   * hNXɂāAw肵ăf[^pbN擾֐`܂B<p>
   *
   * aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbN擾Ai[܂B<p>
   *
   * aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbN擾Ai[܂B<p>
   *
   * aFlag ͎擾̃[hI邽߂̃IvVŁAL̒ʂłB
   *
   * <pre>
   *   E"b"       aFormer, aLater ̗K擾B
   *               擾łȂꍇ́AƂ null ɐݒ肷B
   *   E"f"       aLater 擾łȂĂAaFormer ̂ݎ擾B
   *               aLater 擾łȂꍇAnull ɐݒ肷B
   *   E"l"       aFormer 擾łȂĂAaLater ̂ݎ擾B
   *               aFormer 擾łȂꍇAnull ɐݒ肷B
   *   Ȇ    aFormer, aLater ̂ȂƂ擾B
   *               擾łȂ̂ null ɐݒ肷B
   * </pre>
   *
   * aFlag ̃ftHgĺA "b" łB<p>
   * ܂A  aLater nȂꍇ́A aFlag  f Ƃ aFormer ̂ݎ擾A
   *  aFormer nȂꍇ́A aFlag  l Ƃ aLater ̂ݎ擾܂B<p>
   *
   * f[^L^x _dataTime łf[^pbN^Ăꍇ́A
   * aFormer, aLater Ƃi[܂B<p>
   *
   * 𖞂f[^pbN擾łȂꍇAaFormer, aLater  null ɐݒ肳܂B<p>
   *
   * f[^擾łꍇ́Atrue Ԃ܂B
   * </japanese>
   * <p>
   * Abstract method.
   * Sends the datapacks.
   *
   * @param aFormer an <code>Object[]</code> value
   * @param _dataTime an <code>OhTimePoint</code> value
   * @param aLater an <code>Object[]</code> value
   * @param aMode a <code>char</code> value
   * @return a <code>boolean</code> value
   */
  public abstract boolean getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater, char aMode);


  /**
   * <japanese>
   * svf[^pbN܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
   * ̃NX OhSenderPort łۃ\bhƂĂ܂B<p>
   *
   * ̒[qŎ擾\ȃf[^pbN̂A
   * getNecessaryTimeFrom() f[^L^f[^pbN悤A
   * hNXŒ`܂B
   * </japanese>
   * <p>
   * Abstract method.
   * Deletion of an unnecessary datapacks.
   *
   */
  public abstract void reduceDataPackStock();

  // @end {abstract methods} ---------------------------------------------


  // @begin{initialization} ----------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * </japanese>
   * <p>
   * Initialization.
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean init(String aPortName, int aPortNum) throws OhError {
    return super.init(aPortName, aPortNum);
  }

  // @end{initialization} ------------------------------------------------

  
  // @begin {constructors} -----------------------------------------------
  
  /**
   * <japanese>
   * ftHgRXgN^
   * <a href="OhSenderPort.html#init(String, int)">OhSenderPort.init(String, int)</a> s܂B
   * ̌ʁAȉ̐ݒƂȂ܂B<p>
   *
   * <dl>
   *  <li> _objectName        : null
   *  <li> _objectNumber      : -1
   * </dl>
   *
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA<a href="OhSenderPort.html#init(String, int)">OhSenderPort.init(String, int)</a> 
   * păoĉB
   * </jananese>
   *
   */
  public OhSenderPort() {
    super();

    _receiverArray     = null;
    _numberOfReceivers = -1;
    _className         = "OhSenderPort";
    _commentString     = null;

    _bufferList = new OhLinkedList();
  }


  /**
   * <japanese>
   * NX <a href="OhPort.html">OhPort</a> ̃ftHgRXgN^pA
   * <a href="OhSenderPort.html#init(String, int)">OhSenderPort.init(String, int)</a> pāA
   * oϐȉ̒ʂ菉܂B<p>
   *
   * <dl>
   *  <li> _className         : "OhSenderPort"
   *  <li> _objectName        : aPortName
   *  <li> _objectNumber      : aPortNum
   *  <li> _owner             : null
   *  <li> _bufferList        : ɊmۂBiڑƗpj
   *  <li> _receiverArray     : null
   *  <li> _numberOfReceivers : -1
   * </dl>
   * </japanese>
   *
   * @param aPortName a <code>String</code> value
   * @param aPortNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSenderPort(String aPortName, int aPortNum) throws OhError {
    super();

    _receiverArray     = null;
    _numberOfReceivers = -1;
    _className         = "OhSenderPort";
    _commentString     = null;

    init(aPortName, aPortNum);

    _bufferList = new OhLinkedList();
  }
  
  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  // 1. connection function

  /**
   * <japanese>
   * M(M)[qo^܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̃\bhI[o[ChĂ܂B<p>
   *
   * ̃\bh́A[qԂ̐ڑ̍ۂɌĂяo܂B<p>
   *
   * <a href="OhLinkedList.html#add(Object)">OhLinkedList.add(Object)</a> ɂ
   * _bufferList  ł aReceiver o^A
   * true Ԃ܂B<p>
   *
   * Ȁꍇ̓G[bZ[W\ false Ԃ܂B<p>
   *
   * <ol>
   *  <li> _bufferList  null ̏ꍇB
   *  <li> _receive_port_array 쐬ς݂̏ꍇB
   *  <li> ̒[q aReceiver ƂŁA_dataPackType قȂꍇB
   * </ol>
   *
   * ܂AaReceiver  _bufferList ɓo^ς݂̏ꍇ́Ado^ true Ԃ܂B
   * </japanese>
   * <p>
   * Register a receiver.
   * This function is an abstract method in the base class <a href="OhPort.html">OhPort</a>.
   * Implemented in this class OhSenerPort.
   *
   * @param aReceiver an <code>Object</code> value
   * @return a <code>boolean</code> value
   */
  public boolean registerReceiver(Object aReceiver) throws OhError {    //; (*1)
    if (aReceiver == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException("Cannot register 'receiver'" + str);
    }

    if (_numberOfReceivers >= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)aReceiver).oPrint(out, " > ");
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("\"_bufferList\" has already been switched.message", str);
    }

    if (_bufferList == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      throw new OhError("Cannot register 'receiver'", buffer.toString());
    }      

    // Gets the datapack type which this port sends, and the datapack type which "aReceiver" receives.
    //
    String sType = dataType();                   //; Abstract method call.
    String rType = ((OhPort)aReceiver).dataType();

    if (!sType.equals(rType)) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      ((OhObject)aReceiver).oPrint(out, "    r > ");
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("'dataType' of this port and \"aReceiver\"(arg) are different.", str);
    }
    
    if (_bufferList.contains(aReceiver) == true)
      return true;

    _bufferList.add(aReceiver);
    
    return true;
  }

  

  /**
   * <japanese>
   * [q̐ڑƂ̏Iуf[^MJn̂߂̏ƂsȂ܂B<p>
   *
   * NX OhPort ̒ۃ\bhĂ܂B<p>
   *
   * _bufferList ɓo^Ă鑗MM[q _receiverArray ɔz񉻂A
   * _receiverArray ̗vf _numberOfReceivers ɐݒ肵܂B<p>
   *
   * _receiverArray 쐬ς݂̏ꍇ́AG[bZ[W\ false Ԃ܂B<p>
   *
   * ڑƂ̏IA_owner ̌vZJnOɕKpĉB
   * Ã\bh̎s͐ڑ֌WύXł܂B<p>
   *
   * _receiverArray ɍ쐬ꂽꍇ true Ԃ܂B
   * </japanese>
   * <p>
   *
   * Makes "_receiverArray" from the port of the sending place registered into "_bufferList".
   * And, the number of elements of "_receiverArray" is set as "_numberOfReceivers".
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean canYouSwitch() throws OhError {    //; (*2)
    if (_numberOfReceivers >= 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("\"_receiverArray\" has already been set.", str);
    }

    _receiverArray     = _bufferList.toArray(new Object[_bufferList.size()]);
    _numberOfReceivers = _bufferList.size();

    _bufferList = null;

    return true;
  }


  /**
   * <japanese>
   * <p>
   * vZƑO̊mFsȂ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhĂ܂B<p>
   *
   * f[^pbN𑗐M\ȏԂł邱ƂmF܂B
   * _numberOfReceivers  0 ȏȂ _receiverArray ݒς݂ŁA
   * M[qɃf[^pbN𑗐M邱Ƃł܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> _receiverArray ݒς݂̏ꍇB
   *  Efalse -> _receiverArray ݒ̏ꍇB
   * </pre>
   * </japanese>
   * <p>
   * Check that it is in the state where data packs can be transmitted.
   * If _numberOfReceivers >= 0,
   * _receiverArray is setting ending,
   * and you can send data packs to a send port.
   *
   *
   * @return a <code>boolean</code> value
   */
  public boolean areYouReady() {                    //; (*8)
    return (_numberOfReceivers >= 0) ? true : false;
  }


  /**
   * <japanese>
   * _commentString ݒ肵܂B
   * </japanese>
   * <p>
   * Copy the string "cmt" to "_commentString" which is
   * a instance variable of this class.
   *
   * @param aCmt a <code>String</code> value
   */
  public void setCommentString(String aCmt) {    //; (*6)
    _commentString = aCmt;
  }

  
  /**
   * <japanese>
   * _commentString Ԃ܂B
   * </japanese>
   * <p>
   * Returns "_commentString".
   *
   * @return a <code>String</code> value
   */
  public String getCommentString() {    //; (*7)
    return _commentString;
  }


  // 2. Function for sending datapacks.

  /**
   * <japanese>
   * f[^KvԂ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhĂ܂B<p>
   *
   * ̃\bhł́A
   * _owner  <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a>A
   * M[q <a href="OhPort.html#getNecessaryTimeFrom()">OhPort.getNecessaryTimeFrom()</a> ̂A
   * ŏ̂̂Ԃ܂B
   * ̎A_owner  null Ȃ
   * <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a> Ԃ܂B
   * </japanese>
   * <p>
   *
   * The minimum thing is returned
   * among <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a> of the "_owner"
   * and <a href="OhPort.html#getNecessaryTimeFrom()">OhPort.getNecessaryTimeFrom()</a> of
   * the port of a sending place. 
   *
   * If "_owner" is "null",
   * Returns <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a>.
   *
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeFrom() {      //; (*3)
    if (_owner == null || _numberOfReceivers <= 0) {
      return OhTimePoint.OhTimePointMin;
    }

    OhTimePoint t0 = OhTimePoint.OhTimePointMax;
    OhTimePoint t1 = new OhTimePoint();

    for (int k = 0; k < _numberOfReceivers; k++) {
      Object r = _receiverArray[k];
      t1 = ((OhPort)r).getNecessaryTimeFrom();
      if (t1.compareTo(t0) < 0)
        t0 = t1;
    }
    return t0;
  }
      

  /**
   * <japanese>
   * f[^KvԂ܂B<p>
   *
   * NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhĂ܂B<p>
   *
   * ̃\bhł́A
   * _owner  <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a>A
   * M[q <a href="OhPort.html#getNecessaryTimeTo()">OhPort.getNecessaryTimeTo()</a> ̂A
   * ő̂̂Ԃ܂B
   * ̎A_owner  null Ȃ
   * <a href="OhTimePoint.html#OhTimePointMax">OhTimePoint.OhTimePointMax</a> Ԃ܂B
   * </japanese>
   * <p>
   * The maxmum thing is returned
   * among <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a> of the "_owner"
   * and <a href="OhPort.html#getNecessaryTimeTo()">OhPort.getNecessaryTimeTo()</a> of
   * the port of a sending place. 
   *
   * If "_owner" is "null",
   * Returns <a href="OhTimePoint.html#OhTimePointMax">OhTimePoint.OhTimePointMax</a>.
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeTo() {        //; (*4)
    if (_owner == null || _numberOfReceivers <= 0) {
      return OhTimePoint.OhTimePointMax;
    }

    OhTimePoint t0 = OhTimePoint.OhTimePointMin;
    OhTimePoint t1 = new OhTimePoint();

    for (int k = 0; k < _numberOfReceivers; k++) {
      Object r = _receiverArray[k];
      t1 = ((OhPort)r).getNecessaryTimeTo();
      if (t1.compareTo(t0) > 0)
        t0 = t1;
    }
    return t0;
  }

  
  // 3. imformation output function.

  /**
   * <japanese>
   * ڑo͂܂B <p>
   *
   * ̒[q̐ڑ֌WɊւłXg[ aPw ɏo͂܂B
   *
   * <pre>
   * [in] aPw -- o͐̃Xg[B
   * [in] head - õ͎wb_BftHgl͂SJB
   * </pre>
   * </japanese>
   * <p>
   * The information about the connection relation of this port is outputted to "aPw". <p>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printConnection(PrintWriter aPw, String aHead) {    //; (*5)
    oPrint(aPw, aHead);

    if (_numberOfReceivers <= 0) {
      aPw.print(aHead + "\tNot connected." + NEWLINE);
      return;
    }
    
    StringBuffer sb = new StringBuffer("");

    if (aHead != null)
      sb.append(aHead);

    sb.append("Connected > ");
    String str = sb.toString();

    for (int k = 0; k < _numberOfReceivers; k++) {
      ((OhPort)_receiverArray[k]).oPrint(aPw, str);
    }
  }
  
  // @end {methods} ------------------------------------------------------

} // OhSenderPort
