/**
 * @(#) OhSubSystem; base class for creating subsystems.
 *                   Make "OhSubSystem.java" from "ohsub.h" and "ohsub.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^nf\NXłB<p>
 *
 * nf́AvfEnȂǁA
 * ̃CX^XiȉA\vfA\IuWFNg; componentj ɂ
 * \郂fłB
 * {VXeł́Anf͌ƂĐvZsȂ킸A
 * f[^𒼐ڏƂ͓̗vfɂsȂ̂Ƃ܂B
 * ]āAnf̌vZƂ͍\vfɏvZ߂𑗂邱ƂɂȂ܂B<p>
 *
 * ƂŁAnf́AMpp[qEMpp[qL܂A
 * ̒p[qo^Eڑ葱́Avff̎M[qEM[q̏ꍇ
 * ̎葱ƂĒ`ł܂B<p>
 *
 * ŁA{VXeł͕nfVɍ쐬̂ł͂ȂA
 * vffpč쐬邱ƂƂ܂B
 * nfł́AMpp[qEMpp[q
 * <a href="OhElement.html#_receivePortArray">OhElement._receivePortArray</a>,
 * <a href="OhElement.html#_sendPortArray">OhElement._sendPortArray</a> 
 * ɋLĂ܂B
 * ܂A[q̓o^Ɋւ <a href="OhElement.html">OhElement</a> ̃\bh
 * \vf̓o^ƂɂpA[q̓o^ƑS̎葱ō\vf̓o^sȂA
 * nf쐬҂̗֐߂Ă܂B<p>
 *
 * nfł́AL̂悤ɓo^e\vfEp[qԂ̐ڑ
 * `Ȃ΂Ȃ܂B
 * A{^nfɐڑ̂߂̃\bhɒ`Ă̂ŁA
 * {^nfpč쐬nfł́A
 * ڑƂ͊{^nf̃\bhɈ^݂̂̍ƂɒPĂ܂B<p>
 *
 * ܂AvfEnǂڒʐMpĔvZsȂ߂ɂ́A
 * L镔nǗKv܂B<p>
 *
 * ŁAnfł͔vẐ߂̃\bh`Ă܂B
 *
 * </japanese>
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.97 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhSubSystem extends OhElement
{
  // @begin {fields} ----------------------------------------------------

  /**
   * <japanese>
   * \vf̔złB
   * </japanese>
   * <p>
   * Array for memorizing constituent elements.
   *
   */
  protected Object[] _componentArray;

  /**
   * <japanese>
   * \vf̗vfłB
   * </japanese>
   * <p>
   * Number of constituent elements.
   *
   */
  protected int _numberOfComponents;


  /**
   * <japanese>
   * vZsȂ\vf̔złB
   * </japanese>
   * <p>
   * Array for memorizing the costituent elements which carry out repeated calculation.
   *
   */
  protected Object[][] _iteratorArray;

  /**
   * <japanese>
   * vZsȂvfQ̐łB
   * </japanese>
   * <p>
   * The number of the element groups which carry out repeated calculation.
   *
   */
  protected int _numberOfIteratorSets;


  /**
   * <japanese>
   * ꂼ̗vfQ\vf̐łB
   * </japanese>
   * <p>
   * The number of the elements which constitute an element group.
   *
   */
  protected int[] _numberOfIterators;


  // @end {fields} -------------------------------------------------------


  // @begin {abstract methods} -------------------------------------------

  // 1. creation and reference of "_receivePortArray", "_sendPortArray" and "_componentArray"

  /**
   * <japanese>
   * Mp[qo^܂B<p>
   *
   * hNXŒ`܂B
   * <a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a>
   * Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerReceivePorts() throws OhError;

  /**
   * <japanese>
   * Mp[qo^܂B<p>
   *
   * hNXŒ`܂B
   * <a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a>Ă΂܂B
   * o^ĂȂMp[q͐ڑł܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerSendPorts() throws OhError;

  /**
   * <japanese>
   * vZsȂȂ\IuWFNg _componentArray ɓo^܂B<p>
   *
   * S\vf̂AvZsȂȂ\IuWFNg _componentArray ɓo^܂B
   * o^ĂȂ\IuWFNgɂ͌vZs߂܂B<p>
   *
   * Ⴆ΁AhNXō\IuWFNg abc oϐƂĒ`ꍇA
   * ̂悤 abc o^܂B
   *
   * <pre>
   *  public void registerComponents() {
   *    Register((Object) abc);
   *  }
   * </pre>
   *
   * ̎AK abc  Object ^ɃLXgĉB
   *
   * ɂ abc ́AꎞI _objectList Ɏ^ꂽA
   * ŏII Object ^z _componentArray ɋL܂B
   * i<a href="OhSubSystem.html#construct()">OhSubSystem.construct()</a> QƁj
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerComponents() throws OhError;

  /**
   * <japanese>
   * vZvf _iteratorArray ɓo^܂B<p>
   *
   * hNXŒ`Kv܂A
   * vZsȂȂꍇ͉sȂ悤ɒ`ĂB
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerIterators() throws OhError;

  
  // 2. connection of constituent elements

  /**
   * <japanese>
   * nf̒[q̐ڑ
   * <a href="OhSubSystem.html#connect()">OhSubSystem.connect()</a>
   * pĒ`܂B<p>
   *
   * hNXŒ`܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectPorts() throws OhError;

  /**
   * <japanese>
   * nf̍\IuWFNg̐ڑ`܂B<p>
   *
   * nf̒ڒʐMڑ
   * <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
   * pĒ`܂B
   * Ⴆ΁Â悤ɒ`܂B
   *
   * <pre>
   *  public void connectComponents() {
   *    a.register(b, 0);
   *    b.register(a, 0);
   *    a.register(c, 1);
   *    c.register(a, 0);
   *  }
   * </pre>
   *
   * ŁAa, b, c ͂̕ñoϐłB
   * ɂAvf a  _objectArray[0] ɗvf b i[A
   * vf a  _objectArray[1] ɗvf c i[邱ƂɂȂ܂B
   * vf b  _objectArray[0] ɗvf a i[܂B
   * vf c  _objectArray[0] ɗvf a i[܂B<p>
   *
   * ۃ\bhȂ̂ŔhNXŕK`ĉB
   *
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectComponents() throws OhError;

  // @end {abstract methods} ---------------------------------------------

  
  // @begin {initialization} ---------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAڒʐMvf̔zAvZsȂvfQ̐Af[^
   * ܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aNumOfItr an <code>int</code> value
   * @param aLine a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, int aNumOfObjs, int aNumOfItr, String aLine) throws OhError {
    if (aNumOfItr < 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("Invalid argument for \"_numberOfIteratorSets\"." + NEWLINE);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new IllegalArgumentException(str);
    }

    _numberOfIteratorSets = aNumOfItr;

    return super.init(aObjName, aObjNum, aNumOfObjs, aLine);
  }


  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAڒʐMvf̔zAvZsȂvfQ̐
   * ܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aNumOfItr an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, int aNumOfObjs, int aNumOfItr) throws OhError {
    return init(aObjName, aObjNum, aNumOfObjs, aNumOfObjs, null);
  }


  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAڒʐMvf̔zAf[^
   * ܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aLine a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, int aNumOfObjs, String aLine) throws OhError {
    return init(aObjName, aObjNum, aNumOfObjs, 0, aLine);
  }


  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAڒʐMvf̔z
   * ܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, int aNumOfObjs) throws OhError {
    return init(aObjName, aObjNum, aNumOfObjs, 0, null);
  }


  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAf[^
   * ܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aLine a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, String aLine) throws OhError {
    return init(aObjName, aObjNum, 0, 0, aLine);
  }


  /**
   * <japanese>
   * IuWFNgAIuWFNgԍ܂B
   * ڒʐMvf̔zɊւẮA
   * aNumOfObjs ̗vfzɊmۂ܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum) throws OhError {
    return init(aObjName, aObjNum, 0, 0, null);
  }

  // @end {initialization} -----------------------------------------------

  // @begin {constructors} -----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^͏\bh init() s܂B
   * ̌ʈȉ̐ݒƂȂ܂B
   *
   * <pre>
   *  E_objectName   : null
   *  E_objectNumber : -1
   * </pre>
   *
   * ̑̃oϐɑ΂鏈͈RXgN^ƓłB
   * ̏ꍇA\bh init() păoϐĉB
   * </japanese>
   *
   */
  public OhSubSystem() {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
  }    


  /**
   * <japanese>
   * ̃RXgN^́ANX OhElement ̃ftHgRXgN^ƁA
   * ̃NX̏\bh init() pA
   * oϐȉ̒ʂ菉܂B
   *
   * <pre>
   *  E_className             : "OhSubSystem"
   *  E_objectName            : aObjName
   *  E_objectNumber          : aObjNum
   *  E_owner                 : null
   *  E_objectList            : null
   *  E_receivePortArray      : null
   *  E_numberOfReceivePorts  : -1
   *  E_sendPortArray         : null
   *  E_numberOfSendPorts     : -1
   *  E_elementArray          : null
   *  E_numberOfElements      : -1
   *  E_dataString            : aLine
   *  E_currentTime           : 0
   *  E_timeStep              : -1
   *  E_componemtArray        : null
   *  E_numberOfComponents    : -1
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aNumOfItr an <code>int</code> value
   * @param aLine a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhSubSystem(String aObjName, int aObjNum, int aNumOfObjs, int aNumOfItr, String aLine) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";

    init(aObjName, aObjNum, aNumOfObjs, aNumOfItr, aLine);
  }

  
  /**
   * <japanese>
   * ̃RXgN^́ANX OhElement ̃ftHgRXgN^ƁA
   * ̃NX̏\bh init() pA
   * oϐȉ̒ʂ菉܂B
   *
   * <pre>
   *  E_className             : "OhSubSystem"
   *  E_objectName            : aObjName
   *  E_objectNumber          : aObjNum
   *  E_owner                 : null
   *  E_objectList            : null
   *  E_receivePortArray      : null
   *  E_numberOfReceivePorts  : -1
   *  E_sendPortArray         : null
   *  E_numberOfSendPorts     : -1
   *  E_elementArray          : null
   *  E_numberOfElements      : -1
   *  E_dataString            : null
   *  E_currentTime           : 0
   *  E_timeStep              : -1
   *  E_componemtArray        : null
   *  E_numberOfComponents    : -1
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aNumOfItr an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSubSystem(String aObjName, int aObjNum, int aNumOfObjs, int aNumOfItr) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum, aNumOfObjs, aNumOfItr);
  }


  /**
   * <japanese>
   * ̃RXgN^́ANX OhElement ̃ftHgRXgN^ƁA
   * ̃NX̏\bh init() pA
   * oϐȉ̒ʂ菉܂B
   *
   * <pre>
   *  E_className             : "OhSubSystem"
   *  E_objectName            : aObjName
   *  E_objectNumber          : aObjNum
   *  E_owner                 : null
   *  E_objectList            : null
   *  E_receivePortArray      : null
   *  E_numberOfReceivePorts  : -1
   *  E_sendPortArray         : null
   *  E_numberOfSendPorts     : -1
   *  E_elementArray          : null
   *  E_numberOfElements      : -1
   *  E_dataString            : aLine
   *  E_currentTime           : 0
   *  E_timeStep              : -1
   *  E_componemtArray        : null
   *  E_numberOfComponents    : -1
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @param aLine a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public OhSubSystem(String aObjName, int aObjNum, int aNumOfObjs, String aLine) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum, aNumOfObjs, aLine);
  }
  

  /**
   * <japanese>
   * ̃RXgN^́ANX OhElement ̃ftHgRXgN^ƁA
   * ̃NX̏\bh init() pA
   * oϐȉ̒ʂ菉܂B
   *
   * <pre>
   *  E_className             : "OhSubSystem"
   *  E_objectName            : aObjName
   *  E_objectNumber          : aObjNum
   *  E_owner                 : null
   *  E_objectList            : null
   *  E_receivePortArray      : null
   *  E_numberOfReceivePorts  : -1
   *  E_sendPortArray         : null
   *  E_numberOfSendPorts     : -1
   *  E_elementArray          : null
   *  E_numberOfElements      : -1
   *  E_dataString            : null
   *  E_currentTime           : 0
   *  E_timeStep              : -1
   *  E_componemtArray        : null
   *  E_numberOfComponents    : -1
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfObjs an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSubSystem(String aObjName, int aObjNum, int aNumOfObjs) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum, aNumOfObjs);
  }


  /**
   * <japanese>
   * ̃RXgN^́ANX OhElement ̃ftHgRXgN^ƁA
   * ̃NX̏\bh init() pA
   * oϐȉ̒ʂ菉܂B
   *
   * <pre>
   *  E_className             : "OhSubSystem"
   *  E_objectName            : aObjName
   *  E_objectNumber          : aObjNum
   *  E_owner                 : null
   *  E_objectList            : null
   *  E_receivePortArray      : null
   *  E_numberOfReceivePorts  : -1
   *  E_sendPortArray         : null
   *  E_numberOfSendPorts     : -1
   *  E_elementArray          : null
   *  E_numberOfElements      : -1
   *  E_dataString            : null
   *  E_currentTime           : 0
   *  E_timeStep              : -1
   *  E_componemtArray        : null
   *  E_numberOfComponents    : -1
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public OhSubSystem(String aObjName, int aObjNum) throws OhError {
    super();
    _componentArray     = null;
    _numberOfComponents = -1;
    _className          = "OhSubSystem";
    
    init(aObjName, aObjNum);
  }

  // @end {constructors} -------------------------------------------------


  // @begin {methods} ----------------------------------------------------
  
  // 1. creation and reference of "_receivePortArray", "_sendPortArray" and "_componentArray"

  /**
   * <japanese>
   * _iteratorArray vf _numberOfIteratorAets ̔zɕϊ܂B
   * vZsȂꍇɌĂяo郁\bhłB
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void initializeIteratorArray() throws OhError {                   //; (*8)
    if ((_iteratorArray = new Object[256][_numberOfIteratorSets]) == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot get memory for \"_iteratorArray\".", str);
    }

    if ((_numberOfIterators = new int[_numberOfIteratorSets]) == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot get memory for \"_numberOfIterators\".", str);
    }

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      _iteratorArray[i]     = null;
      _numberOfIterators[i] = -1;
    }
  }


  /**
   * <japanese>
   * [q̓o^ђڒʐMvf̓o^sȂ܂B
   * </japanese>
   *
   * @param aP an <code>Object[]</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean register(Object[] aP) throws OhError {    //; (*9)

    // Look for "i" which is "_iteratorArray = null".
    // Exit if it is nothing.
    //
    int i = 0;
    boolean flag = false;

    for (i = 0; i < _numberOfIteratorSets; i++) {
      if (_iteratorArray[i] == null) {
        flag = true;
        break;
      }
    }

    if (flag == false) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("_iteratorArray is already full. Please check " + NEWLINE);
      out.print("\"_numberOfIteratorSets\"." + NEWLINE);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    int j = 0;
    
    for (j = 0; j < aP.length; j++) {      // Here, Added to "_objectList".
      if (register(aP[j]) == false) {      
        PrintWriter cerr = new PrintWriter(System.err, true);
        cPrint(cerr, "\t");
        return false;
      }
    }

    int[] numberOfIteratorsArray = {_numberOfIterators[i]};
    
    _iteratorArray[i]     = makeObjectArray(numberOfIteratorsArray);    //; OhElement (*4)
    _numberOfIterators[i] = numberOfIteratorsArray[0];

    _objectList.clear();

    return true;
  }


  /**
   * <japanese>
   * [q̓o^ђڒʐMvf̓o^sȂ܂B
   * </japanese>
   *
   * @param aP an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean register(Object aP) throws OhError {
    return super.register(aP);
  }


  /**
   * <japanese>
   * [q̓o^ђڒʐMvf̓o^sȂ܂B
   * </japanese>
   *
   * @param aP an <code>Object</code> value
   * @param aObj an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean register(Object aP, int aObj) throws OhError {
    return super.register(aP, aObj);
  }

  
  /**
   * <japanese>
   * NXAIuWFNgAIuWFNgԍw肵āA
   * _componentArray \IuWFNg(Object ^)擾܂B<p>
   *
   * w肳ꂽIuWFNgo^ĂȂꍇ́Anull Ԃ܂B<p>
   *
   * _numberOfComponents ̏ꍇA_componentArray 쐬ĂȂƔfA
   * G[bZ[W\Anull Ԃ܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaClassName -> 擾\IuWFNg̃NXB
   *  EaObjName ---> 擾\IuWFNg̃IuWFNgB
   *  EaObjNum ----> 擾\IuWFNg̃IuWFNgԍB
   * [RETURN]
   *  EObject -----> w肳ꂽ\IuWFNgBw̃IuWFNg擾łȂꍇ nullB
   * [ACCIDENT]
   *  E_componentArray  Object ^ȊO̕ϐ^ĂꍇB
   * </pre>
   * </japanese>
   *
   * @param aClassName a <code>String</code> value
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getComponent(String aClassName, String aObjName, int aObjNum) throws OhError {    //; (*10)
    Object p = null;

    if ((p = getFromArray(aClassName, aObjName, aObjNum, _componentArray, _numberOfComponents)) != null)
      return p;
    

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      if ((p = getFromArray(aClassName, aObjName, aObjNum, _iteratorArray[i], _numberOfIterators[i])) != null)
        return p;
    }

    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);
    out.print("Cannot get component from array." + NEWLINE);
    out.print("\tAsked > " + aClassName + " : " + aObjName + " : " + aObjNum + NEWLINE);
    oPrint(out, " > ");
    String str = buffer.toString();
    out.close();
    throw new OhError(str);
  }

  
  /**
   * <japanese>
   * IuWFNgAIuWFNgԍw肵āA
   * _componentArray \IuWFNg(Object ^)擾܂B<p>
   *
   * w肳ꂽIuWFNgo^ĂȂꍇ́Anull Ԃ܂B<p>
   *
   * _numberOfComponents ̏ꍇA_componentArray 쐬ĂȂƔfA
   * G[bZ[W\Anull Ԃ܂B<p>
   *
   * <pre>
   * [ARGUMENT]
   *  EaObjName ---> 擾\IuWFNg̃IuWFNgB
   *  EaObjNum ----> 擾\IuWFNg̃IuWFNgԍB
   * [RETURN]
   *  EObject -----> w肳ꂽ\IuWFNgBw̃IuWFNg擾łȂꍇ nullB
   * [ACCIDENT]
   *  E_componentArray  Object ^ȊO̕ϐ^ĂꍇB
   * </pre>
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @return an <code>Object</code> value
   * @exception OhError if an error occurs
   */
  public Object getComponent(String aObjName, int aObjNum) throws OhError {    //; (*11)
    Object p = null;

    if ((p = getFromArray(aObjName, aObjNum, _componentArray, _numberOfComponents)) != null)
      return p;
    

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      if ((p = getFromArray(aObjName, aObjNum, _iteratorArray[i], _numberOfIterators[i])) != null)
        return p;
    }

    StringWriter buffer = new StringWriter();
    PrintWriter  out    = new PrintWriter(buffer);
    out.print("Cannot get component from array." + NEWLINE);
    out.print("\tAsked > " + aObjName + " : " + aObjNum + NEWLINE);
    oPrint(out, " > ");
    String str = buffer.toString();
    out.close();
    throw new OhError(str);
  }

  
  // 2. connection of constituent elements

  /**
   * <japanese>
   * e\vfEp[qԂ̐ڑڑsȂ܂B<p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * Mp[q܂͑Šn̓͒[qƁA
   * \IuWFNg̎M[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[q́A_className, _objectName, _objectNumber 
   * ʂ܂B<p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner sȂ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aMyP an <code>Object</code> value
   * @param aR an <code>Object</code> value
   * @param aRpClassName a <code>String</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean connect(Object aMyP, Object aR, String aRpClassName,
    String aRpName, int aRpNum) throws OhError {                           //; (*1)

    // Gets the receiving port of an internal object.
    //
    Object rp = ((OhElement)aR).getReceivePort(aRpClassName, aRpName, aRpNum);

    if (rp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot connect ports. Cannot get '_receivePort' to connect.", str);
    }

    return ((((OhPort)rp).setSender(aMyP) == true)
      && (((OhPort)aMyP).registerReceiver(rp) == true)) ? true : false;
  }

  
  /**
   * <japanese>
   * e\vfEp[qԂ̐ڑsȂ܂B<p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q́A_className, _objectName, _objectNumber 画ʂ܂B<p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner sȂ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>Object</code> value
   * @param aSpClassName a <code>String</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aR an <code>Object</code> value
   * @param aRpClassName a <code>String</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean connect(Object aS, String aSpClassName, String aSpName, int aSpNum,
    Object aR, String aRpClassName, String aRpName, int aRpNum) throws OhError {          //; (*2)

    // Gets the sending port of an internal object.
    //
    Object sp = ((OhElement)aS).getSendPort(aSpClassName, aSpName, aSpNum);
    
    if (sp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot get '_sendPort' to connect.", str);
    }
    
    // Gets the receiving port of an internal object.
    //
    Object rp = ((OhElement)aR).getReceivePort(aRpClassName, aRpName, aRpNum);

    if (rp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot get '_receivePort' to connect.", str);
    }

    return ((((OhPort)rp).setSender(sp) == true)
      && (((OhPort)sp).registerReceiver(rp) == true)) ? true : false;
  }


  /**
   * <japanese>
   * e\vfEp[qԂ̐ڑsȂ܂B<p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[qƁAMp[q܂͑Šn̏o͒[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[q́A_className, _objectName, _objectNumber 画ʂ܂B<p>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>Object</code> value
   * @param aSpClassName a <code>String</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aMyP an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean connect(Object aS, String aSpClassName, String aSpName,
    int aSpNum, Object aMyP) throws OhError {                          //; (*3)

    // Gets the sending port to connect.
    //
    Object sp = ((OhElement)aS).getSendPort(aSpClassName, aSpName, aSpNum);

    if (sp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot connect ports. Cannot get '_sendPort' to connect.", str);
    }

    return ((((OhPort)aMyP).setSender(sp) == true)
      && (((OhPort)sp).registerReceiver(aMyP) == true)) ? true : false;
  }

  
  /**
   * <japanese>
   * e\vfEp[qԂ̐ڑsȂ܂B<p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * Mp[q܂͑Šn̓͒[qƁA\IuWFNg̎M[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[q́A_objectName, _objectNumber 画ʂ܂B<p>
   * ܂ANX̃`FbNȗ܂B<p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner sȂ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aMyP an <code>Object</code> value
   * @param aR an <code>Object</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean connect(Object aMyP, Object aR, String aRpName, int aRpNum) throws OhError {  //; (*4)

    // Gets the receiving port of an internal object.
    //
    Object rp = ((OhElement)aR).getReceivePort(aRpName, aRpNum);

    if (rp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot connect ports. Cannot get '_receivePort' to connect.", str);
    }

    return ((((OhPort)rp).setSender(aMyP) == true)                         //; OhReceivePort (*1)
      && (((OhPort)aMyP).registerReceiver(rp) == true)) ? true : false;    //; OhSenderPort (*1)
  }


  /**
   * <japanese>
   * e\vfEp[qԂ̐ڑsȂ܂B<p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q̐ڑsȂ܂B<p>
   *
   * \IuWFNg̎M[qAM[q́A_objectName, _objectNumber 画ʂ܂B
   * ܂ANX̃`FbNȗ܂B<p>
   *
   * Lp[qƕnfO̎M[qƂ̐ڑ _owner sȂ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>Object</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aR an <code>Object</code> value
   * @param aRpName a <code>String</code> value
   * @param aRpNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean connect(Object aS, String aSpName, int aSpNum,
    Object aR, String aRpName, int aRpNum) throws OhError {   //; (*5)

    // Gets the sending port of an internal object.
    //
    Object sp = ((OhElement)aS).getSendPort(aSpName, aSpNum);

    if (sp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot connect ports. Cannot get '_sendPort' to connect.", str);
    }

    // Gets the receiving port of an internal object.
    //
    Object rp = ((OhElement)aR).getReceivePort(aRpName, aRpNum);

    if (rp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot connect ports. Cannot get '_receivePort' to connect.", str);
    }

    return ((((OhPort)rp).setSender(sp) == true)
         && (((OhPort)sp).registerReceiver(rp) == true)) ? true : false;
  }


  /**
   * <japanese>
   * e\vfEp[qԂ̐ڑsȂ܂B<p>
   *
   * nf̒[qԂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[qƁAMp[q܂͑Šn̏o͒[qƂ̐ڑsȂ܂B<p>
   *
   * \IuWFNg̑M[q́A_objectName, _objectNumber 画ʂ܂B<p>
   * ܂ANX̃`FbNȗ܂B<p>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> ɐڑłꍇ.
   *  Efalse -> w肵\IuWFNg܂͒[q擾łȂꍇB
   *             OhPort.setSender() ̖߂l false ̏ꍇB
   *             OhPort.registerReceiver() ̖߂l false ̏ꍇB
   * </pre>
   * </japanese>
   *
   * @param aS an <code>Object</code> value
   * @param aSpName a <code>String</code> value
   * @param aSpNum an <code>int</code> value
   * @param aMyP an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean connect(Object aS, String aSpName, int aSpNum, Object aMyP) throws OhError {  //; (*6)

    // Gets the sending port to connect.
    //
    Object sp = ((OhElement)aS).getSendPort(aSpName, aSpNum);

    if (sp == null) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot connect ports. Cannot get '_sendPort' to connect.", str);
    }

    return ((((OhPort)aMyP).setSender(sp) == true)
         && (((OhPort)sp).registerReceiver(aMyP) == true)) ? true : false;
  }


  /**
   * <japanese>
   * e\vfEp[qԂ̐ڑsȂ܂B<p>
   *
   * nf̒[qԂ̐ڑ
   * </japanese>
   *
   * @param aMySp an <code>Object</code> value
   * @param aMyRp an <code>Object</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean connect(Object aMySp, Object aMyRp) throws OhError {    //; (*7)
    return ((((OhPort)aMyRp).setSender(aMySp) == true)
      && (((OhPort)aMySp).registerReceiver(aMyRp) == true)) ? true : false;
  }


  /**
   * <japanese>
   * _receivePortArray, _sendPortArray, _objectArray 쐬A
   * ڑ֌W̒`ɏ]ĕn̐ڑsȂ܂B<p>
   *
   * ̕nfA\IuWFNg̍\ݒ肵܂B
   * ̓Iɂ́Aȉ̎葱sȂ܂B
   *
   * <ol>
   *  <li> OhLinkedList ^IuWFNgq[vɍ쐬 _objectList ɐݒ肷B
   *  <li> <a href="OhElement.html#registerReceivePorts()">OhElement.registerReceivePorts()</a>,
   *       <a href="OhElement.html#makeObjectArray()">OhElement.makeObjectArray()</a> 
   *       ɂ _receivePortArray 쐬B
   *  <li> <a href="OhLinkedList.html#clear()">OhLinkedList.clear()</a> 
   *       ɂ _objectList ɂB
   *  <li> <a href="OhElement.html#registerSendPorts()">OhElement.registerSendPorts()</a>,
   *       <a href="OhElement.html#makeObjectArray()">OhElement.makeObjectArray()</a>
   *       ɂ _sendPortArray 쐬B
   *  <li> <a href="OhLinkedList.html#clear()">OhLinkedList.clear()</a>
   *       ɂ _objectList ɂB
   *  <li> <a href="OhSubSystem.html#registerComponents()">OhSubSystem.registerComponents()</a>,
   *       <a href="OhElement.html#makeObjectArray()">OhElement.makeObjectArray()</a>
   *       ɂ _componentArray 쐬B
   *  <li> <a href="OhSubSystem.html#initializeIteratorArray()">OhSubSystem.initializeIteratorArray()</a> 
   *       ɂ _iteratorArray ̑ꎟzmۂB
   *  <li> <a href="OhSubSystem.html#registerIterators()">OhSubSystem.registerIterators()</a>
   *       ɂ _iteratorArray 쐬B
   *  <li> _objectList  null ɐݒ肷B
   *  <li> <a href="OhElement.html#construct()">OhElement.construct()</a> ɂA\IuWFNg̍\ݒ肷B
   *  <li> <a href="OhSubSystem.html#connectComponents()">OhSubSystem.connectComponent()</a>
   *       ɂAnf̐ڑsȂB
   * </ol>
   *
   * <pre>
   * [RETURN]
   *  Etrue --> nf̍\ɐݒ肳ꂽꍇB
   *  Efalse -> ̃IuWFNg̍\ݒɈُ킪ꍇB
   *             \IuWFNg̍\ݒɈُ킪ꍇB
   *             nf̐ڑɈُ킪ꍇB
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean construct() throws OhError {    //; (*12)
    _objectList = new OhLinkedList();
    
    // --- Makes _receivePortArray. ---

    registerReceivePorts();    // Abstract method call.
                               //; Throws "OhError".
                               //; OhElement  (*3)register() pāA
                               //; ̌n̒[q _bufferList ɓo^B

    int[] numberOfReceivePortsArray = {_numberOfReceivePorts};

    _receivePortArray     = makeObjectArray(numberOfReceivePortsArray);  //; OhElement (*4)
    _numberOfReceivePorts = numberOfReceivePortsArray[0];
    
    _objectList.clear();
    
    // --- Makes _sendPortArray. ---

    registerSendPorts();    // Abstract method call. Throws "OhError".

    int[] numberOfSendPortsArray = {_numberOfSendPorts};
    
    _sendPortArray     = makeObjectArray(numberOfSendPortsArray);
    _numberOfSendPorts = numberOfSendPortsArray[0];

    _objectList.clear();
    
    // --- Makes _componentArray. ---

    registerComponents();

    int[] numberOfComponentsArray = {_numberOfComponents};
    
    _componentArray     = makeObjectArray(numberOfComponentsArray);
    _numberOfComponents = numberOfComponentsArray[0];

    
    _objectList.clear();


    initializeIteratorArray();    //; (*8)
    registerIterators();          // Abstract method call.

    _objectList = null;

    boolean flag = true;
    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++)
      if (((OhElement)_componentArray[k]).construct() != true)
        flag = false;

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if (((OhElement)_iteratorArray[i][k]).construct() != true)
          flag = false;
      }
    }

    if (flag != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot construct \"components\".", str);
    }

    connectPorts();     //; (*4)
    connectComponents();

    return true;
  }


  /**
   * <japanese>
   * MڑƂIɂ̃\bhĂяoA
   * MJn̂߂̏Ƃ܂B<p>
   *
   * ܂ OhElement.canYouSwitch() sA̖߂l false ̏ꍇA
   * G[bZ[W\Afalse Ԃ܂B<p>
   *
   * ɁA\IuWFNg canYouSwitch () s܂B
   * ŁA߂l false ̍\IuWFNg݂ꍇ́A
   * S\IuWFNg canYouSwitch() sA
   * G[bZ[W\ false Ԃ܂B<p>
   *
   * nfɑ݂S[q canYouSwitch() s܂B<p>
   *
   * M[qł́A_receiverArray 쐬Af[^pbN^̏܂B
   * ̓IɌ΁A_dataPackArray ̃mۂA
   * _numberOfDataPacks ̒l -1  0 ɂ܂B
   * p[qł _receiverArray 쐬邾łB<p>
   *
   * ܂Ƃ߂ƁAȉsȂƂɂȂ܂B
   *
   * <ol>
   *  <li> _receiverArray 쐬B
   *  <li> _numberOfDataPacks ̒l -1  0 ɕύXB
   * </ol>
   *
   * <pre>
   * [RETURN]
   *   Etrue --> nf̑S[q𐳏ɓ]łꍇB
   *   Efalse -> nf̒[q̓, ɓ]łȂ݂̂ꍇB
   * </pre>
   *
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean canYouSwitch() throws OhError {    //; (*15)
    if (super.canYouSwitch() != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    boolean flag = true;    // For judgment.
    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++)
      if (((OhElement)_componentArray[k]).canYouSwitch() != true)
        flag = false;      // The element model which cannot "Switch" exists.

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if (((OhElement)_iteratorArray[i][k]).canYouSwitch() != true)
          flag = false;
      }
    }

    if (flag != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot switch those 'components'.", str);
    }

    return flag;
  }


  /**
   * <japanese>
   * vZJnO̊mFsȂ܂B<p>
   *
   * ̃\bh̖߂l true Ȃ OhSubSystem ͌vZJnł܂B<p>
   *
   * ܂ OhElement.areYouReady() pďLMp[qALMp[qɑ΂
   * vZO̊mFƂs܂B
   * ̖߂l false ̏ꍇAG[bZ[W\ false Ԃ܂B <p>
   *
   * ɁAS\IuWFNg areYouReady() pČvZO̊mFƂsȂ܂B
   * ŁA߂l false ̃IuWFNg݂ꍇAS\IuWFNg
   * areYouRedy() ŊmFG[bZ[W\ false Ԃ܂B
   *
   * <pre>
   * [RETURN]
   *  Etrue --> vZJnB
   *  Efalse -> vZJnsB
   * </pre>
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean areYouReady() throws OhError {     //; (*16)
    if (super.areYouReady() != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    boolean flag = true;    // For judgment.
    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++)
      if (((OhElement)_componentArray[k]).areYouReady() != true)
        flag = false;    // The element model which cannot start calculating exists.

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if (((OhElement)_iteratorArray[i][k]).areYouReady() != true)
          flag = false;
      }
    }

    if (flag != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("These 'components' are not ready.", str);
    }

    return flag;
  }


  
  // 3. initialization function

  /**
   * <japanese>
   * \IuWFNg̃f[^܂B
   * </japanese>
   * <p>
   * Deletion of the string of components.
   *
   */
  public void clearDataString() {    //; (*17)
    super.clearDataString();

    for (int k = 0; k < _numberOfComponents; k++)
      ((OhElement)_componentArray[k]).clearDataString();

    for (int i = 0; i < _numberOfIteratorSets; i++) {
      for (int k = 0; k < _numberOfIterators[i]; k++) {
        ((OhElement)_iteratorArray[i][k]).clearDataString();
      }
    }
  }
      

  /**
   * <japanese>
   * p^ݒ肷郁\bhłAȉ̗RŉȂ悤ɒ`Ă܂B<p>
   *
   * ̃\bh́A
   * <a href="OhElement.html">OhElement</a> NX̒ۃ\bhłB
   * WIȕnf́AI\IuWFNgɂsȂ̂ŁA
   * p[^AԗʂȂ̂ƍl܂B
   * ŁÃNXł͉Ȃ悤ɒ`܂B
   * KvȂΔhNXŕύXĉB
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setParameter(OhBufferedReader aBr) throws OhError {}


  /**
   * <japanese>
   * ̃\bh́Ang̃p^ƁAn̍\vf̃p^ǂݎ܂B
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void getParameter(OhBufferedReader aBr) throws OhError {    //; (*18)
    setParameter(aBr);

    try {
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError("Parameter file error.");
        throw e;
      }

      if ((char)aBr.read() != '{') {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("Now reading parameters for ");
        oPrint(out, " > ");
        String str = buffer.toString();
        out.close();
        throw new OhError("{ should come first", str);
      }
      
      aBr.readLine();

      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;
      
      while (true) {
        if (aBr.skipWhites() == -1) {
          OhError e = new OhError("Parameter file error.");
          throw e;
        }
        
        headLine = aBr.readLine();
        
        if (headLine.substring(0, 1).equals("}"))
          break;
        
        String s = headLine;
        StringTokenizer st = new StringTokenizer(s);
        int i = 0;
        while (st.hasMoreTokens()) {
          st.nextToken();
          i++;
        }

        strToken = new StringTokenizer(headLine);
        
        if (i == 1) {
          objName = strToken.nextToken();
          obj = this.getComponent(objName, 0);
        }
        else if (i == 2) {
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(objName, objNum);
        }
        else if (i == 3) {
          className = strToken.nextToken();
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(className, objName, objNum);
        }
        else {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          oPrint(out, "    ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError(errStr);
        }

        ((OhElement)obj).getParameter(aBr);

      }
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in parameter line.", str);
    }
  }


  /**
   * <japanese>
   * \IuWFNg̎܂B<p>
   *
   * _currentTime ǂݍ݂܂B
   * ǂݍ񂾒l 0 ȂA_currentTime ύX܂B
   * </japanese>
   * <p>
   * Initialization of the time of constituent objects. <p>
   *
   * "_currentTime" is read.
   * If it is 0, "_currentTime" will not be changed.
   *
   * @param aInitTime an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean isInitialTime(OhTimePoint aInitTime) throws OhError {    //; (*23)
    if (super.isInitialTime(aInitTime) != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    boolean flag = true;    // For judgment.
    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++)
      if (((OhElement)_componentArray[k]).isInitialTime(aInitTime) != true)
        flag = false;

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if (((OhElement)_iteratorArray[i][k]).isInitialTime(aInitTime) != true)
          flag = false;
      }
    }

    if (flag != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError("Cannot set initial time. At those 'components' of this 'OhSubSystem' error.", str);
    }

    return flag;
  }


  /**
   * <japanese>
   * ng̏ԗʂݒ肵܂B<p>
   *
   * ng̃NXAIuWFNgAIuWFNgԍǂݎA
   * ԗʂĂs̐擪ɓǂݎʒuԂł̃\bhĂ΂܂B<p>
   * 
   * ԗʂǂ݂ƂAt@C|C^͏ԂǂݎsɒuĂ܂B<p>
   *
   * ̃\bh́AgetInitialState() Ă΂܂B
   * </japanese>
   * <p>
   * Sets the the initial state of sub systems itself.
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setInitialState(OhBufferedReader aBr) throws OhError {      //; (*19)

    int  year = 0;
    long tsec = 0L;

    try {
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }

    if (year > 0) {
      _currentTime = new OhTimePoint(year, tsec);
    }
  }


  /**
   * <japanese>
   * ng̏ԗʂƍ\IuWFNg̏ԗʂ̐ݒsȂ܂B
   * </japanese>
   * <p>
   * Sets the the initial state of sub systems itself and components.
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void getInitialState(OhBufferedReader aBr) throws OhError {    //; (*20)

    setInitialState(aBr);
    
    try {
      if (aBr.skipWhites() == -1) {
        OhError e = new OhError("Initial file error in \"OhSubSystem\".");
        throw e;
      }
      
      if ((char)aBr.read() != '{') {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("Now reading parameters for ");
        oPrint(out, " > ");
        String str = buffer.toString();
        out.close();
        throw new OhError("{ should come first", str);
      }
      
      aBr.readLine();
      
      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;
      
      while (true) {
        if (aBr.skipWhites() == -1) {
          OhError e = new OhError("Parameter file error.");
          throw e;
        }
        
        headLine = aBr.readLine();
        
        if (headLine.substring(0, 1).equals("}"))
          break;
        
        String s = headLine;
        StringTokenizer st = new StringTokenizer(s);
        int i = 0;
        while (st.hasMoreTokens()) {
          st.nextToken();
          i++;
        }

        strToken = new StringTokenizer(headLine);
        
        if (i == 1) {
          objName = strToken.nextToken();
          obj = this.getComponent(objName, 0);
        }
        else if (i == 2) {
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(objName, objNum);
        }
        else if (i == 3) {
          className = strToken.nextToken();
          objName = strToken.nextToken();
          objNum = Integer.parseInt(strToken.nextToken());
          obj = this.getComponent(className, objName, objNum);
        }
        else {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          oPrint(out, "    ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError(errStr);
        }
        
        ((OhElement)obj).getInitialState(aBr);
        
      }
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in initial state line.", str);
    }
  }


  /**
   * <japanese>
   * ng̍ŏIԂo̓Xg[ aPw ɏo܂B
   * ̏ꍇ́Ã\bhpăt@Cɕۑ܂B
   * </japanese>
   * <p>
   * Save to the file of the own last state of the subsystems.
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveTerminalState(PrintWriter aPw) {
    if (aPw != null)
      aPw.print(_currentTime.getYear() + " " + _currentTime.getTsec());
  }
      


  
  /**
   * <japanese>
   * ng̍ŏIԂƍ\vf̍ŏIԂt@CXg[ aPw ɏo܂B<p>
   *
   * ̃\bh́A\vfɍŏIԂo悤Ɏw߂𑗂܂B
   * </japanese>
   * <p>
   * Save the last state of the own subsystems and the components.
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void terminalState(PrintWriter aPw) {    //; (*24)
    int i = 0;
    int k = 0;

    if (aPw != null) {
      // Save the last state of the own subsystem.
      //
      aPw.print(_className + " " + _objectName + " " + _objectNumber + NEWLINE);
      saveTerminalState(aPw);
      aPw.print(NEWLINE);

      // Save the last state of the components.
      //
      aPw.print("{" + NEWLINE);
      for (k = 0; k < _numberOfComponents; k++) {
        if (k > 0) {
          aPw.print(NEWLINE);
        }
        ((OhElement)_componentArray[k]).terminalState(aPw);
      }

      for (i = 0; i < _numberOfIteratorSets; i++) {
        for (k = 0; k < _numberOfIterators[i]; k++) {
          aPw.print(NEWLINE);
          ((OhElement)_iteratorArray[i][k]).terminalState(aPw);
        }
      }
      aPw.print("}" + NEWLINE);
    }
  }


  /**
   * <japanese>
   * ̃\bh́A܂gɑ΂ canYouShareInfo() sA
   * Ɏ̍\vfɑ΂ doShareInfo() s܂B
   * ܂߂Ĉł canYouShareInfo() svf
   * <a href="OperationFlag.html#E_DONE">OperationFlag.E_DONE</a>
   * Ԃ܂B
   * SĂ̗vfł canYouShareInfo sĂ
   * <a href="OperationFlag.html#E_ALREADY">OperationFlag.E_ALREADY</a>
   * Ԃ܂B
   * ǂ̏ꍇłȂƂ
   * <a href="OperationFlag.html#E_UNDONE">OperationFlag.E_UNDONE</a>
   * Ԃ܂B
   * </japanese>
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doShareInfo() throws OhError {    //; (*14)
    OperationFlag flag;

    boolean calcany = false;
    boolean calcall = true;

    if (_shareInfoDoneP == false) {
      if (canYouShareInfo() == true) {
        _shareInfoDoneP = true;
        calcany = true;
      }
    }

    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++) {
      flag = ((OhElement)_componentArray[k]).doShareInfo();
      if (flag == OperationFlag.E_DONE)
        calcany = true;
      if (flag == OperationFlag.E_UNDONE)
        calcall = false;
    }

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        flag = ((OhElement)_iteratorArray[i][k]).doShareInfo();
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }
    }

    if (calcany == true)
      return OperationFlag.E_DONE;
    if (calcall == true)
      return OperationFlag.E_ALREADY;
    
    return OperationFlag.E_UNDONE;
  }

  
  /**
   * <japanese>
   * n̍\vf̂ꂼɎw߂𑗂A
   * o̓Xg[ aPw Ŏw肳Ăt@Cɏo܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveSendPortStock(PrintWriter aPw) {    //; (*21)
    int i = 0;
    int k = 0;

    if (aPw != null) {
      if (_numberOfComponents <= 0)
        return;

      if (_numberOfComponents > 0) {
        aPw.print(_className + " " + _objectName + " " + _objectNumber + NEWLINE);
        aPw.print("{" + NEWLINE);

        for (k = 0; k < _numberOfComponents; k++) {
          ((OhElement)_componentArray[k]).saveSendPortStock(aPw);
        }

        for (i = 0; i < _numberOfIteratorSets; i++) {
          for (k = 0; k < _numberOfIterators[i]; k++) {
            ((OhElement)_iteratorArray[i][k]).saveSendPortStock(aPw);
          }
        }

        aPw.print("}" + NEWLINE);
      }
    }
  }


  // 4. information output function

  /**
   * <japanese>
   * \IuWFNg̏o͂܂B
   * </japanese>
   * <p>
   * Prints out the information on a constituent objects.
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printComponents(PrintWriter aPw, String aHead) {    //; (*22 cpp)
    printObjects(aPw, aHead, "Components :", _componentArray, _numberOfComponents);

    for (int i = 0; i < _numberOfIteratorSets; i++)
      printObjects(aPw, aHead, "Components :", _iteratorArray[i], _numberOfIterators[i]);
  }


  /**
   * <japanese>
   * \IuWFNg̏o͂܂B
   * </japanese>
   * <p>
   * Prints out the information on a constituent objects.
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void printComponents(PrintWriter aPw) {                  //; (*22 h)
    printObjects(aPw, "    ", "Components :", _componentArray, _numberOfComponents);

    for (int i = 0; i < _numberOfIteratorSets; i++)
      printObjects(aPw, "    ", "Components :", _iteratorArray[i], _numberOfIterators[i]);
  }



  // 5. the execution function of calculation

  /**
   * <japanese>
   * ̏o͂sȂ܂B<p>
   *
   * ԗʂ߂ɁA_currentTime ł̏o͂`܂B
   * M[qɎƏo͒l݂܂B
   * _currnet_time ł̏o͂vZłꍇ true Ԃ܂B
   * Kvȓ̓f[^ĂȂȂǂāA
   * _currentTime ̏o͂vZłȂꍇ false Ԃ܂B<p>
   *
   * ȂȀꍇ͕n̏̏o͂͂ȂƎv̂ŁA
   * ł͏ true Ԃ悤ɒ`Ă܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean initialOutput() throws OhError {
    return true;
  }

  
  /**
   * <japanese>
   * ƂsȂ܂B<p>
   *
   * ɂāASf[^擾܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean initialWork() throws OhError {
    return true;
  }

  
  /**
   * <japanese>
   *  initialWork() Iǂ𔻒f܂B<p>
   * </japanese>
   * <p>
   * A first-time output.
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doInitialWork() throws OhError {    //; (*13)
    OperationFlag flag;

    boolean calcany = false;
    boolean calcall = true;

    if (_initialWorkDoneP == false) {
      if (initialWork() == true) {
        _initialWorkDoneP = true;
        calcany = true;
      }
    }

    int i = 0;
    int k = 0;

    for (k = 0; k < _numberOfComponents; k++) {
      flag = ((OhElement)_componentArray[k]).doInitialWork();    //; Throws "OhError".
      if (flag == OperationFlag.E_DONE)
        calcany = true;
      if (flag == OperationFlag.E_UNDONE)
        calcall = false;
    }

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        flag = ((OhElement)_iteratorArray[i][k]).doInitialWork();    //; Throws "OhError".
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }
    }

    if (calcany == true)
      return OperationFlag.E_DONE;
    if (calcall == true)
      return OperationFlag.E_ALREADY;
    
    return OperationFlag.E_UNDONE;
  }
    

  /**
   * <japanese>
   * vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎ׂf[^̃f[^L^Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   *
   * @param aRp an <code>Object</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeFrom(Object aRp) {    //; (*25)
    int k = 0;
    int j = -1;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      if (aRp == _receivePortArray[k])
        j = k;
    }

    if (j == -1)
      return _currentTime;

    return ((OhSenderPort)aRp).getNecessaryTimeFrom();
  }


  /**
   * <japanese>
   * vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎ׂf[^̃f[^L^Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   *
   * @param aRp an <code>Object</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeTo(Object aRp) {    //; (*26)
    int k = 0;
    int j = -1;

    for (k = 0; k < _numberOfReceivePorts; k++) {
      if (aRp == _receivePortArray[k])
        j = k;
    }

    if (j == -1)
      return _currentTime;

    return ((OhSenderPort)aRp).getNecessaryTimeTo();
  }



  /**
   * <japanese>
   * vZO̊mFsȂ܂B
   * ł́A true Ԃ悤ɒ`Ă܂B
   * </japanese>
   * <p>
   * The check before calculation.
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean canYouCalculate() throws OhCannotGetDataError, OhError {
    return true;
  }


  /**
   * <japanese>
   * ^CXebv̌vZs܂B
   * ł́A 0 Ԃ悤ɒ`Ă܂B
   * </japanese>
   * <p>
   * Calculation of the time step.
   *
   * @return a <code>long</code> value
   * @exception OhError if an error occurs
   */
  public long calculateTimeStep() throws OhError {
    return 0L;
  }


  /**
   * <japanese>
   * PXebv̌vZsȂ܂B
   * ł͏ true Ԃ悤ɒ`Ă܂B
   * </japanese>
   * <p>
   * Calculation for 1 step.
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean calculate() throws OhCannotGetDataError, OhError {
    return true;
  }


  /**
   * <japanese>
   * vZvfQvZڕWɓBǂ`FbN܂B<p>
   *
   * ̃NXł́AwǗҁxƂĂ OhSubSystem ̗ꂩ`Ă܂B
   * Ȃ킿A̓ɔvZsȂvfQƂƂOƂĂA
   * gvZsȂꍇɑΉĒ`Ă킯ł͂܂B
   * āAgvZsȂꍇ́Aꂪ\ł悤ɍĒ`ĉB
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean doYouReachTargetTime(int aNum) {    //; (*27)
    boolean flag = true;

    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      if (((OhElement)_iteratorArray[aNum][i]).doYouReachTargetTime() == false)
        flag = false;

    return flag;
  }


  /**
   * <japanese>
   * vZvfQ`FbN܂B<p>
   *
   * ̃NXł́AwǗҁxƂĂ OhSubSystem ̗ꂩ`Ă܂B
   * Ȃ킿A̓ɔvZsȂvfQƂƂOƂĂA
   * gvZsȂꍇɑΉĒ`Ă킯ł͂܂B
   * āAgvZsȂꍇ́Aꂪ\ł悤ɍĒ`ĉB
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean doYouConverge(int aNum) throws OhError {    //; (*28)
    boolean flag = true;

    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      if (((OhElement)_iteratorArray[aNum][i]).doYouConverge() == false)
        flag = false;

    return flag;
  }

  
  /**
   * <japanese>
   * ݎ̍XVsȂ܂B<p>
   *
   * _componentArray ɓo^Ăvf _currentTime 
   * _iteratorArray ɓo^Ăvf _fixedTime ̂A
   * łxĂ _currentTime ƂB
   * </japanese>
   * <p>
   * Renewal of the current time.
   *
   * @param aNum an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouRenewCurrentTime(int aNum) throws OhError {
    int k = 0;
    OhTimePoint[] tp = new OhTimePoint[2];
    tp[0] = OhTimePoint.OhTimePointMax;

    // _componentArray
    //
    for (k = 0; k < _numberOfComponents; k++) {
      tp[1] = ((OhElement)_componentArray[k]).getCurrentTime();
      if (tp[1].compareTo(tp[0]) < 0) {
        OhTimePoint tP = new OhTimePoint();
        tP.copy(tp[1]);
        tp[0] = tP;
      }
    }

    // _iteratorArray
    //
    for (k = 0; k < _numberOfIteratorSets; k++) {
      tp[1] = ((OhElement)_iteratorArray[k][0]).getFixedTime();
      if (tp[1].compareTo(tp[0]) < 0) {
        OhTimePoint timeP = new OhTimePoint();
        timeP.copy(tp[1]);
        tp[0] = timeP;
      }
    }

    if (tp[0].compareTo(_currentTime) < 0) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("error" + NEWLINE);
      out.print("\tPut back _currentTime from " + _currentTime + " to " + tp[0] + NEWLINE);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    _currentTime = tp[0];

    return true;
  }
      

  /**
   * <japanese>
   * vZȂvfQ(_iteratorArray[aNum])ɑ΂
   * rewindCurrentTime() s悤߂܂B
   * </japanese>
   * <p>
   * This orders to perform "rewindCurrentTime()" to
   * the element group which repeated calculation did not converge.
   *
   * @param aNum an <code>int</code> value
   */
  public void rewindCurrentTime(int aNum) throws OhError {          //; (*30)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).rewindCurrentTime();
  }

  
  /**
   * <japanese>
   * vZȂvfQ(_iteratorArray[aNum])ɑ΂
   * flushData() s悤߂܂B
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public void flushData(int aNum) throws OhError {                  //; (*31)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).flushData();
  }


  /**
   * <japanese>
   * vZȂvfQ(_iteratorArray[aNum])ɑ΂
   * renewFixedTime() s悤߂܂B
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @exception OhError if an error occurs
   */
  public void renewFixedTime(int aNum) throws OhError {             //; (*32)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).renewFixedTime();
  }


  /**
   * <japanese>
   * vZȂvfQ(_iteratorArray[aNum])ɑ΂
   * renewTargetTime() s悤߂܂B
   * </japanese>
   *
   * @param aNum an <code>int</code> value
   * @param aItrs a <code>long</code> value
   */
  public void renewTargetTime(int aNum, long aItrs) {            //; (*33)
    for (int i = 0; i < _numberOfIterators[aNum]; i++)
      ((OhElement)_iteratorArray[aNum][i]).renewTargetTime(aItrs);
  }


  /**
   * <japanese>
   * vZڕW _targetTime XV܂B
   * ɔvZԊԊuƂ܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void initializeTargetTime() throws OhError {    //; (*34)
    int i = 0;

    // _iteratorArray
    //
    for (i = 0; i < _numberOfIteratorSets; i++)
      renewTargetTime(i, calculateIterationStep(i));

    // _componentArray
    //
    for (i = 0; i < _numberOfComponents; i++)
      ((OhElement)_componentArray[i]).initializeTargetTime();
  }


  /**
   * <japanese>
   * ̃\bh _iteratorArray[aNum]  calculateIterationStep() ̖߂l̂Aŏ̒lԂ܂B
   * </japanese>
   * <p>
   * This returns the minimum value among the return values of "calculateIterationStep()"
   * of "_iteratorArray[aNum]".
   *
   * @param aNum an <code>int</code> value
   * @return a <code>long</code> value
   */
  public long calculateIterationStep(int aNum) throws OhError {    //; (*35)
    long[] t = new long[2];
    t[0] = Long.MAX_VALUE;

    for (int i = 0; i < _numberOfIterators[aNum]; i++) {
      t[1] = ((OhElement)_iteratorArray[aNum][i]).calculateIterationStep();
      if (t[1] < t[0])
        t[0] = t[1];
    }
    return t[0];
  }


  /**
   * <japanese>
   * e\vfɏvZs߂𑗂܂B
   * </japanese>
   * <p>
   * Operation function.
   *
   * @return a <code>boolean</code> value
   */
  public boolean work() throws OhCannotGetDataError, OhError {    //; (*36)
    int i = 0;
    int j = 0;
    int k = 0;
    boolean calc    = false;    // Whether the function was called and it calculated also at once.
                                //; ֐Ă΂ĈxłvZǂB
    boolean flagAll = false;    // "true" when a component calculates at least one.
                                //; \vfivZvf܂ށjPłvZ trueB
    boolean flagItr = false;    // "true" when a repeated calculation element calculates at least one.
                                //; vZvfPłvZ trueB

    int[] itrCounter = new int[_numberOfIteratorSets];

    for (i = 0; i < _numberOfIteratorSets; i++)
      itrCounter[i] = 0;

    // Calculation of components.
    //
    do {
      flagAll = false;

      // _componentArray
      //
      for (k = 0; k < _numberOfComponents; k++) {
        if (((OhElement)_componentArray[k]).work() == true) {    //; Throws "OhCannotGetDataError".
          calc = flagAll = true;
          break;
        }
      }
      while (++k < _numberOfComponents)
        ((OhElement)_componentArray[k]).work();    //; Throws "OhCannotGetDataError".

      // _iteratorArray
      //
      for (i = 0; i < _numberOfIteratorSets; i++) {
        do {
          flagItr = false;
          for (j = 0; j < _numberOfIterators[i]; j++) {
            if (((OhElement)_iteratorArray[i][j]).work() == true) {
              calc = flagAll = flagItr = true;
              break;
            }
          }
          while (++j < _numberOfIterators[i])
            ((OhElement)_iteratorArray[i][j]).work();    //; Throws "OhCannotGetDataError".
        }
        while (flagItr == true);

        if (doYouReachTargetTime(i) == true) {
          if (doYouConverge(i) == true) {
            flushData(i);
            renewFixedTime(i);
            renewTargetTime(i, calculateIterationStep(i));
            itrCounter[i] = 0;
          }
          else {
            rewindCurrentTime(i);
            if (++itrCounter[i] >= 1000) {
              StringWriter buffer = new StringWriter();
              PrintWriter  out    = new PrintWriter(buffer);
              out.print("Iterative calculation cannot converge" + NEWLINE);
              out.print("tin group [" + i + "]." + NEWLINE);
              oPrint(out, " > ");
              String str = buffer.toString();
              out.close();
              throw new OhError(str);
            }
          }
        }
      }
    }
    while (flagAll == true);

    if (calc == false) {
      return false;
    }

    if (canYouRenewCurrentTime() != true) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, " > ");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    return true;
  }
    
} // OhSubSystem
