package ohymosj;

import ohymosj.*;
import java.util.*;
import java.text.*;


/**
 * <japanese>
 * Nƌoߎ[sec]Ƃŕ\V~[V\NXłB
 * </japanese>
 *
 * @author <a href="mailto:shiiba@toewin.kuciv.kyoto-u.ac.jp">shiiba</a>
 * @version $Revision: 1.7 $ $Date: 2004/02/10 16:45:15 $
 */
public class OhTimePoint implements Cloneable, Comparable {
  /**
   * <japanese>
   * NłB
   * </japanese>
   *
   */
  protected int _year;

  /**
   * <japanese>
   * ΏۂƂN̔Nňoߎ[sec]łB
   * </japanese>
   *
   */
  protected long _tsec;
  protected long _oneYearInSec;
  protected static int[] nDay = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
  protected static int[] lDay = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

  /**
   * The index of the element of the calendar array which contains the
   * datum of year. The calendar array is obtained by toCalendarArray()
   */
  public static final int YEAR = 0;
  /**
   * The index of the element of the calendar array which contains the
   * datum of month. The calendar array is obtained by toCalendarArray()
   */
  public static final int MONTH = 1;
  /**
   * The index of the element of the calendar array which contains the
   * datum of day. The calendar array is obtained by toCalendarArray()
   */
  public static final int DAY = 2;
  /**
   * The index of the element of the calendar array which contains the
   * datum of hour. The calendar array is obtained by toCalendarArray()
   */
  public static final int HOUR = 3;
  /**
   * The index of the element of the calendar array which contains the
   * datum of munute. The calendar array is obtained by toCalendarArray()
   */
  public static final int MINUTE = 4;
  /**
   * The index of the element of the calendar array which contains the
   * datum of second. The calendar array is obtained by toCalendarArray()
   */
  public static final int SECOND = 5;
  
  /**
   * Creates a new default <code>OhTimePoint</code> instance.
   * The default time is the beginning of the year 0.
   */
  public OhTimePoint (){
    _year = 0;  _tsec = 0;  _oneYearInSec = 31622400;
  }

  /**
   * Creates a new <code>OhTimePoint</code> instance.
   *
   * @param aYear an <code>int</code> value.
   * @param aTsec a <code>long</code> value. Time from the beginning
   * of the year.
   */
  public OhTimePoint (int aYear, long aTsec) {
    _year = aYear;
    if (_year == 0) { _oneYearInSec = 31622400; } // 366 * 24 * 3600
    else {
      _oneYearInSec = 31536000; // 365 * 24 * 3600
      if (isLeapYear(_year)) _oneYearInSec = 31622400;
    }
    if (aTsec < 0 || aTsec >= _oneYearInSec) {
      throw new IllegalArgumentException("illegal aTsec");
    }
    _tsec = aTsec;
  }

  /**
   * Creates a new <code>OhTimePoint</code> instance.
   *
   * @param aYear an <code>int</code> value. year ex. 2003
   * @param aMonth an <code>int</code> value. month from 1 to 12. 1 for January
   * @param aDay an <code>int</code> value. day 1 for the first day of
   * the month
   * @param aHour an <code>int</code> value. hour from 0 to 23
   * @param aMin an <code>int</code> value. min from 0 to 59
   * @param aSec an <code>int</code> value. second from 0 to 59
   */
  public OhTimePoint (int aYear, int aMonth, int aDay, int aHour, int aMin, int aSec) {
    _year = aYear;
    boolean wLeapYearP = isLeapYear(aYear);
    _oneYearInSec = 31536000; // 365 * 24 * 3600
    if (wLeapYearP) _oneYearInSec += 86400;

    _tsec = 0;
    
    if (aMonth < 1 || aMonth > 12) {
      throw new IllegalArgumentException("illegal aMonth");
    }
    for (int i = 0; i < aMonth -1; i++) {
      _tsec += (wLeapYearP ? lDay[i] : nDay[i]) * 86400;
    }
    if (aDay < 1 ||
      aDay > (wLeapYearP ? lDay[aMonth-1] : nDay[aMonth-1])) {
      throw new IllegalArgumentException("illegal aDay");
    }
    _tsec += (aDay - 1) * 86400;
    if (aHour < 0 || aHour >= 24) {
      throw new IllegalArgumentException("illegal aHour");
    }
    _tsec += aHour * 3600;
    if (aMin < 0 || aMin >= 60) {
      throw new IllegalArgumentException("illegal aMin");
    }
    _tsec += aMin * 60;
    if (aSec < 0 || aSec >= 60) {
      throw new IllegalArgumentException("illegal aSec");
    }
    _tsec += aSec;
  }

  /**
   * Creates a new <code>OhTimePoint</code> instance from another
   * <code>OhTimePoint</code> instance.
   *
   * @param tp an </code>OhTimePoint</code> value
   */
  public OhTimePoint(OhTimePoint tp) {
    _year = tp._year;
    _tsec = tp._tsec;
    _oneYearInSec = tp._oneYearInSec;
  }
  
  /**
   * Returns a deep copy of the current object
   *
   * @return an <code>Object</code> value
   */
  public Object clone() {
    return new OhTimePoint(_year, _tsec);
  }

  /**
   * Copies fields from those of <code>OhTimePoint</code> tp.
   *
   * @param tp an <code>Object</code> value
   */
  public void copy(OhTimePoint tp) {
    _year = tp._year;
    _tsec = tp._tsec;
    _oneYearInSec = tp._oneYearInSec;
  }    

  /**
   * Tests if the specified Object is an instance of OhTimePoint and if
   * it equals this OhTimePoint. 
   *
   * @param arg an <code>Object</code> value
   * @return a <code>boolean</code> value
   */
  public boolean equals(Object arg) {
    return ( (arg != null) && (arg instanceof OhTimePoint) &&
      _year == ((OhTimePoint)arg)._year) &&
      (_tsec == ((OhTimePoint)arg)._tsec);
  }
  
  /**
   * Gets the year
   *
   * @return an <code>int</code> value
   */
  public int getYear() {
    return _year;
  }
  
  /**
   * Gets the time from the beginning of the year of the time expressed
   * by this OhTimePoint object
   *
   * @return a <code>long</code> value
   */
  public long getTsec() {
    return _tsec;
  }

  /**
   * Gets the number of seconds contained in the year of the time expressed
   * by this OhTimePoint object
   *
   * @return a <code>long</code> value
   */
  public long getOneYearInSec() {
    return _oneYearInSec;
  }

  /**
   * Tests if the year 'aYear' is a leap year or not.
   *
   * @param aYear an <code>int</code> value
   * @return a <code>boolean</code> value
   */
  public static boolean isLeapYear(int aYear) {
    if (aYear % 4 != 0) return false;
    if (aYear % 100 != 0) return true;
    if (aYear % 400 != 0) return false;
    return true;
  }

  /**
   * Converts the current OhTimePoint object to the Calendar array.
   * When cal is the returned array, cal[OhTimePoint.YEAR],
   * cal[OhTimePoint.MONTH], cal[OhTimePoint.DAY],
   * cal[OhTimePoint.HOUR], cal[OhTimePoint.MINUTE],
   * cal[OhTimePoint.SECOND] contain year, month, hour, minute and
   * second data.
   *
   * @return an <code>int[]</code> value
   */
  public int[] toCalendarArray() {
    int[] cal = new int[6];
    
    int year = _year;
    int month = 1;
    int day = 1;
    int hour = 0;
    int minute = 0;
    int second = 0;
    int wT = (int)_tsec;
    int yearSec;
    boolean wLeapYearP;
    while (true) {
      wLeapYearP = isLeapYear(year);
      yearSec = 31536000; // 365 * 24 * 3600;
      if (wLeapYearP) { yearSec += 86400; }
      if (wT <= yearSec) break;
      year += 1;
      wT -= yearSec;
    }
    long monthSec;
    while (true) {
      monthSec = (wLeapYearP ? lDay[month] : nDay[month]) * 86400; 
      if (wT <= monthSec) break;
      month += 1;
      wT -= monthSec;
    }
    day = wT / 86400 + 1;
    wT -= (day - 1) * 86400;
    hour = wT / 3600;
    wT -= hour * 3600;
    minute = wT / 60;
    wT -= minute * 60;
    second = wT;

    cal[YEAR] = year; cal[MONTH] = month; cal[DAY] = day;
    cal[HOUR] = hour; cal[MINUTE] = minute; cal[SECOND] = second;

    return cal;
  }

  /**
   * Sets the time of this OhTimePoint object to the beginning of the 
   * year aYear.
   *
   * @param aYear an <code>int</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint setYear(int aYear) {
    if (aYear < -1) {
      throw new IllegalArgumentException("illega aYear");
    }
    _year = aYear;
    _oneYearInSec = 365 * 24 * 3600;
    if (isLeapYear(_year)) _oneYearInSec += 86400;
    _tsec = 0;
    return this;
  }

  /**
   * Set this OhTimePoint time at the time point when 'aTsec' seconds
   * have elapsed from the beginning of the year.
   *
   * @param aTsec a <code>long</code> value
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint setTsec(long aTsec) {
    if (aTsec < 0 || aTsec > _oneYearInSec) {
      throw new IllegalArgumentException("illega aTsec");
    }
    _tsec = aTsec;
    return this;
  }

  /**
   * Compares the current OhTimePoint object to another 
   * object 'arg' and returns the result.
   * The returend value is 1, 0, -1 
   *
   * @param arg an <code>Object</code> value
   * @return an <code>int</code> value.
   *   the value 0 if this OhTimePoint object expresses the same time as
   *   the argument OhTimePoint object; 
   *   the value -1 0 if this OhTimePoint object expresses earlier
   *   time than the
   *   argument OhTimePoint object;
   *   the value 1 if this OhTimePoint object expresses later time than the
   *   argument OhTimePoint object.
   */
  public int compareTo(Object arg) {
    OhTimePoint a = (OhTimePoint)arg;
    if (_year > a._year) return 1;
    if (_year < a._year) return -1;
    if (_tsec > a._tsec) return 1;
    if (_tsec < a._tsec) return -1;
    return 0;
  }

  /**
   * Adds the specified amount of seconds to the current object and
   * returns the updated object.
   *
   * @param timeSec a <code>long</code> value. 
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint add(long timeSec) {
    if (timeSec >= 0) {
      _tsec += timeSec;
      while (_tsec >= _oneYearInSec) {
        _tsec -= _oneYearInSec;
        _year += 1;
        _oneYearInSec = 31536000L; // 365 * 24 * 3600;
        if (isLeapYear(_year)) _oneYearInSec = 31622400L;
      }
    }
    else {
      _tsec += timeSec;
      while (_tsec < 0) {
        _year -= 1;
        _oneYearInSec = 31536000L; // 365 * 24 * 3600;
        if (isLeapYear(_year)) _oneYearInSec = 31622400L;
        _tsec += _oneYearInSec;
      }
    }
    return this;
  }


//////////////////////////////////////////////////////////////////////////
// added by Y.Satoh (2003/10/03)
  /**
   * <japanese>
   * aT Ƃ̍[sec]Ԃ܂B
   * </japanese>
   *
   * @param aT an <code>OhTimePoint</code> value
   * @return a <code>long</code> value
   */
  public long subtract(OhTimePoint aT) {
    long t    = 0L;
    int  year = aT.getYear();
    long tsec = aT.getTsec();
    
    if (_year == year) {
      t = Math.abs(_tsec - tsec);
    }
    else if (_year < year) {
      int y = _year;
      while (y < year) {
        if (isLeapYear(y))
          t = t + 31622400L;
        else
          t = t + 31536000L;
        y++;
      }
      t = t - _tsec + tsec;
    }
    else {
      int y = year;
      while (y < _year) {
        if (isLeapYear(y))
          t = t + 31622400L;
        else
          t = t + 31536000L;
        y++;
      }
      t = t - tsec + _tsec;
    }
    return t;        
  }

  
  /**
   * Returns "" + _year + " " + _tsec.
   *
   * @return a <code>String</code> value
   */
  public String toString() {
    StringBuffer a = new StringBuffer("");
    a.append(_year).append(" ").append(_tsec);
    return a.toString();
  }

  /**
   * Returns the CalendaString. For example, this method returns
   * "20030101010101" for 2003 January 1st, 01:01:01.
   *
   * @return a <code>String</code> value
   */
  public String toCalendarString() {
    int[] cal;
    cal = toCalendarArray();
    StringBuffer a = new StringBuffer("");
    DecimalFormat df = new DecimalFormat("00");
    a.append(cal[0]).
       append(df.format(cal[1])).
       append(df.format(cal[2])).
       append(df.format(cal[3])).
       append(df.format(cal[4])).
       append(df.format(cal[5]));
    return a.toString();
  }

  
//////////////////////////////////////////////////////////////////////////
// added by Y.Satoh (2003/10/03)
  /**
   * <japanese>
   * Iɂȏ̂̎͂ȂƎv鎞łB<p>
   * <ul>
   *  <li> _year : -1
   *  <li> _tsec : 0
   * </ul>
   * <p>
   * Ē`Ă܂B
   *
   */
  public static OhTimePoint OhTimePointMin = new OhTimePoint(-1, 0L);

  /**
   * <japanese>
   * IɂȏŐV̎͂ȂƎv鎞łB<p>
   * <ul>
   *  <li> _year : 10000
   *  <li> _tsec : 0
   * </ul>
   * <p>
   * Ē`Ă܂B
   *
   */
  public static OhTimePoint OhTimePointMax = new OhTimePoint(10000, 0L);

  
}// ohtimepoint
