/**
 * @(#) OhTotalSystem; base class for creating total systems.
 *                     Make "OhTotalSystem.java" from "ohtotal.h" and "ohtotal.cpp" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;

/**
 * <japanese>
 * {^Šnf\NXłB<p>
 *
 * Šnf́At@CóAlbg[NɂóAL[{[h́ACRT o͂ȂǁA
 * XGȃf[^̓o͂ɑΉ܂B
 * Šnf́AGȃf[^n̓Ɏ荞ގ́AS
 * <a href="OhDataPack.html">OhDataPack</a> ^̃f[^ɕϊA
 * V~[VʂV~[Vs҂ɒ񋟂鎞́A
 * <a href="OhDataPack.html">OhDataPack</a> ^Œ~ςĂf[^
 * V~[Vs҂̖]ރf[^`ɕϊ܂B
 * ŁAf[^̕ϊɊւ̓ÍAVɍ쐬͒[qAo͒[qōsȂA
 * Šnfł́A͒[qEo͒[q̎葱ő삷̂Ƃ܂B<p>
 *
 * ܂A<a href="OhDataPack.html">OhDataPack</a> ^ɕϊ̓f[^̑ɁA
 * Šnf́AŠnɊ܂܂Svf̃p[^Eԗʂ̏l
 * V~[Vs҂pӂf[^t@Cǂݎ@\Ă܂B
 * ܂AV~[Vs҂Ƃ̑ΘbɂA
 * V~[VڕW肷@\Ă܂B<p>
 *
 * Šnf́AvfEnɂ\AƂĐvZsȂ킸A
 * nfƓl̍\Ă܂B
 * Ŗ{VXeł́AnfpđŠnf쐬܂B<p>
 *
 * Anfnɑ΂Cvbgf[^𕔕nOMē̂ɑ΂A
 * ŠńA͒[qpăCvbgf[^琶A\vfɑM܂B
 * ܂A\vfAnOɃAEgvbgf[^A
 * nf͒PɊOɑM邾łA
 * Šnf͏o͒[qpāAႦ΁Ao̓t@CɋL^ƂsȂ܂B
 *
 * </japanese>
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.89 $ $Date: 2004/03/08 17:03:02 $
 */
public abstract class OhTotalSystem extends OhSubSystem
{
  // @begin {fields} -----------------------------------------------------

  /**
   * <japanese>
   * V~[VڕWłB
   * </japanese>
   */
  protected OhTimePoint _terminalTime;

  /**
   * <japanese>
   * ԖڕWłB
   * </japanese>
   * <p>
   * Medium term target time.
   *
   */
  protected OhTimePoint _mediumTermTargetTime;


  /**
   * <japanese>
   * ŐṼf[^ԂłB
   * </japanese>
   * <p>
   * Newest data time collection.
   *
   */
  protected String _newestDataTimeCol;

  // @end {fields} -------------------------------------------------------


  // @begin {abstract methods} -------------------------------------------

  // 1. registration of possession ports, possession element objects and possession subsystem objects.

  /**
   * <japanese>
   * ͒[qo^܂B<p>
   *
   * ۃ\bhłB
   * Šnf̃vO}`Kv܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerInputPorts() throws OhError;

  /**
   * <japanese>
   * o͒[qo^܂B<p>
   *
   * ۃ\bhłB
   * Šnf̃vO}`Kv܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerOutputPorts() throws OhError;

  /**
   * <japanese>
   * Šn̍\vfo^钊ۃ\bhłB<p>
   *
   * ̃NXpđŠn쐬vO}́A
   * ̃\bh`Kv܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerComponents() throws OhError;

  /**
   * <japanese>
   * vZvf _iteratorArray ɓo^܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void registerIterators() throws OhError;

  /**
   * <japanese>
   * Šnf̒[q̐ڑ
   * <a href="OhSubSystem.html#connect()">OhSubSystem.connect()</a>
   * pĒ`܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectPorts() throws OhError;

  /**
   * <japanese>
   * Šnf̍\IuWFNg̐ڑsȂ܂B<p>
   *
   * Šnf̒ڒʐMڑ
   * <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
   * pĒ`܂B<p>
   *
   * ۃ\bhȂ̂ŔhNXŕK`ĉB
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public abstract void connectComponents() throws OhError;

  // @end {abstract methods} ---------------------------------------------

  
  // @begin {initialization} ---------------------------------------------

  /**
   * <japanese>
   * IuWFNgAIuWFNgԍAvZsȂvfQ̐
   * ܂B
   * ڒʐMvf 0 ɐݒ肳܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @param aObjNum an <code>int</code> value
   * @param aNumOfItr an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName, int aObjNum, int aNumOfItr) throws OhError {
    return super.init(aObjName, aObjNum, 0, aNumOfItr, null);
  }

  /**
   * <japanese>
   * IuWFNg܂B
   * IuWFNgԍ 0 ɁAڒʐMvfєvZsȂvf 0 ɐݒ肳܂B
   * </japanese>
   *
   * @param aObjName a <code>String</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(String aObjName) throws OhError {
    return super.init(aObjName, 0, 0, 0, null);
  }
  
  /**
   * <japanese>
   * IuWFNgԍAvZsȂvfQ̐
   * ܂B
   * IuWFNg "totalSystem" ɁA
   * ڒʐMvf 0 ɐݒ肳܂B
   * </japanese>
   *
   * @param aObjNum an <code>int</code> value
   * @param aNumOfItr an <code>int</code> value
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init(int aObjNum, int aNumOfItr) throws OhError {
    return super.init("totalSystem", aObjNum, 0, aNumOfItr, null);
  }

  /**
   * <japanese>
   * IuWFNg "totalSystem" ɁAIuWFNgԍ 0 ɁA
   * ڒʐMvfєvZsȂvf 0 ɐݒ肳܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean init() throws OhError {
    return super.init("totalSystem", 0, 0, 0, null);
  }
  
  // @end {initialization} -----------------------------------------------

  // @begin {constructors} -----------------------------------------------

  /**
   * <japanese>
   * ftHgRXgN^łB<p>
   *
   * ftHgRXgN^͏\bh init() s܂B
   * ̌ʈȉ̐ݒƂȂ܂B
   *
   * <pre>
   *  E_className    : -1
   *  E_terminalTime : OhTimePoint.OhTimePointMin
   * </pre>
   *
   * 䂦A
   * IuWFNgAIuWFNgԍȂǂ邽߁A
   * [U`ŠnNX̃RXgN^̂Ȃ
   * Iɏ\bh init() ĂԂ悤ɂĂB
   * </japanese>
   *
   */
  public OhTotalSystem() {
    super();
    _terminalTime      = OhTimePoint.OhTimePointMin;
    _newestDataTimeCol = "";
    _className         = "OhTotalSystem";
  }    

  // @end {constructors} -------------------------------------------------


  // @begin {methods} ----------------------------------------------------

  /**
   * <japanese>
   * _newestDataTimeCol Ԃ܂B
   * </japanese>
   * <p>
   * Returns "_newestDataTimeCol".
   *
   * @return a <code>String</code> value
   */
  public String getNewestDataTimeCol() {
    return _newestDataTimeCol;
  }

  /**
   * <japanese>
   * _newestDataTimeCol  aStr ɐݒ肵܂B
   * </japanese>
   * <p>
   * Returns "_newestDataTimeCol + aStr (arg)".
   *
   * @param aStr a <code>String</code> value
   */
  public void addNewestDataTime(String aStr) {
    _newestDataTimeCol = _newestDataTimeCol + aStr;
  }
  
  // 0. Simulation option.

  /**
   * <japanese>
   * [UƂ̑ΘbɂAOHyMoSJ IvV肷B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void setOhymosjOption() throws OhError {    //; (*1)

    int opt;
    
    StringWriter sw = new StringWriter();
    PrintWriter  pw = new PrintWriter(sw);
    
    pw.print("# Key in simulation option." + NEWLINE);
    pw.print("# TimeControlOption" + NEWLINE);
    
    if (OhOption.getDefault().getTimeControlOption() == TimeControl.GO_BEYOND_TERMINAL_TIME) {
      pw.print("#   current selection: " + "GO_BEYOND_TERMINAL_TIME" + NEWLINE);
    }
    else if (OhOption.getDefault().getTimeControlOption() == TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME) {
      pw.print("#   current selection: " + "DO_NOT_GO_BEYOND_TERMINAL_TIME" + NEWLINE);
    }

    pw.print("#   select 1 or 2" + NEWLINE);
    pw.print("    1. GO_BEYOND_TERMINAL_TIME (Default)" + NEWLINE);
    pw.print("    2. DO_NOT_GO_BEYOND_TERMINAL_TIME" + NEWLINE);

    String message = sw.toString();

    pw.close();
    

    String textLine = OhInterface.getReply(message, "1");

    if (textLine.equals(null))
      throw new OhError("Quit");
    

    StringReader sr = new StringReader(textLine);

    try {
      int c = sr.read();
      opt = Character.getNumericValue((char)c);
      //;   ͂ꂽlȂꍇ -1 ԂB
      //;   ̐l𕉂łȂƂĕ\łȂꍇilȂǁj -2 ԂB
      
      switch (opt) {
        
      case 1:
        OhOption.getDefault().setTimeControlOption(TimeControl.GO_BEYOND_TERMINAL_TIME);
        break;
        
      case 2:
        OhOption.getDefault().setTimeControlOption(TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME);
        break;
        
      default:
        OhOption.getDefault().setTimeControlOption(TimeControl.GO_BEYOND_TERMINAL_TIME);
        break;
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }


  // 1. registration of possession ports, possession element objects and possession subsystem objects.

  /**
   * <japanese>
   * Šn registerSendPorts() ́A
   * Šn̒ۃ\bh
   * <a href="OhTotalSystem.html#registerInputPorts()">OhTotalSystem.registerInputPorts()</a>
   * s悤ɒ`Ă܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerSendPorts() throws OhError {
    registerInputPorts();
  }


  /**
   * <japanese>
   * Šn registerReceivePorts() ́A
   * Šn̒ۃ\bh
   * <a href="OhTotalSystem.html#registerOutputPorts()">OhTotalSystem.registerOutputPorts()</a>
   * s悤ɒ`Ă܂B
   * </japanese>
   *
   * @exception OhError if an error occurs
   */
  public void registerReceivePorts() throws OhError {
    registerOutputPorts();
  }


  // 2. initialization of total systems.

  /**
   * <japanese>
   * f[^쐬܂B
   *
   * <pre>
   * (
   * (0)
   * (1)
   * (2000)
   * (38600)
   * (2.19956e+07)
   * )
   * </pre>
   *
   * ̃tH[}bgŋLqĂf[^t@ĆA
   * f[^ "0 1 2000 38600 2.19956e+07" 擾ł܂B
   *
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @return a <code>String</code> value
   */
  public String makeDataString(OhBufferedReader aBr) {    //; (*Satoh original method)
    String dataString = null;
    String[] data = new String[1024];
    int i = 0;
    int k = 0;
    try {
      while ((data[i] = aBr.readWord()).equals(")") != true) {
        if ((data[i].startsWith("(")) == false) {
          System.err.println("Data file format error.");
          System.exit(1);
        }
        int length = data[i].length();
        data[i] = data[i].substring(1, length - 1);
        if (aBr.skipWhites() == -1) {
          System.err.println("Data file format error.");
          System.exit(1);
        }
        i++;
      }

      StringBuffer sb = new StringBuffer();

      for (k = 0; k < i; k++) {
        sb.append(data[k]);
        sb.append(" ");
      }

      dataString = sb.toString();
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    return dataString;
  }
        

  /**
   * <japanese>
   * evfɕ^ăp^܂͏l̐ݒ܂B<p>
   *
   * f[^t@C͂āAevfɕ^ăp^ݒ肷邩lݒ肵܂B<p>
   *
   * <ul>
   *  <li> aContent == DataFileContent.PARSET ̂ƂAp^ݒ
   *  <li> aContent == DataFileContent.INITIAL ̂ƂAlݒ
   *  <li> aContent == DataFileContent.SDPDATA ̂ƂAM[qɎcĂf[^̍Đݒ
   * </ul>
   * <p>
   *
   * PARSET ȂǂɂẮA
   * f[^t@C̓e\NX
   * <a href="DataFileContent.html">DataFileContent</a>
   * QƂĂB<p>
   *
   * ̃NX
   * <a href="OhTotalSystem.html#readSendPortFile()">OhTotalSystem.readSendPortFile()</a>
   * ŎgpĂ܂B
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @param aContent a <code>DataFileContent</code> value
   * @exception OhError if an error occurs
   */
  public void deliverDataString(String aFileName, DataFileContent aContent) throws OhError {  //; (*3)
    FileReader fr;
    OhBufferedReader br;

    try {
      fr = new FileReader(aFileName);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      if (aContent == DataFileContent.SDPDATA) {
        return;
      }

      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("File open error.", str);
    }

    int  current;
    char currentChar;
    
    int paren = 0;    //; ʁop̐[B
    
    Object element = null;
    Object owner = null;
    
    String dataStr;
    
    String[] objArray = new String[3];    //; objArray[0] = className, objArray[1] = objName.
    int objNum;

    try {
      
    LOOP:
      while (true) {  // ---------------------------- "LOOP" loop starts.
        current = br.skipWhites();
        if (current == -1)
          break LOOP;
        currentChar = (char)current;

        switch (current) {    // ------------------------- switch starts.

        case '{':    //; owner ̕ύXBiPցj
          br.read();    //; ۂ '{' ǂݎB
          paren++;
          if ((owner = element) == null) {
            System.err.println("Trouble about '{' in data file.");
            System.exit(1);
          }
          element = this;
          continue LOOP;

        case '}':    //; owner ̕ύXBiPOցj
          br.read();    //; ۂ '}' ǂݎB
          if ((--paren) < 0) {
            System.err.println("Trouble about '}' in data file.");
            System.exit(1);
          }
          element = owner;
          continue LOOP;

        case '(':    //; element  _dataString ^B
          br.read();    //; ۂ '(' ǂݎB
          if (br.skipWhites() == -1) {
            System.err.println("Trouble about '(' in data file.");
            System.exit(1);
          }
          
          if (current == -1) {
            System.err.println("Trouble about '(' in data file.");
            System.exit(1);
          }

          dataStr = makeDataString(br);    //(*Original)

          if (((OhElement)element).canYouGetDataString(dataStr) != true) {
            System.err.println("Trouble about '(' in data file.");
            System.exit(1);
          }

          if (aContent == DataFileContent.SDPDATA)
            ((OhElement)element).restoreSendPortStock();    //; OhElement (*26)

          ((OhElement)element).clearDataString();

          continue LOOP;

        case ')':
          System.err.println("Trouble about ')' in data file.");
          System.exit(1);

        default:
          objArray[0] = br.readWord();
          objArray[1] = br.readWord();
          objArray[2] = br.readWord();

          if (objArray[1].equals("(") || objArray[1].equals(")") ||
              objArray[1].equals("{") || objArray[1].equals("}") || objArray[1].equals("")) {
            System.err.println("Trouble about ')' in data file.");
            System.exit(1);
          }

          if (objArray[2].equals("(") || objArray[2].equals(")") ||
              objArray[2].equals("{") || objArray[2].equals("}") || objArray[2].equals("")) {
            objNum = 0;
            br.pushBack();    //; '(' ȂǂǂݎȂƂɂăt@C|C^P߂B
          }
          else {
            objNum = Integer.parseInt(objArray[2]);
          }

          if (objCompare(objArray[0], objArray[1], objNum) == true) {
            element = this;
            owner   = this;
          }
          else if ((element = ((OhElement)owner).getComponent(objArray[0], objArray[1], objNum)) == null) {
            System.err.println("Trouble about ')' in data file.");
            System.exit(1);
          }

          continue LOOP;
          
        }    // -------------------------------------------- switch ends.

      }    // ----------------------------------------- "LOOP" loop ends.

      if (paren != 0) {
        System.err.println("Trouble in data file.");
        System.exit(1);
      }

      br.close();
      return;

    }
    catch (OhError e) {
      System.err.println("Trouble in data file.");
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
  }

          
  /**
   * <japanese>
   * p^t@C "aParamFile" p^ǂݍ݂܂B<p>
   *
   * p^́AꊇĂP̃t@Cɏ܂ĂƂ܂B<p>
   *
   * yp^t@C̗z
   * <pre>
   *   # data for test
   *   #
   *   TestOfElement totalSystem 0 
   *   3
   *   {
   *     Mountain ibuki 0 
   *     1.5 2
   *     
   *     Lake biwa 0 
   *     5
   *     {
   *         Sky sky 0 
   *         foo 2
   *     
   *         Outlet outlet 0 
   *         3.5 2
   *         3.1 8.0
   *     }
   *   }
   * </pre>
   *
   * PsQŝ悤ɁA# Ŏn܂s̓RgsłB
   *
   * <pre>
   * TestOfElement totalSystem 0 
   * Mountain ibuki 0 
   * Lake biwa 0 
   * Sky sky 0 
   * Outlet outlet 0
   * </pre>
   *
   * ̂悤ȍśAnfvffʂ邽߂̏łB
   * NXAIuWFNgAIuWFNgԍ̂R̏󔒂ŋ؂ď܂B<p>
   *
   * fʂ邽߂̂̏ɑāA󔒍sԂɒuɁA
   * ̃fǂ݂Ƃׂ܂B
   * ́̕AsetParameter(OhBufferedReader) ǂݍޕłB
   * s̃f[^Kv́Aǂݍ݃vO setParameter(OhBufferedReader) ɂČ܂܂B
   * fʂ邽߂̏sƁAǂ݂Ƃׂf[^̊ԂɁA
   * # Ŏn܂RgsuƂł܂B<p>
   *
   * fʂ邽߂̏sƂɑf[^sł܂Ƃčl΁A
   * ̌ł܂ƕʂ̌ł܂Ƃ̊Ԃɂ́A󔒍s # Ŏn܂sĂ\܂B<p>
   *
   * Lake biwa 0 ̂悤ɁA\vfnf́A{ } ň͂ŁA
   * ̍\vfɊւf[^܂B
   * nłĂA\vf̃p^ݒ̂߂̃f[^ǂݍ݂svȂA
   * { } ̕Kv͂܂B
   *
   * </japanese>
   *
   * @param aParamFile a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void readParameterFile(String aParamFile) throws OhError {    //; (*4)

    FileReader       fr;
    OhBufferedReader br;

    try {
      fr = new FileReader(aParamFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("cannot open \"" + aParamFile + "\"!");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    try {
      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;

      if (br.skipWhites() == -1) {
        OhError e = new OhError("Parameter file error.");
        throw e;
      }
        
      headLine = br.readLine();
        
      String str = headLine;                       //; For work.
      StringTokenizer st = new StringTokenizer(str);
      int i = 0;
      while (st.hasMoreTokens()) {
        st.nextToken();
        i++;
      }

      strToken = new StringTokenizer(headLine);
        
      if (i == 1) {
        objName = strToken.nextToken();

        if (objCompare(objName, 0) == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
          oPrint(out, "   ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError("readParameterFile(): unmatched element", errStr);
        }
        
      }
      else if (i == 2) {
        objName = strToken.nextToken();
        objNum  = Integer.parseInt(strToken.nextToken());
        
        if (objCompare(objName, objNum) == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
          oPrint(out, "   ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError("readParameterFile(): unmatched element", errStr);
        }
        
      }
      else if (i == 3) {
        className = strToken.nextToken();
        objName   = strToken.nextToken();
        objNum    = Integer.parseInt(strToken.nextToken());

        if (objCompare(className, objName, objNum) == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
          oPrint(out, "   ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError("readParameterFile(): unmatched element", errStr);
        }
        
      }
      else {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
        oPrint(out, "      ");
        String errStr= buffer.toString();
        out.close();
        throw new OhError("illegal line", errStr);
      }

      super.getParameter(br);

    }
    catch (OhError e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("readParameterFile() method detected an OhError." + NEWLINE);
      out.print("when reading \"" + aParamFile + "\"");
      String errStr= buffer.toString();
      out.close();
      throw new OhError(errStr, e.what(" > "));
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("readParameterFile() method detected an IOExeption." + NEWLINE);
      out.print("when reading \"" + aParamFile + "\"");
      String errStr= buffer.toString();
      out.close();
      throw new OhError(errStr);
    }
  }


  /**
   * <japanese>
   * <p>
   * ԗʃt@C "aInitStateFile" 珉ԗʂǂݍ݂܂B<p>
   *
   * Ŏw肳ꂽlt@C珉lǂݎ܂B
   * ĺAꊇĂP̃t@Cɏ܂ĂƂ܂B
    </japanese>
   *
   * @param aInitStateFile a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void readInitialStateFile(String aInitStateFile) throws OhError {    //; (*5)
    FileReader       fr;
    OhBufferedReader br;
    
    try {
      fr = new FileReader(aInitStateFile);
      br = new OhBufferedReader(fr, true);
    }
    catch (FileNotFoundException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("cannot open " + aInitStateFile);
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }

    try {
      
      String headLine = null;
      StringTokenizer strToken = null;
      String className = "";
      String objName = "";
      int objNum = 0;
      Object obj = null;

      if (br.skipWhites() == -1) {
        OhError e = new OhError("Initial State file error.");
        throw e;
      }
        
      headLine = br.readLine();
        
        
      String str = headLine;                       //; For work.
      StringTokenizer st = new StringTokenizer(str);
      int i = 0;
      while (st.hasMoreTokens()) {
        st.nextToken();
        i++;
      }
        
      strToken = new StringTokenizer(headLine);
        
      if (i == 1) {
        objName = strToken.nextToken();

        if (objCompare(objName, 0) == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
          oPrint(out, "   ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError("readInitialStateFile(): unmatched element", errStr);
        }
      }
      else if (i == 2) {
        objName = strToken.nextToken();
        objNum  = Integer.parseInt(strToken.nextToken());
        
        if (objCompare(objName, objNum) == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
          oPrint(out, "   ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError("readInitialStateFile(): unmatched element", errStr);
        }
        
      }
      else if (i == 3) {
        className = strToken.nextToken();
        objName   = strToken.nextToken();
        objNum    = Integer.parseInt(strToken.nextToken());

        if (objCompare(className, objName, objNum) == false) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
          oPrint(out, "   ");
          String errStr = buffer.toString();
          out.close();
          throw new OhError("readInitialStateFile(): unmatched element", errStr);
        }
      }
      else {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        out.print("  " + className + "  " + objName + "  " + objNum + NEWLINE);
        oPrint(out, "      ");
        String errStr= buffer.toString();
        out.close();
        throw new OhError("illegal line", errStr);
      }

      super.getInitialState(br);
    }
    catch (OhError e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("readInitialStateFile() method detected an error" + NEWLINE);
      out.print("when reading \"" + aInitStateFile + "\"");
      String errStr= buffer.toString();
      out.close();
      throw new OhError(errStr, e.what(" > "));
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      out.print("readInitialStateFile() method detected an error" + NEWLINE);
      out.print("when reading \"" + aInitStateFile + "\"");
      String errStr= buffer.toString();
      out.close();
      throw new OhError(errStr);
    }
  }


  /**
   * <japanese>
   * t@CɎ^ĂM[q̃f[^ǂݍ݂܂B
   * </japanese>
   * <p>
   * Read the data of the sending port recorded on the file.
   *
   * @param aSendFile a <code>String</code> value
   */
  public void readSendPortFile(String aSendFile) throws OhError {
    deliverDataString(aSendFile, DataFileContent.SDPDATA);
  }


  // Setting of initial state.

  /**
   * <japanese>
   * ̓Xg[ aBr  _currentTime ǂݍ݂܂B
   * t@C _currentTime ǂݎꍇقƂǂƎv܂B
   * KvȂApĂŠnfŕύXĂB
   * </japanese>
   *
   * @param aBr an <code>OhBufferedReader</code> value
   * @exception OhError if an error occurs
   */
  public void setInitialState(OhBufferedReader aBr) throws OhError {      //; (*6)
    int  year = 0;
    long tsec = 0L;
    
    try {
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }
    
    if (year > 0) {
      _currentTime = new OhTimePoint(year, tsec);
    }

    super.isInitialTime(_currentTime);
  }


  /**
   * <japanese>
   * <a href="OhElement.html#canYouShareInfo()">OhElement.canYouShareInfo()</a>
   * s܂B
   * </japanese>
   *
   * @return an <code>OperationFlag</code> value
   * @exception OhError if an error occurs
   */
  public OperationFlag doShareInfo() throws OhError {    //; (*7)
    OperationFlag flag;

    boolean calcany = false;
    boolean calcall = true;

    
    if (_shareInfoDoneP == false) {
      if (canYouShareInfo() == false) {
        StringWriter buffer = new StringWriter();
        PrintWriter  out    = new PrintWriter(buffer);
        String str = buffer.toString();
        out.close();
        throw new OhError(str);
      }
      _shareInfoDoneP = true;
    }

    int i = 0;
    int k = 0;

    do {
      calcany = false;
      calcall = true;
    
      for (k = 0; k < _numberOfComponents; k++) {
        flag = ((OhElement)_componentArray[k]).doShareInfo();
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }
      
      for (i = 0; i < _numberOfIteratorSets; i++) {
        for (k = 0; k < _numberOfIterators[i]; k++) {
          flag = ((OhElement)_iteratorArray[i][k]).doShareInfo();
          if (flag == OperationFlag.E_DONE)
            calcany = true;
          if (flag == OperationFlag.E_UNDONE)
            calcall = false;
        }
      }
    } while (calcany == true);
    

    if (calcall == true)
      return OperationFlag.E_ALREADY;


    for (k = 0; k < _numberOfComponents; k++) {
      if ( ((OhElement)_componentArray[k]).doShareInfo() == OperationFlag.E_UNDONE ) {
        ((OhElement)_componentArray[k]).oPrint(new PrintWriter(System.err, true), "\t");
      }
    }

    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if ( ((OhElement)_iteratorArray[i][k]).doShareInfo() == OperationFlag.E_UNDONE ) {
          ((OhElement)_iteratorArray[i][k]).oPrint(new PrintWriter(System.err, true), "\t");
        }
      }
    }

    return OperationFlag.E_UNDONE;
  }


  // 3. Calculation of the total system model.

  /**
   * <japanese>
   * _terminalTime Ԃ܂B
   * </japanese>
   * <p>
   * Returns "_terminalTime".
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getTerminalTime() {
    return _terminalTime;
  }


  /**
   * <japanese>
   * _mediumTermTargetTime Ԃ܂B
   * </japanese>
   * <p>
   * Returns "_mediumTermTargetTime"
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getMediumTermTargetTime() {
    return _mediumTermTargetTime;
  }


  /**
   * <japanese>
   *  aInputFrom 玞 aInputTo ܂ł̃f[^ł邩ǂ̔f܂B
   * </japanese>
   *
   * @param aInputFrom an <code>OhTimePoint</code> value
   * @param aInputTo an <code>OhTimePoint</code> value
   * @return a <code>boolean</code> value
   */
  public boolean canYouInputDataTime(OhTimePoint aInputFrom, OhTimePoint aInputTo) {    //(*8)
    OhTimePoint inputFrom = OhTimePoint.OhTimePointMin;
    OhTimePoint inputTo   = OhTimePoint.OhTimePointMax;

    OhArrayList dpArray  = null;
    int         numOfDps = 0;

    OhTimePoint buf = null;

    OhArrayList[] dpArrayArray  = {dpArray};
    int[]         numOfDpsArray = {numOfDps};

    for (int k = 0; k < _numberOfSendPorts; k++) {
      ((OhInputPort)_sendPortArray[k]).getDataPackArray(dpArrayArray, numOfDpsArray);
      dpArray  = dpArrayArray[0];
      numOfDps = numOfDpsArray[0];

      if (numOfDps <= 0)
        return false;

      buf = ((OhDataPack)dpArray.get(0)).getDataTime();

      inputFrom = (aInputFrom.compareTo(buf) > 0) ? inputFrom : buf;

      buf = ((OhDataPack)dpArray.get(numOfDps - 1)).getDataTime();

      inputTo = (aInputTo.compareTo(buf) > 0) ? inputTo : buf;
    }

    aInputFrom.copy(inputFrom);
    aInputTo.copy(inputTo);

    return true;
  }


  /**
   * <japanese>
   * ŠnO̓͂sȂ܂B<p>
   *
   * ׂĂ̓͒[qɃf[^ǂݍނ悤Ɏw߂܂B
   * <a href="OhTotalSystem.html#initialWork()">OhTotalSystem.initialWork()</a>,
   * <a href="OhTotalSystem.html#work()">OhTotalSystem.work()</a>
   * Ă΂܂B
   * </japanese>
   * <p>
   * Orders all input ports to read data.
   *
   * @exception OhError if an error occurs
   */
  public void inputData() throws OhError {       //; (*9)
    for (int k = 0; k < _numberOfSendPorts; k++) {
      ((OhInputPort)_sendPortArray[k]).inputData();    //; OhInputFile.inputData() will be called. 
    }
  }

  
  /**
   * <japanese>
   * ŠnOւ̏o͂sȂ܂B<p>
   *
   * ׂĂ̏o͒[qɃf[^o悤Ɏw߂܂B
   * ʂ́At@Cɏo͂܂B
   * <a href="OhTotalSystem.html#work()">OhTotalSystem.work()</a>
   * Ă΂܂B
   * </japanese>
   * <p>
   * Orders all output ports to write out.
   *
   * @exception IOException if an error occurs
   */
  public void outputData() throws IOException {      //(*10)
    for (int k = 0; k < _numberOfReceivePorts; k++) {
      ((OhOutputPort)_receivePortArray[k]).outputData();
    }
  }


  /**
   * <japanese>
   * ̏o͂sȂ܂B<p>
   *
   * ԗʂ߂ɁA_currentTime ł̏o͂`܂B
   * M[qɎƏo͒l݂܂B
   * _currnet_time ł̏o͂vZłꍇ true Ԃ܂B
   * Kvȓ̓f[^ĂȂȂǂāA
   * _currentTime ̏o͂vZłȂꍇ false Ԃ܂B<p>
   *
   * ȂȀꍇ͕n̏̏o͂͂ȂƎv̂ŁA
   * ł͏ true Ԃ悤ɒ`Ă܂B
   * </japanese>
   * <p>
   * Usually there is no output from total system.
   *
   * @return a <code>boolean</code> value
   */
  public boolean initialOutput() throws OhError {
    return true;
  }


  /**
   * <japanese>
   * ƂsȂ܂B<p>
   *
   * ɂāASf[^擾܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhError if an error occurs
   */
  public boolean initialWork() throws OhError {    //(*11)
    
    if (_initialWorkDoneP == false) {
      if (initialOutput() == false) {
        throw new OhError("Cannot do initialOutput() for total system.", "");
      }
      _initialWorkDoneP = true;
    }

    inputData();

    boolean calCall;

    while (true) {

      int i;
      int k;
      OperationFlag flag = null;
      boolean calcany = false;
      boolean calcall = true;
      
      for (k = 0; k < _numberOfComponents; k++) {
        flag = ((OhElement)_componentArray[k]).doInitialWork();  //; OhElement(*35)
        
        if (flag == OperationFlag.E_DONE)
          calcany = true;
        if (flag == OperationFlag.E_UNDONE)
          calcall = false;
      }

      for (i = 0; i < _numberOfIteratorSets; i++) {
        for (k = 0; k < _numberOfIterators[i]; k++) {
          flag = ((OhElement)_iteratorArray[i][k]).doInitialWork();
          
          if (flag == OperationFlag.E_DONE)
            calcany = true;
          if (flag == OperationFlag.E_UNDONE)
            calcall = false;
        }
      }
      
      if (calcany == true)
        continue;

      calCall = calcall;
      break;
    }

    if (calCall == true) {
      initializeTargetTime();    // Initialize "_targetTime".
      return true;
    }

    // Error process.
    //
    int i = 0;
    int k = 0;
    for (k = 0; k < _numberOfComponents; k++) {
      if (((OhElement)_componentArray[k]).doInitialWork() == OperationFlag.E_UNDONE)
        ((OhElement)_componentArray[k]).oPrint(new PrintWriter(System.err, true), "\t");
    }
    
    for (i = 0; i < _numberOfIteratorSets; i++) {
      for (k = 0; k < _numberOfIterators[i]; k++) {
        if (((OhElement)_iteratorArray[i][k]).doInitialWork() == OperationFlag.E_UNDONE) {
          ((OhElement)_iteratorArray[i][k]).oPrint(new PrintWriter(System.err, true), "\t");
        }
      }
    }
    
    System.out.println("OhTotalSystem.initialWork()");
    
    return false;
  }



  /**
   * <japanese>
   * _currentTime Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeFrom() {
    return _currentTime;
  }

  /**
   * <japanese>
   * _terminalTime Ԃ܂B<p>
   *
   * getNecessaryTimeFrom()  getNecessaryTimeTo() ܂ł̃f[^pbN
   * M[q aRp MĂȂ΁Avf͌vZJn邱Ƃł܂B
   * </japanese>
   *
   * @return an <code>OhTimePoint</code> value
   */
  public OhTimePoint getNecessaryTimeTo() {
    return _terminalTime;
  }


  /**
   * <japanese>
   * V~[VڕWV~[Vs҂ƑΘbČ肵܂B
   * </japanese>
   *
   * @return a <code>long</code> value
   * @exception OhError if an error occurs
   */
  public long calculateTimeStep() throws OhError {    //; (*12)
    long timeStep;

    while (true) {    // ------------------------------ while loop starts.
    
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter(sw);
    
      pw.print(_newestDataTimeCol + "# Key in time step or terminal time for \'total system\'.");
      pw.print(NEWLINE);
      pw.print("# current time (year tsec) = " + _currentTime.getYear() + " " + _currentTime.getTsec());
      pw.print(NEWLINE);
      pw.print("# time step (day hour min sec) or terminal time (year tsec)");
      pw.print(NEWLINE);
      

      StringWriter dsw = new StringWriter();
      PrintWriter  dpw = new PrintWriter(dsw);
      
      dpw.print(_currentTime.getYear() + " " + _currentTime.getTsec());

      String message = sw.toString();
      String reply   = dsw.toString();

      pw.close();
      dpw.close();
      
      String textLine = OhInterface.getReply(message, reply);

      if (textLine == null)
        throw new OhError("Quit.");


      String str = textLine;                        // For work.
      StringTokenizer st = new StringTokenizer(str);
      int i = 0;
      while (st.hasMoreTokens()) {
        st.nextToken();
        i++;
      }


      StringTokenizer strToken = new StringTokenizer(textLine);
        
      if (i == 4) {
        int day  = Integer.parseInt(strToken.nextToken());
        int hour = Integer.parseInt(strToken.nextToken());
        long min = Long.parseLong(strToken.nextToken());
        long sec = Long.parseLong(strToken.nextToken());
        timeStep = sec + 60 * (min + 60 * (hour + 24 * day));

        if (timeStep <= 0) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("OHyMoSJ pause:" + NEWLINE);
          out.print("You inputed:   " + textLine + NEWLINE);
          out.print("But this produces non-positive time step");
          String pauseComment = buffer.toString();
          out.close();

          OhInterface.pause(pauseComment);

          continue;
        }

        break;
      }
      else if (i == 2) {
        int  year = Integer.parseInt(strToken.nextToken());
        long tsec = Long.parseLong(strToken.nextToken());
        OhTimePoint tp = new OhTimePoint(year, tsec);
        timeStep = tp.subtract(_currentTime);
     
        if (timeStep <= 0) {
          StringWriter buffer = new StringWriter();
          PrintWriter  out    = new PrintWriter(buffer);
          out.print("OHyMoSJ pause:" + NEWLINE);
          out.print("You inputed:   " + textLine + NEWLINE);
          out.print("But this produces non-positive time step");
          String pauseComment = buffer.toString();
          out.close();

          OhInterface.pause(pauseComment);
          
          continue;
        }
   
        break;
      }
      else {
        System.out.print("# Invalid input" + NEWLINE);
        continue;
      }
      
    }    // ---------------------------------------- while loop ends.
    
    return timeStep;
  }
    

  /**
   * <japanese>
   * PXebv̌vZs܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean calculate() throws OhCannotGetDataError, OhError {
    return true;
  }


  /**
   * <japanese>
   * V~[VvZƂ̈A̗̕WIȂ̂`Ă܂B
   * ܂A
   * o͗p[qpāAŠnɑ΂f[^̓o͂sȂ܂B
   * ܂A\vfɌvZs߂𑗂܂B
   * </japanese>
   *
   * @return a <code>boolean</code> value
   * @exception OhCannotGetDataError if an error occurs
   * @exception OhError if an error occurs
   */
  public boolean work() throws OhCannotGetDataError, OhError {    //; (*13)
    boolean calc    = false;
    boolean flagAll = false;
    boolean flagItr = false;
    int i = 0;
    int j = 0;
    int k = 0;
    char reply;
    int[] itrCounter = new int[_numberOfIteratorSets];
    long mtti = 0L;    // "Medium Term Target Time" increment.

    try {
      
    LOOP:
      while(true) {      // ---------------------------------------------- LOOP starts.
        
        _timeStep = calculateTimeStep();    //; (*12)
        
        while(true) {    // ------------------------------------- Infinite loop starts.

          StringWriter sw = new StringWriter();
          PrintWriter  pw = new PrintWriter(sw);
    
          pw.print("# _timeStep = " + _timeStep + " (sec)");
          pw.print(NEWLINE);
          pw.print("# Key in \"Medium Term Target Time\" increment");
          pw.print(NEWLINE);
          pw.print("#   ((sec) or (min sec) or (hour min sec) or (day hour min sec))");
          pw.print(NEWLINE);
      

          StringWriter dsw = new StringWriter();
          PrintWriter  dpw = new PrintWriter(dsw);
      
          dpw.print(_timeStep);

          String message        = sw.toString();
          String defaultMessage = dsw.toString();

          pw.close();
          dpw.close();
      
          String textLine = OhInterface.getReply(message, defaultMessage);

          mtti = 0;

          if (textLine == null)
            throw new OhError("Quit.");
          
          String str = textLine;                        // For work.
          StringTokenizer st = new StringTokenizer(str);
          int a = 0;
          while (st.hasMoreTokens()) {
            st.nextToken();
            a++;
          }
          
          StringTokenizer strToken = new StringTokenizer(textLine);
        
          if (a == 4) {
            int day  = Integer.parseInt(strToken.nextToken());
            int hour = Integer.parseInt(strToken.nextToken());
            long min = Long.parseLong(strToken.nextToken());
            long sec = Long.parseLong(strToken.nextToken());
            mtti = sec + 60 * (min + 60 * (hour + 24 * day));
          }
          else if (a == 3) {
            int hour = Integer.parseInt(strToken.nextToken());
            long min = Long.parseLong(strToken.nextToken());
            long sec = Long.parseLong(strToken.nextToken());
            mtti = sec + 60 * (min + 60 * hour);
          }
          else if (a == 2) {
            long min = Long.parseLong(strToken.nextToken());
            long sec = Long.parseLong(strToken.nextToken());
            mtti = sec + 60 * min;
          }
          else if (a == 1) {
            mtti = Long.parseLong(strToken.nextToken());
          }
          
          if (mtti > 0)    //; Escapes from an infinite loop.
            break;
          
          System.out.println("# Invalid input : \"Medium Term Target Time\" increment <= 0");
          
        }    // -------------------------------------------------------- Infinite loop ends.


        OhTimePoint crnTp = new OhTimePoint();
        crnTp.copy(_currentTime);
        _terminalTime = crnTp.add(_timeStep);

        OhTimePoint crnT = new OhTimePoint();
        crnT.copy(_currentTime);
        _mediumTermTargetTime = crnT;
        
        Date day = new Date();
        long start;          //; vZJn
        start = day.getTime();

      WORK:
        while(true) {  // ------------------------------------------------------------- WORK starts.
          
          calc = false;
          for (i = 0; i < _numberOfIteratorSets; i++)
            itrCounter[i] = 0;

          _mediumTermTargetTime.add(mtti);
          
          if (_currentTime.compareTo(_mediumTermTargetTime) > 0) {
            if (_currentTime.compareTo(_terminalTime) >= 0)
              break WORK;
            else
              continue WORK;
          }

          do {
            flagAll = false;

            for (k = 0; k < _numberOfComponents; k++) {
              if (((OhElement)_componentArray[k]).work() == true) {  // Abstract method call. "OhCannotGetDataError".
                calc = flagAll = true;
                break;                // Escapes from this for loop.
              }
            }

            while (++k < _numberOfComponents)
              ((OhElement)_componentArray[k]).work();    // Abstract method call. Trows "OhCannotGetDataError".


            // _iteratorArray
            //
            for (i = 0; i < _numberOfIteratorSets; i++) {  // -------------------- for loop 1 starts.
              do {
                flagItr = false;

                for (j = 0; j < _numberOfIterators[i]; j++) {
                  if (((OhElement)_iteratorArray[i][j]).work() == true) { // Abstract method call."OhCannotGetDataError"
                    calc = flagAll = flagItr = true;
                    break;                          // Escapes from this for loop.
                  }
                }

                while (++j < _numberOfIterators[i])
                  ((OhElement)_iteratorArray[i][j]).work();    // Abstract method call. Trows "OhCannotGetDataError".

              } while (flagItr == true);

              if (doYouReachTargetTime(i) == true) {    //; Sub(*27)
                if (doYouConverge(i) == true) {         //; Sub(*28)
                  flushData(i);                         //; Sub(*31)
                  renewFixedTime(i);                    //; Sub(*32)
                  renewTargetTime(i, calculateIterationStep(i));    //; Sub(*33),(*35)
                  itrCounter[0] = 0;
                }
                else {
                  rewindCurrentTime(i);      //; Sub(*30)

                  if (++itrCounter[i] >= 1000) {
                    PrintWriter pw = new PrintWriter(System.err, true);
                    pw.println("\tin group [" + i + "].");
                    oPrint(pw, "\t");
                    pw.close();
                    throw new OhError("");
                  }
                }
              }
            }    // --------------------------------------------------------- for loop 1 ends.
          } while (flagAll == true);

          
          if (calc == false) {
            System.out.print(NEWLINE);
            System.out.print("# No object has worked." + NEWLINE);
            break WORK;
          }

          if (canYouRenewCurrentTime() != true) {
            System.err.println(NEWLINE);
            System.err.println("# Error : OhTotalSystem.canYouRenewCurrentTime()" + NEWLINE);
          }
            
          if (_currentTime.compareTo(_mediumTermTargetTime) < 0) {
            System.err.println(NEWLINE);
            System.err.println("# Error : Stopped before \"MediumTermTargetTime\"." + NEWLINE);
            break WORK;
          }
          
          if (_mediumTermTargetTime.compareTo(_terminalTime) < 0) {
            continue WORK;
          }
          
          outputData();

          break WORK;
          
        }  // --------------------------------------------------------------------------- WORK ends.

        day = new Date();
        long end;       //; vZI
        end = day.getTime();
        long calculationTime;  //; vZv
        calculationTime = end - start;

      ASK:
        while(true) {
          
          StringWriter sw = new StringWriter();
          PrintWriter  pw = new PrintWriter(sw);

          pw.print("# The time which simulation calculation took is " + calculationTime + "[msec].");
          pw.print(NEWLINE);
          pw.print("# Now : _currentTime = " + _currentTime + " : _terminalTime = " + _terminalTime);
          pw.print(NEWLINE);
          pw.print("# Quit? Key in [Y(y)] or [N(n)]");
          pw.print(NEWLINE);

          String message = sw.toString();

          pw.close();
      
          String textLine = OhInterface.getReply(message, "y");

          if (textLine == null)
            throw new OhError("Quit.");

          StringReader     sr = new StringReader(textLine);
          OhBufferedReader br = new OhBufferedReader(sr);

          int c = br.read();    //; 󂫕i""ĵƂ -1 ԂB

          reply = (char)c;

          
          switch(reply) {

          case 'y':
          case 'Y':
            break LOOP;
            
          case 'n':
          case 'N':
            inputData();
            continue LOOP;
            
          default:
            System.out.print("# Invalid input : Key in \'Y\' or \'N\'" + NEWLINE + NEWLINE);
            continue ASK;
          }

        }  // --------------------------------------------------------------- ASK loop ends.

      }  // --------------------------------------------------------------------- LOOP ends.

    }  // ------------------------------------------------------------------------ try ends.
    catch (IOException e) {
      OhInterface.pause("# Error : Now in catch block" + NEWLINE);
    }

    return true;
  }


  // 4. Save function of the last state.

  /**
   * <japanese>
   * _currentTime  o̓Xg[ aPw ɏ݂܂B<p>
   *
   * t@CɏޏꍇقƂǂƎv܂B
   * KvȂApĂŠnfŕύXĂB
   * </japanese>
   * <p>
   * Write "_currentTime".
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void saveTerminalState(PrintWriter aPw) {    //; (*14)
    aPw.print(_currentTime);
  }


  /**
   * <japanese>
   * ŏIԂt@C֕ۑ܂B<p>
   *
   * t@C aFimeName ijVKɃI[vāAŠn̍ŏIԂۑ܂B
   * t@C aFimeName łɑ݂ꍇ͏㏑܂B<p>
   *
   * ŁA
   * <a href="OhSubSystem.html#terminalState(PrintWriter)">OhSubSystem.terminalState(PrintWriter)</a>
   * ǂł܂B
   * </japanese>
   * <p>
   * Saves to the file of the last state.
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void writeTerminalState(String aFileName) throws OhError {    //; (*15)
    FileWriter     fw = null;
    BufferedWriter bw = null;
    PrintWriter    pw = null;
    
    try {
      fw = new FileWriter(aFileName);
      bw = new BufferedWriter(fw);
      pw = new PrintWriter(bw);

      pw.print("# terminal state of " + _objectName  + " " + NEWLINE);
      pw.print("# _currentTime =    " + _currentTime + " " + NEWLINE);

      super.terminalState(pw);    //; OhSubSystem(*24)

      pw.close();
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
  }


  /**
   * <japanese>
   * o͒[qɎcĂf[^t@Cɕۑ܂B<p>
   *
   * t@C aFileName ijVKɃI[vāA
   * o͒[qɎcĂf[^t@Cɕۑ܂B
   * t@C aFilefName łɑ݂ꍇ͏㏑܂B<p>
   *
   * ŁA
   * <a href="OhElement.html#saveSendPortStock(PrintWriter)">OhElement.saveSendPortStock(PrintWriter)</a>,
   * <a href="OhSubSystem.html#saveSendPortStock(PrintWriter)">OhSubSystem.saveSendPortStock(PrintWriter)</a>
   * Ăł܂B
   * </japanese>
   *
   * @param aFileName a <code>String</code> value
   * @exception OhError if an error occurs
   */
  public void writeSendPortStock(String aFileName) throws OhError {    //; (*16)
    FileWriter     fw = null;
    BufferedWriter bw = null;
    PrintWriter    pw = null;
  
    try {
      fw = new FileWriter(aFileName);
      bw = new BufferedWriter(fw);
      pw = new PrintWriter(bw);

      pw.print("# send port data of  " + _objectName + " " + NEWLINE);
      pw.print("# _currentTime =     " + _currentTime + " " + NEWLINE + "# " + NEWLINE);

      ((OhElement)this).saveSendPortStock(pw);

      int i = 0;
      int k = 0;

      if ((_numberOfComponents > 0) || (_numberOfIteratorSets > 0)) {
        pw.print("{" + NEWLINE);

        for (k = 0; k < _numberOfComponents; k++) {
          ((OhElement)_componentArray[k]).saveSendPortStock(pw);
        }

        for (i = 0; i < _numberOfIteratorSets; i++) {
          for (k = 0; k < _numberOfIterators[i]; k++) {
            ((OhElement)_iteratorArray[i][k]).saveSendPortStock(pw);
          }
        }

        pw.print("}" + NEWLINE);
      }

      pw.close();
      
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError(str);
    }
  }
    
      
  // 5. Information display function.

  /**
   * <japanese>
   * ͒[qo͂܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   *  EaHead -> õ͎wb_B
   * </pre>
   * </japanese>
   * <p>
   * The input of output port information.
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printInputs(PrintWriter aPw, String aHead) {
    printObjects(aPw, aHead, "Inputs :", _sendPortArray, _numberOfSendPorts);
  }

  /**
   * <japanese>
   * ͒[qo͂܂B
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   * </pre>
   *
   * õ͎wb_͂SJɂȂ܂B
   * </japanese>
   * <p>
   * The input of output port information.
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void printInputs(PrintWriter aPw) {
    printObjects(aPw, "    ", "Inputs :", _sendPortArray, _numberOfSendPorts);
  }





  /**
   * The output of output port information.
   *
   * <japanese>
   * <p>
   * o͒[q̏o
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   *  EaHead -> õ͎wb_B
   * </pre>
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   * @param aHead a <code>String</code> value
   */
  public void printOutputs(PrintWriter aPw, String aHead) {
    printObjects(aPw, aHead, "Outputs :", _receivePortArray, _numberOfReceivePorts);
  }

  /**
   * <japanese>
   * <p>
   * o͒[q̏o
   *
   * <pre>
   * [ARGUMENT]
   *  EaPw ---> o͐̃t@CXg[B
   * </pre>
   *
   * õ͎wb_͂SJɂȂ܂B
   * </japanese>
   *
   * @param aPw a <code>PrintWriter</code> value
   */
  public void printOutputs(PrintWriter aPw) {
    printObjects(aPw, "    ", "Outputs :", _receivePortArray, _numberOfReceivePorts);
  }

} // OhTotalSystem
