/**
 * @(#) OperationFlag; class to express OperationFlag.
 *                     Make "OperationFlag.java" from "ohelement.h" by Satoh.
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * <japanese>
 * ƂɊւtO\NXłB<p>
 *
 * ^ɈSȒ萔̃NXłB<p>
 *
 * <ul>
 *  <li> E_UNDONE :
 *   <dd>  Ƃ邱ƂłȂB
 *  <li> E_DONE :
 *   <dd>  ƂsB
 *  <li> E_ALREADY :
 *   <dd>  Ƃ͂łɎsς݂łB
 * <li>
 *
 * R̌^́A
 * <a href="OhElement.html#doShareInfo()">OhElement.doShareInfo()</a>,
 * <a href="OhElement.html#doInitialWork()">OhElement.doInitialWork()</a> ȂǂŗpĂ܂B
 *
 * </japanese>
 * <p>
 * OperationFlag;
 *
 * OperationFlag class. <p>
 *
 * <ul>
 *  <li> E_UNDONE  : Initial work could not be done.
 *  <li> E_DONE    : Initial work was completed.
 *  <li> E_ALREADY : Initial work had been done already.
 * <li>
 *
 * This type is used in <a href="OhElement.html#doShareInfo()">OhElement.doShareInfo()</a>,
 * <a href="OhElement.html#doInitialWork()">OhElement.doInitialWork()</a>.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.10 $ $Date: 2004/02/10 14:25:47 $
 */
public final class OperationFlag implements Serializable
{
  // @begin {instance variable or static variable} -----------------------

  private static int nextOrdinal = 0;
  private final  int ordinal     = nextOrdinal++;

  /**
   * <japanese>
   * Ƃ邱ƂłȂƂ\^ɈSȒ萔łB
   * </japanese>
   * <p>
   * E_UNDONE: <br>
   * Initial work could not be done. <p>
   *
   */
  public static final OperationFlag E_UNDONE  = new OperationFlag();

  /**
   * <japanese>
   * ƂsƂ\^ɈSȒ萔łB
   * </japanese>
   * <p>
   * E_DONE: <p>
   * Initial work was completed. <p>
   *
   */
  public static final OperationFlag E_DONE    = new OperationFlag();

  /**
   * <japanese>
   * Ƃ͂łɎsς݂łƂ\^ɈSȒ萔łB
   * </japanese>
   * <p>
   * E_ALREADY: <p>
   * Initial work had been done already.
   *
   */
  public static final OperationFlag E_ALREADY = new OperationFlag();

  private static final OperationFlag[] PRIVATE_VALUES = {E_UNDONE, E_DONE, E_ALREADY};
  
  // @end {instance variable or static variable} -------------------------
  
  
  // @begin {constructors} -----------------------------------------------
  
  private OperationFlag() {}

  // @end {constructors} -------------------------------------------------

  
  // @begin {methods} ----------------------------------------------------

  private Object readResolve() throws ObjectStreamException {
    return PRIVATE_VALUES[ordinal];
  }

  // @end {methods} ------------------------------------------------------

} // OperationFlag
