/**
 * @(#)  TestSimpleDam.java
 *
 */

package ohymosj;

import ohymosj.*;
import java.util.*;
import java.io.*;


/**
 * TestSimpleDam.
 *
 * Test the simple dam model.
 *
 * @author <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
 * @version $Revision: 1.9 $ $Date: 2004/03/08 17:03:02 $
 */
public class TestSimpleDam extends OhTotalSystem {

  // @begin {instance variable or static variable} -----------------------

  // 1. Input port.

  protected OhInputFileDouble _inFlow0;
  protected OhInputFileDouble _inFlow1;

  // 2. Output port.

  protected OhOutputFileDouble _outQ;
  protected OhOutputFileDouble _outStorage;

  // 3. Components.

  protected SimpleDam _dam;


  // @end {instance variable or static variable} -------------------------

  // @begin {constructors} -----------------------------------------------

  public TestSimpleDam() throws OhError {
    super();

    _inFlow0 = new OhInputFileDouble("inflow.i0");
    _inFlow1 = new OhInputFileDouble("inflow.i1");

    _outQ       = new OhOutputFileDouble("tsimpledam.oq");
    _outStorage = new OhOutputFileDouble("tsimpledam.os");

    _dam = new SimpleDam("dam", 0, null, 2);    //; "2" is the number of receive ports.

    _outQ.setCommentString("outflow discharge of the dam");
    _outStorage.setCommentString("storage of the dam");

    _className = "TestSimpleDam";

    init("totalSystem", 0, 0);
  }
  
  // @end {constructors} -------------------------------------------------

  // @begin {methods} ----------------------------------------------------

  // 1. Input port.

  public void registerInputPorts() throws OhError {
    register((Object)_inFlow0);
    register((Object)_inFlow1);
  }

  // 2. Output port.

  public void registerOutputPorts() throws OhError {
    register((Object)_outQ);
    register((Object)_outStorage);
  }
  
  // 3. Components.

  public void registerComponents() throws OhError {
    register((Object)_dam);
  }

  public void registerIterators() {}

  
  // 4. Connection among components.

  public void connectPorts() throws OhError {
    //; Inflow of two tributaries into dam.
    //;
    connect(((Object)_inFlow0), ((Object)_dam), "rp", 0);
    connect(((Object)_inFlow1), ((Object)_dam), "rp", 1);
    
    //; Output.
    //;
    connect(((Object)_dam), "sp", 0, ((Object)_outQ));
    connect(((Object)_dam), "sp", 1, ((Object)_outStorage));
  }

  
  public void connectComponents() {}


  // 5. Initial Status setting.

  public void setInitialState(OhBufferedReader aBr) throws OhError {
    int  year = 0;
    long tsec = 0L;
    String ignoreStr;
    
    try {
      year = Integer.parseInt(aBr.readWord());
      tsec = Long.parseLong(aBr.readWord());

      _currentTime = new OhTimePoint(year, tsec);
    }
    catch (IOException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }
    catch (NumberFormatException e) {
      StringWriter buffer = new StringWriter();
      PrintWriter  out    = new PrintWriter(buffer);
      oPrint(out, "\t");
      String str = buffer.toString();
      out.close();
      throw new OhError("Error in setInitialState().", str);
    }
  }

  // @begin {methods} ----------------------------------------------------

} // TestSimpleDam

