// @(#) class OhBufferedReaderTest

import junit.framework.*;
import java.io.*;
import ohymosj.*;

/**
 * OhBufferedReaderTest.java
 *
 *
 * Created: Sat May 24 09:10:26 2003
 *
 * @author <a href="mailto:shiiba@tonewin.kuciv.kyoto-u.ac.jp">shiiba</a>
 * @version $Revision: 1.1 $ $Date: 2003/12/07 15:03:03 $
 *
 * <japanese>
 * OhBufferedReader ́ABufferedReader Ƀ\bh
 *
 *   public String readWord()
 *   public int skipWhites()
 *   public void pushBack(char c)
 *
 * ǉ܂BreadWord() ́A̒Po܂B
 * </japanese>
 */

public class OhBufferedReaderTest extends TestCase {
  String nl = System.getProperty("line.separator");
  final String sampleStr = "This \tis the first line." + nl
    + "This is the second line." + nl
    + "# This is the third line." + nl
    + "This is the fourth line." + nl;
  
  StringReader sr;
  OhBufferedReader _sample;

  public OhBufferedReaderTest (String name){
    super(name);
  }

  protected void setUp() throws IOException {
    //;
    //;  _sample ̓e OhBufferedReader ɂăt@C
    //; ̂悤ɓǂނƂɂ܂B
    //;
    sr = new StringReader(sampleStr);
    _sample = new OhBufferedReader(sr, true);
  }

  protected void tearDown() {
    try {
      _sample.close();
      _sample = null;
      sr.close();
      sr = null;
    }
    catch (IOException e) {
    }
  }

  public void testGetTreatCommentP() {
    boolean expected = true;
    boolean reality = _sample.getTreatCommentP();
    assertEquals("testGetTreatCommentP", expected, reality);
  }

  public void testTreatComment() throws IOException {
    String nl = System.getProperty("line.separator");
    String sampleStr = "foo" + nl
      + "bar # This is the third line." + nl
      + "aho" + nl;
    StringReader sr = new StringReader(sampleStr);
    OhBufferedReader mbr = new
      OhBufferedReader(sr, /* treatCommentP */ true);

    String word;
    word = mbr.readWord();
    assertEquals("treatCommentMode 1", "foo", word);
    word = mbr.readWord();
    word = mbr.readWord();
    assertEquals("treatCommentMode 2", "aho", word);
    mbr.close();
    sr.close();
  }

  public void testTreatCommentAdd() throws IOException {
    String nl = System.getProperty("line.separator");
    String sampleStr =
      "foo \"ab " + nl +
      "bar " + nl;
    StringReader sr = new StringReader(sampleStr);
    OhBufferedReader mbr = null;
    mbr = new OhBufferedReader(sr, true);
    try {
      String word;
      word = mbr.readWord();
      assertEquals("treatCommentAdd 1", (Object)"foo", word);
    }
    catch (IOException e) {
      // System.out.println(e.getMessage());
      return;
    }
    finally {
      // System.out.println("finally was visitted.");
      mbr.close();
    }
    fail("should catch IO error");
  }

  public void testReadLine() throws IOException {
    String line = _sample.readLine();
    assertEquals("testReadLine 1", "This \tis the first line.", line);
    line = _sample.readLine();
    assertEquals("testReadLine 2", "This is the second line.", line);
    //; RgsXLbvĂ邩eXg
    line = _sample.readLine();
    assertEquals("testReadLine 3", "This is the fourth line.", line);
    line = _sample.readLine();
    assertNull("testReadLine 4", line);
  }

  public void testRead() throws IOException {
    String nl = System.getProperty("line.separator");
    String sampleStr = "ab" + nl
      + "# This is the third line." + nl
      + "c" + nl
      ;
    StringReader sr = new StringReader(sampleStr);
    OhBufferedReader mbr = new OhBufferedReader(sr, true);
      
    int ic;
    ic = mbr.read();
    assertEquals("testRead 1", 'a', (char)ic);
    ic = mbr.read();
    assertEquals("testRead 2", 'b', (char)ic);
    ic = mbr.read();
    assertEquals("testRead 3", '\n', (char)ic);
    ic = mbr.read();
    assertEquals("testRead 4", 'c', (char)ic);
  }

  /**
   * <japanese>
   * readWord() ́AXg[̒P擾ĕԂ܂B
   * sɂPǂ񂾌Aǂݎ|CǵAsɂ܂B
   * PƂ́A󔒕ŋ؂ꂽA̔󔒕̗łB
   * </japanese>
   *
   * @exception IOException if an error occurs
   */
  public void testReadWord() throws IOException {
    String nl = System.getProperty("line.separator");
    String sampleStr = "foo" + nl
      + "" + nl // blank line
      + "bar\"ohymos is good.\" " + nl;
    StringReader sr = new StringReader(sampleStr);
    OhBufferedReader mbr = new OhBufferedReader(sr);
      
    String word;
    word = mbr.readWord();
    assertEquals("testReadWord 1", (Object)"foo", word);
    word = mbr.readWord();
    assertEquals("testReadWord 2", (Object)"", word);
    word = mbr.readWord();
    assertEquals("testReadWord 3", (Object)"bar", word);
    word = mbr.readWord();
    assertEquals("testReadWord 4",
      (Object)"\"ohymos is good.\"", word);
  }

  /**
   * <japanese>
   * skipWhites ́AXg[̋󔒕 (sR[h) XLb
   * vA̓ǂݎʒu𔭌󔒕̏ɒuA
   * ̔󔒕̕R[hԂ܂B󔒕 skip Ă
   * 邤ɃXg[̏I[̓BĂ܂A-1 Ԃ܂B
   * </japanese>
   *
   * @exception IOException if an error occurs
   */
  public void testSkipWhites() throws IOException {
    String nl = System.getProperty("line.separator");
    String sampleStr = "foo " + nl
      + "  /bar" + nl;
    StringReader sr = new StringReader(sampleStr);
    OhBufferedReader mbr = new OhBufferedReader(sr);

    String word;
    int ic;
    word = mbr.readWord();
    assertEquals("testSkipWhites 1", "foo", word);
    ic = mbr.skipWhites();
    assertEquals("testSkipWhites 2", '/', (char)ic);
    ic = mbr.read();
    assertEquals("testSkipWhites 3", '/', (char)ic);
    word = mbr.readWord();
    ic = mbr.skipWhites();
    assertEquals("testSkipWhites 4", -1, ic);
  }
  
  /**
   * <japanese>
   * pushBack() ́AŌɓǂݍ1oCgXg[ɕԂ܂B
   * szĂ܂A߂܂B
   * </japanese>
   *
   * @param c a <code>char</code> value
   */
  public void testPushBack() throws IOException {
    String nl = System.getProperty("line.separator");
    String sampleStr = "foo (bar)" + nl;
    StringReader sr = new StringReader(sampleStr);
    OhBufferedReader mbr = new OhBufferedReader(sr);

    String word;
    word = mbr.readWord();
    mbr.skipWhites();
    int ic = mbr.read();
    assertEquals("testPushBack 1", '(', (char)ic);
    mbr.pushBack();
    word = mbr.readWord();
    assertEquals("testPushBack 2", "(bar)", word);
  }

}// OhBufferedReaderTest

